Package: SuperSurv
Type: Package
Title: A Unified Framework for Machine Learning Ensembles in Survival
        Analysis
Version: 0.1.7
Authors@R: 
    person("Yue", "Lyu", email = "yuelyu0521@gmail.com", role = c("aut", "cre"))
Author: Yue Lyu [aut, cre]
Maintainer: Yue Lyu <yuelyu0521@gmail.com>
Description: Implements a Super Learner framework for right-censored survival data. 
    The package fits convex combinations of parametric, semiparametric, and machine 
    learning survival learners by minimizing cross-validated risk using inverse 
    probability of censoring weighting (IPCW). It provides tools for automated 
    hyperparameter grid search, high-dimensional variable screening, and evaluation 
    of prediction performance using metrics such as the Brier score, Uno's C-index, 
    and time-dependent area under the curve (AUC). Additional utilities support 
    model interpretation for survival ensembles, including Shapley additive 
    explanations (SHAP), and estimation of covariate-adjusted restricted mean 
    survival time (RMST) contrasts. The methodology is related to treatment-specific 
    survival curve estimation using machine learning described by Westling et al.
    (2024) <doi:10.1080/01621459.2023.2205060>, and the unified 
    ensemble framework described in Lyu et al. (2026) <doi:10.64898/2026.03.11.711010>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.0
Depends: R (>= 4.0.0)
LazyData: true
Imports: survival, nnls, future.apply, stats, utils, dplyr, magrittr
URL: https://github.com/yuelyu21/SuperSurv,
        https://yuelyu21.github.io/SuperSurv/
BugReports: https://github.com/yuelyu21/SuperSurv/issues
Suggests: aorsf, BART, CoxBoost, glmnet, gbm, mgcv, randomForestSRC,
        ranger, rpart, survivalsvm, xgboost, survex, ggplot2, tidyr,
        quadprog, ggforce, patchwork, knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-11 05:13:58 UTC; a98455
Repository: CRAN
Date/Publication: 2026-06-11 07:00:18 UTC
Built: R 4.5.2; ; 2026-06-11 10:56:23 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(censor_weights,SuperSurv)
S3method(coef,SuperSurv)
S3method(eval_times,SuperSurv)
S3method(event_weights,SuperSurv)
S3method(learner_names,SuperSurv)
S3method(predict,SuperSurv)
S3method(predict,surv.aorsf)
S3method(predict,surv.bart)
S3method(predict,surv.coxboost)
S3method(predict,surv.coxph)
S3method(predict,surv.gam)
S3method(predict,surv.gbm)
S3method(predict,surv.glmnet)
S3method(predict,surv.km)
S3method(predict,surv.parametric)
S3method(predict,surv.ranger)
S3method(predict,surv.rfsrc)
S3method(predict,surv.ridge)
S3method(predict,surv.rpart)
S3method(predict,surv.svm)
S3method(predict,surv.xgboost)
S3method(print,SuperSurv)
S3method(print,summary.SuperSurv)
S3method(selected_variables,SuperSurv)
S3method(summary,SuperSurv)
S3method(training_variables,SuperSurv)
export(SuperSurv)
export(censor_weights)
export(create_grid)
export(estimate_marginal_rmst)
export(eval_brier)
export(eval_cindex)
export(eval_summary)
export(eval_timeROC)
export(eval_times)
export(event_weights)
export(explain_kernel)
export(explain_survex)
export(learner_names)
export(list_wrappers)
export(plot_beeswarm)
export(plot_benchmark)
export(plot_calibration)
export(plot_dependence)
export(plot_global_importance)
export(plot_marginal_rmst_curve)
export(plot_patient_waterfall)
export(plot_predict)
export(plot_rmst_vs_obs)
export(plot_survival_heatmap)
export(screen.all)
export(screen.elasticnet)
export(screen.glmnet)
export(screen.marg)
export(screen.rfsrc)
export(screen.var)
export(selected_variables)
export(surv.aorsf)
export(surv.bart)
export(surv.coxboost)
export(surv.coxph)
export(surv.exponential)
export(surv.gam)
export(surv.gbm)
export(surv.glmnet)
export(surv.km)
export(surv.loglogistic)
export(surv.lognormal)
export(surv.parametric)
export(surv.ranger)
export(surv.rfsrc)
export(surv.ridge)
export(surv.rpart)
export(surv.svm)
export(surv.weibull)
export(surv.xgboost)
export(training_variables)
importFrom(dplyr,desc)
importFrom(magrittr,"%>%")
importFrom(nnls,nnls)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,var)
importFrom(survival,Surv)

# SuperSurv: A Unified Framework for Machine Learning Ensembles in Survival Analysis <img src="man/figures/logo.png" align="right" width="140"/>

[![R-CMD-check](https://github.com/yuelyu21/SuperSurv/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/yuelyu21/SuperSurv/actions/workflows/R-CMD-check.yaml)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)

**SuperSurv** is an R package for building, evaluating, and interpreting ensemble models for right-censored survival data.

At its core, the package implements a Super Learner-style ensemble framework for continuous-time survival prediction under right censoring. Using inverse probability of censoring weighting (IPCW), it combines heterogeneous base learners by minimizing cross-validated prediction risk. The framework supports learners that return full survival curves as well as learners that return only risk scores, which are calibrated to a common survival-probability scale on a shared evaluation time grid.

Beyond ensemble fitting, **SuperSurv** provides tools for:

- hyperparameter tuning
- high-dimensional screening
- time-dependent model benchmarking
- SHAP-based interpretability
- covariate-adjusted restricted mean survival time (RMST) contrasts

The package also provides a more user-friendly model interface through `print()`, `summary()`, `coef()`, and exported accessors such as `event_weights()`, `censor_weights()`, `learner_names()`, `training_variables()`, `selected_variables()`, and `eval_times()`.

---

## 🚀 Get Started

The best place to start is the installation and setup tutorial:

👉 **[Tutorial 0: Installation & Setup](https://yuelyu21.github.io/SuperSurv/articles/installation.html)**

You can also browse the full documentation site here:

👉 **[SuperSurv website](https://yuelyu21.github.io/SuperSurv/)**

---

## 📦 Installation

Install the CRAN release:

```r
install.packages("SuperSurv")
```

Install the development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("yuelyu21/SuperSurv")
```

---

## 📦 Dependency philosophy

To keep installation lightweight, several heavier machine-learning engines are listed in `Suggests` rather than imported as strict dependencies. This means users can install only the modeling backends they plan to use. If a requested learner is unavailable, **SuperSurv** will prompt the user to install the required package.

---

## 📚 Included learners and screeners

**SuperSurv** currently standardizes a broad set of prediction wrappers and screening methods within a unified interface.

### Prediction learners

- **Machine learning:** random forests, gradient boosting, XGBoost, support vector machines, BART, ranger
- **Penalized/high-dimensional:** elastic net, ridge regression, CoxBoost
- **Tree-based:** RPART
- **Classical/parametric:** Cox proportional hazards, Weibull, exponential, log-logistic, log-normal, generic parametric models
- **Smoothing/splines:** generalized additive models
- **Baseline models:** Kaplan–Meier

### Screening methods

- keep all features
- marginal Cox screening
- variance-based screening
- elastic-net-based screening
- random forest variable hunting

The framework is extensible, and users can add custom learners and screeners. See the extensibility vignette for details.

---

## 📖 Documentation

The package website includes tutorials covering:

- **00. Installation & Setup**
- **01. The SuperSurv Ensemble**
- **02. Model Performance**
- **03. Selection vs. Ensemble**
- **04. Screening Methods**
- **05. Hyperparameter Tuning**
- **06. Random Forests**
- **07. Parametric Models**
- **08. SHAP Interpretability**
- **09. Causal Inference (RMST)**
- **10. Extending SuperSurv**

---

## 📖 Citation

To cite the package, use:

```r
citation("SuperSurv")
```

If you would also like to cite the accompanying preprint:

Lyu, Y., Lin, S. H., Huang, X., & Li, Z. (2026).  
*SuperSurv: A Unified Framework for Machine Learning Ensembles in Survival Analysis*.  
bioRxiv.  
https://doi.org/10.64898/2026.03.11.711010

Related methodological work:

Westling, T., Luedtke, A., Gilbert, P. B., & Carone, M. (2024).  
*Inference for treatment-specific survival curves using machine learning*.  
Journal of the American Statistical Association.  
https://doi.org/10.1080/01621459.2023.2205060

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----install-core, eval=FALSE-------------------------------------------------
# # Install devtools if you don't have it
# if (!requireNamespace("devtools", quietly = TRUE)) {
#   install.packages("devtools")
# }
# 
# # Install SuperSurv from GitHub
# devtools::install_github("yuelyu21/SuperSurv")

## ----load-package, eval=FALSE-------------------------------------------------
# library(SuperSurv)
# 
# # See all 19 prediction models and 6 screening algorithms!
# list_wrappers()

## ----install-suggests, eval=FALSE---------------------------------------------
# # List of highly recommended modeling engines
# ml_packages <- c(
#   "survival",        # Classical Cox models
#   "randomForestSRC", # Random Survival Forests
#   "ranger",          # Fast Random Forests
#   "xgboost",         # Extreme Gradient Boosting
#   "glmnet",          # Elastic Net & Penalized Regression
#   "rpart",           # Decision Trees
#   "survex"           # Time-Dependent XAI (Interpretability)
# )
# 
# # Identify which ones you are missing
# missing_pkgs <- ml_packages[!(ml_packages %in% installed.packages()[,"Package"])]
# 
# # Install the missing ones
# if(length(missing_pkgs)) install.packages(missing_pkgs)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----eval=FALSE---------------------------------------------------------------
# install.packages(c("future", "future.apply"))

## ----parallel-setup, message=FALSE, warning=FALSE, eval=FALSE-----------------
# library(SuperSurv)
# library(future)
# library(survival)
# 
# data("metabric", package = "SuperSurv")
# 
# # 1. Define the parallel plan
# # 'multisession' opens background R sessions.
# # We tell it to use 4 CPU cores (workers).
# plan(multisession, workers = 4)

## ----run-parallel, eval=FALSE-------------------------------------------------
# X <- metabric[, grep("^x", names(metabric))]
# new.times <- seq(50, 200, by = 25)
# 
# # 2. Run the model with parallel = TRUE
# fit_parallel <- SuperSurv(
#   time = metabric$duration,
#   event = metabric$event,
#   X = X,
#   newX = X,
#   new.times = new.times,
#   event.library = c("surv.coxph", "surv.weibull", "surv.rfsrc"),
#   cens.library = c("surv.coxph"),
#   parallel = TRUE,     # <--- The magic argument
#   nFolds = 5
# )

## ----close-parallel, eval=FALSE-----------------------------------------------
# # 3. Return to sequential processing
# plan(sequential)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_rpart <- requireNamespace("rpart", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

# Our max follow-up is well beyond 200, so this grid is safe.
new.times <- seq(50, 200, by = 25) 

## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
my_library <- c("surv.coxph", "surv.weibull")
if (has_rpart) {
  my_library <- c(my_library, "surv.rpart")
}

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)

## ----evaluate-performance-----------------------------------------------------
# Evaluate performance directly using the fitted model and test data
performance_results <- eval_summary(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## ----plot-benchmark, fig.align='center', fig.width=9, fig.height=3------------
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## ----plot-calibration, fig.align='center'-------------------------------------
# Assess calibration specifically at Time = 150
plot_calibration(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_time = 150,
  bins = 5 # Group patients into 5 risk quintiles
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

new.times <- seq(50, 200, by = 25) 

## ----message=FALSE, warning=FALSE---------------------------------------------

# Define our hyperparameter search space
rf_tuning_params <- list(
  nodesize = c(3, 15, 30), # Deep vs. Shallow trees
  ntree = c(100, 500)      # Number of trees in the forest
)

## ----create-grid--------------------------------------------------------------
# Automatically generate 6 different Random Forest wrappers
rf_grid <- create_grid(
  base_learner = "surv.rfsrc", 
  grid_params = rf_tuning_params
)

# View the dynamically generated function names
print(rf_grid)

## ----fit-tuned-model, eval=FALSE----------------------------------------------
# # Combine baseline with our 6 tuned forests
# my_ml_library <- c("surv.coxph", rf_grid)
# 
# # Fit the ensemble (pseudo-code)
# fit_tuned <- SuperSurv(
#   time = train$duration,
#   event = train$event,
#   X = X_tr,
#   newdata = X_te,
#   new.times = new.times,
#   event.library = my_ml_library,
#   cens.library = c("surv.coxph"),
#   nFolds = 3
# )
# 
# # Inspect which hyperparameter combination won!
# round(event_weights(fit_tuned), 3)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----custom-learner-----------------------------------------------------------
my.surv.km <- function(time, event, X, newdata, new.times,
                       obsWeights = NULL, id = NULL, ...) {
  if (is.null(obsWeights)) {
    obsWeights <- rep(1, length(time))
  }

  fit_km <- survival::survfit(
    survival::Surv(time, event) ~ 1,
    weights = obsWeights
  )

  step_surv <- stats::stepfun(fit_km$time, c(1, fit_km$surv), right = FALSE)
  surv_probs <- step_surv(new.times)

  pred <- matrix(
    surv_probs,
    nrow = nrow(newdata),
    ncol = length(new.times),
    byrow = TRUE
  )

  fit <- list(object = fit_km)
  class(fit) <- "my.surv.km"

  list(pred = pred, fit = fit)
}

predict.my.surv.km <- function(object, newdata, new.times, ...) {
  fit_km <- object$object
  step_surv <- stats::stepfun(fit_km$time, c(1, fit_km$surv), right = FALSE)
  surv_probs <- step_surv(new.times)

  matrix(
    surv_probs,
    nrow = nrow(newdata),
    ncol = length(new.times),
    byrow = TRUE
  )
}

## ----custom-learner-check-----------------------------------------------------
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
times <- seq(50, 150, by = 50)

wrapper_out <- my.surv.km(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X[1:5, , drop = FALSE],
  new.times = times
)

dim(wrapper_out[["pred"]])
dim(predict(wrapper_out[["fit"]], newdata = X[1:5, , drop = FALSE], new.times = times))

## ----custom-fit, eval=FALSE---------------------------------------------------
# fit <- SuperSurv(
#   time = train$duration,
#   event = train$event,
#   X = X_train,
#   newdata = X_test,
#   new.times = seq(50, 300, by = 50),
#   event.library = c("surv.coxph", "my.surv.km"),
#   cens.library = c("surv.coxph"),
#   control = list(saveFitLibrary = TRUE)
# )
# 
# event_weights(fit)

## ----custom-screener, eval=FALSE----------------------------------------------
# fit <- SuperSurv(
#   time = train$duration,
#   event = train$event,
#   X = X_train,
#   newdata = X_test,
#   new.times = seq(50, 300, by = 50),
#   event.library = list(c("surv.coxph", "screen.all", "my.screen.first3")),
#   cens.library = c("surv.coxph")
# )
# 
# selected_variables(fit, learner = 2)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_rpart <- requireNamespace("rpart", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Standard Train/Test Split
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# Define a diverse library of base learners
my_library <- c("surv.coxph", "surv.weibull")
if (has_rpart) {
  my_library <- c(my_library, "surv.rpart")
}

## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
# 1. The Ensemble Super Learner (Weighted Average)
fit_ensemble <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "ensemble",     # <-- Calculates fractional weights
  nFolds = 3
)

# 2. The 'Best' Super Learner (Winner-Take-All)
fit_best <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "best",         # <-- Selects the single best model
  nFolds = 3
)

## ----inspect-weights----------------------------------------------------------
cat("\n--- ENSEMBLE WEIGHTS (selection = 'ensemble') ---\n")
print(round(event_weights(fit_ensemble), 4))

cat("\n--- BEST MODEL WEIGHTS (selection = 'best') ---\n")
print(round(event_weights(fit_best), 4))

## ----evaluate-comparison------------------------------------------------------
# Evaluate the Ensemble
cat("Performance of the ENSEMBLE Super Learner:\n")
eval_summary(fit_ensemble, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)

cat("\nPerformance of the BEST MODEL Super Learner:\n")
eval_summary(fit_best, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
has_rfsrc <- requireNamespace("randomForestSRC", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
set.seed(123)

# Load built-in data
data("metabric", package = "SuperSurv")

# Define predictors and time grid
X <- metabric[, grep("^x", names(metabric))]
new.times <- seq(10, 150, by = 10)

## ----train-model--------------------------------------------------------------
event_library <- c("surv.coxph", "surv.weibull")
if (has_rfsrc) {
  event_library <- c("surv.coxph", "surv.rfsrc")
}

fit <- SuperSurv(
  time = metabric$duration,
  event = metabric$event,
  X = X,
  newdata = X,
  new.times = new.times,
  event.library = event_library,
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE) 
)

## ----causal-effect------------------------------------------------------------
# Estimate the adjusted difference up to tau = 100 months
results <- estimate_marginal_rmst(
  fit = fit, 
  data = metabric, 
  trt_col = "x4", 
  times = new.times, 
  tau = 100
)



print(results$ATE_RMST)

## -----------------------------------------------------------------------------
rmst_results_inf <- estimate_marginal_rmst(
  fit = fit,
  data = metabric,
  trt_col = "x4",
  times = new.times,
  tau = 100,
  inference = TRUE,
  B = 100,
  seed = 123
)

rmst_results_inf$ATE_RMST
rmst_results_inf$SE_RMST
rmst_results_inf$CI_RMST
format.pval(rmst_results_inf$p_value, digits = 3, eps = 1e-16)

## ----plot-curve---------------------------------------------------------------
# Plot the Delta RMST across a sequence of tau values
tau_grid <- seq(20, 140, by = 30)
plot_marginal_rmst_curve(
  fit = fit, 
  data = metabric, 
  trt_col = "x4", 
  times = new.times, 
  tau_seq = tau_grid,
  inference = TRUE, 
  B = 100, 
  seed = 123, 
  ci_level = 0.95
)

## ----plot-obs-----------------------------------------------------------------
plot_rmst_vs_obs(
  fit = fit, 
  data = metabric, 
  time_col = "duration", 
  event_col = "event", 
  times = new.times, 
  tau = 350
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_rfsrc <- requireNamespace("randomForestSRC", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

new.times <- seq(50, 200, by = 25) 

## ----standalone-rf, eval=requireNamespace("randomForestSRC", quietly = TRUE)----
# 1. Fit the standalone wrapper
rf_standalone <- surv.rfsrc(
  time = train$duration,
  event = train$event,
  X = X_tr,
  new.times = new.times 
)

# 2. Extract the fitted model object and prediction matrix
rf_fit <- rf_standalone[["fit"]]
rf_pred_matrix <- rf_standalone[["pred"]]

## ----plot-standalone, fig.align='center', eval=requireNamespace("randomForestSRC", quietly = TRUE)----
# Plot the first 3 patients in our training set
plot_predict(preds = rf_pred_matrix, eval_times = new.times, patient_idx = 1:3)

## ----eval-standalone, fig.align='center', fig.height=4, fig.width= 9, eval=requireNamespace("randomForestSRC", quietly = TRUE)----
# The function automatically detects this is a single model and plots it!
plot_benchmark(
  object = rf_fit,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## -----------------------------------------------------------------------------

# plot_calibration(
#   object   = rf_fit,
#   newdata  = X_te,
#   time     = test$duration,
#   event    = test$event,
#   eval_time = 150,
#   bins     = 2
# )


## ----fit-models, results='hide', message=FALSE, warning=FALSE, eval=requireNamespace("randomForestSRC", quietly = TRUE)----
my_library <- c("surv.coxph", "surv.weibull", "surv.rfsrc")

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = c("surv.coxph"), 
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)

## ----plot-ensemble-benchmark, fig.align='center', fig.height=9, eval=requireNamespace("randomForestSRC", quietly = TRUE)----
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# Define a library covering different parametric assumptions
parametric_library <- c("surv.coxph",       # Semi-parametric baseline
                        "surv.weibull",     # Assumes hazard increases/decreases monotonically
                        "surv.exponential", # Assumes constant hazard over time
                        "surv.lognormal")   # Assumes hazard rises then falls

## ----fit-parametric, results='hide', message=FALSE, warning=FALSE-------------
fit_parametric <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = parametric_library,
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 3
)

## ----evaluate-parametric------------------------------------------------------
summary(fit_parametric)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_glmnet <- requireNamespace("glmnet", quietly = TRUE)
has_rpart <- requireNamespace("rpart", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Create 100 columns of random noise (simulating irrelevant genes/biomarkers)
n_patients <- nrow(metabric)
noise_data <- matrix(rnorm(n_patients * 20), nrow = n_patients)
colnames(noise_data) <- paste0("noise_", 1:20)
# Combine the real data with the noise
high_dim_data <- cbind(metabric, noise_data)

# Standard Train/Test Split
train_idx <- sample(1:nrow(high_dim_data), 0.7 * nrow(high_dim_data))
train <- high_dim_data[train_idx, ]
test  <- high_dim_data[-train_idx, ]

X_tr <- train[, grep("^x|^noise", names(high_dim_data))]
X_te <- test[, grep("^x|^noise", names(high_dim_data))]
new.times <- seq(50, 200, by = 25)

## ----define-library-----------------------------------------------------------
# We pass a list of vectors. The first element is the prediction model, 
# and the second element is the screening algorithm.

my_screen_library <- list(
  c("surv.coxph", "screen.all"),         # Baseline: Cox model with ALL variables
  c("surv.coxph", "screen.marg")         # Screen by marginal association, then fit Cox
)

if (has_glmnet) {
  my_screen_library[[length(my_screen_library) + 1L]] <- c("surv.weibull", "screen.elasticnet")
}

if (has_rpart) {
  my_screen_library[[length(my_screen_library) + 1L]] <- c("surv.rpart", "screen.var")
}

# For the censoring library, we use an unscreened approach 
cens_library <- c("surv.coxph")

## ----fit-screen, results='hide', message=FALSE, warning=FALSE-----------------
fit_highdim <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_screen_library, # Pass our screened library
  cens.library = cens_library,
  control = list(saveFitLibrary = TRUE),
  verbose = T,
  nFolds = 3
)

## ----evaluate-screen----------------------------------------------------------
# Check the integrated performance
screened_performance <- eval_summary(
  object = fit_highdim,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

# Plot the benchmark
# plot_benchmark(fit_highdim, newdata = X_te, time = test$duration, event = test$event, eval_times = new.times)

## ----extract-features---------------------------------------------------------
# Get the variables retained by the second event learner.
selected_features <- selected_variables(fit_highdim, learner = 2)

cat("Total features evaluated:", ncol(X_tr), "\n")
cat("Features retained by learner 2:", length(selected_features), "\n\n")

cat("Selected Features:\n")
print(selected_features)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
has_rpart <- requireNamespace("rpart", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

# Load built-in METABRIC data
data("metabric", package = "SuperSurv")

# Quick 80/20 Train-Test split
set.seed(42)
n_total <- nrow(metabric)
train_idx <- sample(1:n_total, 0.8 * n_total)

train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

# Extract just the X covariates (assuming they are named x0, x1, etc.)
x_cols <- grep("^x", names(metabric), value = TRUE)
X_tr <- train[, x_cols]
X_te <- test[, x_cols]

# Define the prediction time grid (e.g., survival at 50, 100, 150, 200 months)
new.times <- c(50, 100, 150, 200)

## ----define-library-----------------------------------------------------------
my_library <- c("surv.coxph", "surv.weibull")
if (has_rpart) {
  my_library <- c(my_library, "surv.rpart")
}

## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
# Fit 1: Least Squares Meta-learner
fit_ls <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,                 # Predict on the test set immediately
  new.times = new.times,       # Our evaluation time grid
  event.library = my_library,
  cens.library = my_library,
  metalearner = "brier", 
  control = list(saveFitLibrary = TRUE), 
  verbose = T,             # Turn to TRUE in practice to see progress!
  selection = "ensemble",
  nFolds = 5                   # 5-fold CV for the meta-learner
)

# Fit 2: Negative Log-Likelihood Meta-learner
fit_nll <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  metalearner = "logloss",       # Swap to nloglik
  control = list(saveFitLibrary = TRUE), 
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 5
)

## ----object-interface---------------------------------------------------------
fit_ls

summary(fit_ls)

event_weights(fit_ls)

learner_names(fit_ls)

eval_times(fit_ls)

selected_variables(fit_ls, learner = 1)

## ----inspect-weights----------------------------------------------------------
cat("\n--- LEAST SQUARES METALEARNER ---\n")
summary(fit_ls)

cat("\n--- NLOGLIK METALEARNER ---\n")
summary(fit_nll)

## ----predict-new--------------------------------------------------------------
# Select 3 brand new patients from our test set
new_patients <- X_te[1:6, ]

# Generate predictions using the Least Squares ensemble
ensemble_preds <- predict(
  object = fit_ls, 
  newdata = new_patients, 
  new.times = new.times,
  type = "event"
)

cat("\n--- PREDICTED SURVIVAL PROBABILITIES ---\n")
final_matrix <- ensemble_preds
colnames(final_matrix) <- paste0("Time_", new.times)
rownames(final_matrix) <- paste0("Patient_", 1:6)

print(round(final_matrix, 4))

## ----plot-predict, fig.align='center', fig.width=6, fig.height=5--------------
# Plot the predicted survival curves for our 3 new patients (Rows 1, 2, and 3)
plot_predict(
  preds = ensemble_preds,
  eval_times = new.times,
  patient_idx = 1:6
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_rfsrc <- requireNamespace("randomForestSRC", quietly = TRUE)
has_survex <- requireNamespace("survex", quietly = TRUE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)
X_explain_subset <- X_te[1:50, ]
X_background_subset <- X_tr[1:100, ]

my_library <- c("surv.coxph", "surv.weibull")
if (has_rfsrc) {
  my_library <- c(my_library, "surv.rfsrc")
}

fit_sl <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = c("surv.coxph"),
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 3
)

## ----calc-shap, message=FALSE, warning=FALSE, eval=FALSE----------------------
# # Calculate weighted Ensemble SHAP values using Kernel SHAP
# shap_vals <- explain_kernel(
#   model = fit_sl,
#   X_explain = X_explain_subset,
#   X_background = X_background_subset,
#   nsim = 20
# )

## ----plot-global, fig.width=6, fig.height=4, eval=FALSE-----------------------
# plot_global_importance(shap_vals, top_n = 5)

## ----plot-beeswarm, fig.width=7, fig.height=5, warning=FALSE, eval=FALSE------
# plot_beeswarm(shap_vals, data = X_explain_subset, top_n = 5)

## ----plot-waterfall, fig.width=6, fig.height=4, eval=FALSE--------------------
# # Explain Patient #1 from our test subset
# plot_patient_waterfall(shap_vals, patient_index = 1, top_n = 5)

## ----plot-dependence, fig.width=6, fig.height=4, warning=FALSE, eval=FALSE----
# plot_dependence(shap_vals, data = X_explain_subset, feature_name = "x0")

## ----plot-heatmap, fig.width=7, fig.height=5, eval=requireNamespace("ggplot2", quietly = TRUE) && requireNamespace("tidyr", quietly = TRUE) && requireNamespace("dplyr", quietly = TRUE)----
# Plot the survival trajectories for the first 50 test patients
plot_survival_heatmap(fit_sl, newdata = X_explain_subset, times = new.times)

## ----survex-bridge, message=FALSE, warning=FALSE, eval=requireNamespace("survex", quietly = TRUE)----
library(survex)

# 1. Create the true survival object for the explanation subset
y_explain <- survival::Surv(test$duration[1:50], test$event[1:50])

# 2. Build the survex explainer using our custom function
surv_explainer <- explain_survex(
  model = fit_sl, 
  data = X_explain_subset, 
  y = y_explain, 
  times = new.times
)

## ----survex-importance, fig.width=7, fig.height=5, message=FALSE, warning=FALSE, eval=requireNamespace("survex", quietly = TRUE)----
# Calculate time-dependent model parts (permutation feature importance)
time_importance <- model_parts(surv_explainer)

# Plot the dynamic importance over time
plot(time_importance)

## ----survex-profile, fig.width=7, fig.height=5, message=FALSE, warning=FALSE, eval=requireNamespace("survex", quietly = TRUE)----
# Calculate the partial dependence profile for feature 'x0'
pdp_time <- model_profile(surv_explainer, variables = "x0")

plot(pdp_time)

## ----survex-survshap, fig.width=7, fig.height=5, message=FALSE, warning=FALSE, eval=requireNamespace("survex", quietly = TRUE)----
# Explain Patient #1 over time
patient_1_data <- X_explain_subset[1, , drop = FALSE]

# Calculate SurvSHAP(t)
survshap_t <- predict_parts(surv_explainer, new_observation = patient_1_data, type = "survshap")

plot(survshap_t)

## ----survex-performance, fig.width=7, fig.height=5, message=FALSE, warning=FALSE, eval=requireNamespace("survex", quietly = TRUE)----
# Calculate time-dependent performance metrics via survex
survex_perf <- model_performance(surv_explainer)

# Plot the Brier score and AUC curves
plot(survex_perf)

#' Control parameters for the SuperSurv Ensemble
#'
#' @param max.SL.iter Maximum iterations for the iterative weighting algorithm. Default 20.
#' @param event.t.grid Optional time grid for event risk calculation.
#' @param cens.t.grid Optional time grid for censoring risk calculation.
#' @param saveFitLibrary Logical. If TRUE (default), saves models for future predictions.
#' @param initWeightAlg The learner used for the very first step of IPCW.
#' @param initWeight Whether to start by fitting "censoring" or "event" weights.
#' @param ... Additional ignored arguments.
#' @return A list of control parameters.
#' @keywords internal
SuperSurv.control <- function(max.SL.iter = 20,
                              event.t.grid = NULL,
                              cens.t.grid = NULL,
                              saveFitLibrary = TRUE,
                              initWeightAlg = "surv.coxph", # Changed to a faster default
                              initWeight = "censoring",
                              ...) {

  if(!(initWeight %in% c("event", "censoring"))) {
    stop("initWeight must be one of 'event' or 'censoring'.")
  }

  # We return a list. If t.grids are NULL, the main SuperSurv function
  # will calculate them based on the observed data.
  list(
    max.SL.iter = max.SL.iter,
    event.t.grid = event.t.grid,
    cens.t.grid = cens.t.grid,
    saveFitLibrary = saveFitLibrary,
    initWeightAlg = initWeightAlg,
    initWeight = initWeight
  )
}





#' Control parameters for Cross-Validation in SuperSurv
#'
#' @param V Number of folds. Default is 10.
#' @param stratifyCV Logical. If TRUE, ensures event rates are balanced across folds.
#' @param shuffle Logical. If TRUE, shuffles rows before splitting.
#' @param validRows Optional custom list of indices for folds.
#'
#' @return A list of CV parameters.
#' @keywords internal
SuperSurv.CV.control <- function(V = 10L,
                                 stratifyCV = TRUE,
                                 shuffle = TRUE,
                                 validRows = NULL) {

  V <- as.integer(V)

  # Logic check: Can't have more folds than observations
  # (This will be double-checked in the main function once N is known)

  if (!is.null(validRows)) {
    if (!is.list(validRows)) {
      stop("validRows must be a list of length V.")
    }
    if (!identical(V, length(validRows))) {
      # If user provided 5 folds of indices but set V=10, we must stop.
      V <- length(validRows)
      warning(paste("V adjusted to match length of validRows:", V))
    }
  }

  list(V = V, stratifyCV = stratifyCV, shuffle = shuffle, validRows = validRows)
}

########## function previously inside the main function
### Get cross-validated survivals for S and T
#' @noRd
.crossValFUN <- function(valid, validRows, time, event, dataX, id, obsWeights, t.grid,
                         library, kScreen, k, p, verbose) {

  foldNum <- as.numeric(which(unlist(lapply(validRows, function(v) all.equal(v, valid) == TRUE))))
  tempLearn <- dataX[-valid, , drop = FALSE]
  tempTime <- time[-valid]
  tempEvent <- event[-valid]
  tempValid <- dataX[valid, , drop = FALSE]
  tempWhichScreen <- matrix(NA, nrow = kScreen, ncol = p)
  tempId <- id[-valid]
  tempObsWeights <- obsWeights[-valid]
  for (s in seq(kScreen)) {
    if(verbose) message(paste("CV ", library$screenAlgorithm[s],
                              ", fold ", foldNum, sep = ""))
    screen_fn <- get(library$screenAlgorithm[s])
    testScreen <- try(do.call(screen_fn, list(time = tempTime,
                                              event = tempEvent,
                                              X = tempLearn, id = tempId,
                                              obsWeights = tempObsWeights)))
    if (inherits(testScreen, "try-error")) {
      warning(paste("replacing failed screening algorithm,",
                    library$screenAlgorithm[s], ", with All()",
                    "\n "))
      tempWhichScreen[s, ] <- TRUE
    }
    else {
      tempWhichScreen[s, ] <- testScreen
    }
    if (verbose) {
      message(paste("Number of covariates in ", library$screenAlgorithm[s],
                    " is: ", sum(tempWhichScreen[s, ]), sep = ""))
    }
  }

  uniqueScreen <- unique(tempWhichScreen)
  screenMap <- apply(uniqueScreen, 1, function(row) which(apply(tempWhichScreen, 1, function(row2) all.equal(row, row2) == TRUE)))

  out <- array(NA, dim = c(nrow(tempValid), length(t.grid), k))

  for (predAlg in unique(library$library$predAlgorithm)) {
    if (verbose) message(paste("CV ", predAlg, ", fold ", foldNum, sep = ""))
    pred_fn <- get(predAlg)
    for(j in seq(nrow(uniqueScreen))) {
      testAlg <- try(do.call(pred_fn, list(time = tempTime, event = tempEvent,
                                           X = subset(tempLearn, select = uniqueScreen[j,], drop = FALSE),
                                           newdata = subset(tempValid, select = uniqueScreen[j,], drop = FALSE),
                                           new.times = t.grid,
                                           id = tempId,
                                           obsWeights = tempObsWeights)))
      if (inherits(testAlg, "try-error")) {
        warning(paste("Error in algorithm", predAlg,
                      "\n  The Algorithm will be removed from the Super Learner (i.e. given weight 0) \n"))
      } else {
        libraryRows <- which(library$library$predAlgorithm == predAlg & library$library$rowScreen %in% unlist(screenMap[j]))
        for (row in libraryRows) {
          # --- Add this safety check ---
          expected_length <- nrow(out[, , row]) * ncol(out[, , row])
          actual_length <- length(testAlg$pred)

          if (actual_length != expected_length) {
            stop(sprintf("\n>>> CRASH CAUGHT! <<<\nAlgorithm Index: %s\nExpected matrix size: %s x %s\nBut the algorithm returned length: %s\nCheck if this wrapper was updated to accept 'newdata'!",
                         row, nrow(out[, , row]), ncol(out[, , row]), actual_length))
          }

          out[,,row] <- testAlg$pred
        }
      }
    }
  }


  invisible(list(out = out))
}




# Generate Full Screening logic
#' @noRd
.screenFun <- function(fun, list) {
  screen_fn <- get(fun)
  # Robust screen call
  testScreen <- try(do.call(screen_fn, list), silent = TRUE)

  if (inherits(testScreen, "try-error")) {
    # Print the ACTUAL error so the developer can see it!
    warning("Screening crashed in: ", fun, call. = FALSE)
    warning("Error log: ", testScreen[1], call. = FALSE)
    warning("Defaulting to keeping all variables.", call. = FALSE)
    out <- rep(TRUE, ncol(list$X))
  } else {
    out <- testScreen
  }
  return(out)
}


.predFun <- function(index, lib, time, event, dataX, newdata, whichScreen, t.grid,
                     family, id, obsWeights, verbose, control, libraryNames) {
  if (verbose) {
    message(paste("full", libraryNames[index]))
  }
  pred_fn <- get(lib$predAlgorithm[index])
  testAlg <- try(do.call(pred_fn, list(time = time, event = event,
                                       X = subset(dataX, select = whichScreen[lib$rowScreen[index], ],
                                                  drop = FALSE),
                                       newdata = subset(newdata, select = whichScreen[lib$rowScreen[index],
                                       ], drop = FALSE), id = id,
                                       obsWeights = obsWeights, new.times = t.grid)))
  if (inherits(testAlg, "try-error")) {
    warning(paste("Error in algorithm", lib$predAlgorithm[index],
                  " on full data", "\n  The Algorithm will be removed from the Super Learner (i.e. given weight 0) \n"))
    out <- rep.int(NA, times = nrow(newdata))
    model_out <- NULL
  }
  else {
    out <- testAlg$pred
    if(control$saveFitLibrary) model_out <- testAlg$fit
    else model_out <- NULL
  }

  invisible(list(out = out, model_out = model_out))
}






########## function previously inside the main function
#' @noRd
.checkInputs <- function(time, event, X, newdata, id, obsWeights, verbose) {
  if(any(time < 0)) stop("Only non-negative event/censoring times allowed!")
  if(any(!(event %in% c(0,1)))) stop("Event must be binary.")
  if(any(is.na(time)) | any(is.na(event))) stop("No missing values allowed in time or event.")
  if(any(is.na(X)) | any(is.na(newdata))) stop("No missing values allowed in X or new X.")
  if(length(time) != length(event) | length(time) != nrow(X)) stop("time and event must be n x 1 vectors and X must have n rows.")
  if(!is.data.frame(X) | !is.data.frame(newdata)) stop("X and newdata must be data frames.")
  if(!identical(names(X), names(newdata))) stop("X and newdata must have the same features.")
  if(any(obsWeights < 0)) stop("obsWeights < 0 not allowed.")
  if(!(verbose %in% c(TRUE, FALSE))) stop("verbose must be TRUE/FALSE.")
  if(!is.null(id) && !identical(length(id), length(time))) stop("id vector must have the same dimension as time")
}


#' @noRd
.validate_library_argument <- function(survSL.library, arg_name) {
  format_spec <- paste0(
    "`", arg_name, "` must be either a character vector of learner names ",
    "(for example, `c(\"surv.coxph\", \"surv.km\")`) or a list where each element ",
    "is a character vector beginning with a learner name followed optionally by screener names ",
    "(for example, `list(c(\"surv.coxph\", \"screen.all\"))`)."
  )

  validate_names <- function(x, context) {
    context_label <- if (nzchar(context)) paste0(context, " ") else ""
    if (!is.character(x)) {
      stop("`", arg_name, "` ", context_label, "must be a character vector. ", format_spec, call. = FALSE)
    }
    if (length(x) < 1L) {
      stop("`", arg_name, "` ", context_label, "must contain at least one function name. ", format_spec, call. = FALSE)
    }
    if (any(is.na(x)) || any(!nzchar(trimws(x)))) {
      stop("`", arg_name, "` ", context_label, "contains missing or empty function names. ", format_spec, call. = FALSE)
    }

    missing_funs <- x[!vapply(x, exists, logical(1L), mode = "function", inherits = TRUE)]
    if (length(missing_funs) > 0L) {
      stop(
        "`", arg_name, "` ", context_label, "contains unknown function name(s): ",
        paste(unique(missing_funs), collapse = ", "),
        ". Define these functions before calling `SuperSurv()`. ",
        format_spec,
        call. = FALSE
      )
    }
  }

  if (inherits(survSL.library, "SuperSurv_grid")) {
    validate_names(as.character(survSL.library), "generated by `create_grid()`")
    return(invisible(TRUE))
  }

  if (is.character(survSL.library)) {
    validate_names(survSL.library, "")
    return(invisible(TRUE))
  }

  if (!is.list(survSL.library)) {
    stop(format_spec, call. = FALSE)
  }

  if (length(survSL.library) < 1L) {
    stop("`", arg_name, "` must not be empty. ", format_spec, call. = FALSE)
  }

  for (ii in seq_along(survSL.library)) {
    validate_names(survSL.library[[ii]], paste0("element ", ii))
  }

  invisible(TRUE)
}


#' @noRd
.createLibrary <- function (survSL.library)  {
  if (is.character(survSL.library)) {
    k <- length(survSL.library)
    whichScreen <- matrix(1, nrow = 1, ncol = k)
    screenAlgorithm <- "screen.all"
    library <- data.frame(predAlgorithm = survSL.library, rowScreen = 1,
                          stringsAsFactors = FALSE)
  }
  else if (is.list(survSL.library)) {
    predNames <- sapply(survSL.library, FUN = "[", 1)
    NumberScreen <- (sapply(survSL.library, FUN = length) - 1)
    if (sum(NumberScreen == 0) > 0) {
      for (ii in which(NumberScreen == 0)) {
        survSL.library[[ii]] <- c(survSL.library[[ii]], "")
        NumberScreen[ii] <- 1
      }
    }
    screenAlgorithmFull <- unlist(lapply(survSL.library, FUN = "[", -1))
    screenAlgorithm <- unique(screenAlgorithmFull)
    library <- data.frame(predAlgorithm = rep(predNames,
                                              times = NumberScreen), rowScreen = match(screenAlgorithmFull,
                                                                                       screenAlgorithm), stringsAsFactors = FALSE)
  }
  else {
    stop("format for survSL.library is not recognized")
  }
  out <- list(library = library, screenAlgorithm = screenAlgorithm)
  return(out)
}


#' @noRd
.survCVFolds <- function (N, id, event, cvControl) {
  if (!is.null(cvControl$validRows)) return(cvControl$validRows)
  stratifyCV <- cvControl$stratifyCV
  shuffle <- cvControl$shuffle
  V <- cvControl$V
  if (!stratifyCV) {
    if (shuffle) {
      if (is.null(id)) {
        validRows <- split(sample(1:N), rep(1:V, length = N))
      }
      else {
        n.id <- length(unique(id))
        id.split <- split(sample(1:n.id), rep(1:V, length = n.id))
        validRows <- vector("list", V)
        for (v in seq(V)) {
          validRows[[v]] <- which(id %in% unique(id)[id.split[[v]]])
        }
      }
    }
    else {
      if (is.null(id)) {
        validRows <- split(1:N, rep(1:V, length = N))
      }
      else {
        n.id <- length(unique(id))
        id.split <- split(1:n.id, rep(1:V, length = n.id))
        validRows <- vector("list", V)
        for (v in seq(V)) {
          validRows[[v]] <- which(id %in% unique(id)[id.split[[v]]])
        }
      }
    }
  }
  else {
    # if (sum(event) < V | sum(1-event) < V) {
    #   stop("number of (event = 1) or (event = 0) is less than the number of folds")
    # }
    if (shuffle) {
      if (is.null(id)) {
        event.0 <- which(event == 0)
        event.1 <- which(event == 1)
        rows.0 <- split(sample(event.0), rep(1:V, length = length(event.0)))
        rows.1 <- split(sample(event.1), rep(1:V, length = length(event.1)))
        validRows <- vector("list", length = V)
        names(validRows) <- paste(seq(V))
        for (vv in seq(V)) {
          if (length(rows.0) >= vv) {
            if (length(rows.1) >= vv) validRows[[vv]] <- c(rows.0[[vv]], rows.1[[vv]])
            else validRows[[vv]] <- rows.0[[vv]]
          } else {
            validRows[[vv]] <- rows.1[[vv]]
          }
        }
      }
      else {
        stop("Stratified sampling with id not currently implemented. Either remove id or set control(stratifyCV = FALSE).")
      }
    }
    else {
      if (is.null(id)) {
        within.split <- suppressWarnings(tapply(1:N,
                                                INDEX = event, FUN = split, rep(1:V)))
        validRows <- vector("list", length = V)
        names(validRows) <- paste(seq(V))
        for (vv in seq(V)) {
          validRows[[vv]] <- c(within.split[[1]][[vv]],
                               within.split[[2]][[vv]])
        }
      }
      else {
        stop("Stratified sampling with id not currently implemented. Either remove id or set control(stratifyCV = FALSE).")
      }
    }
  }
  return(validRows)
}











#' Internal function to perform iterative Super Learner optimization
#' @noRd
.surviterativeSL <- function(event.Z, cens.Z, time, event, X, obsWeights, id, control, verbose,
                             event.errorsInLibrary, cens.errorsInLibrary,
                             metalearner = "brier") {

  if (verbose) message("Performing iterative SuperLearner optimization...")
  if (!metalearner %in% c("brier", "entropy", "logloss")) {
    stop("metalearner must be one of: 'brier', 'entropy', 'logloss'")
  }

  trace <- isTRUE(control$traceIter)   # don't depend on verbose
  tol   <- if (!is.null(control$tol)) control$tol else 1e-5

  event.k <- dim(event.Z)[3]
  cens.k  <- dim(cens.Z)[3]
  N <- length(time)
  event.n.time <- length(control$event.t.grid)
  cens.n.time  <- length(control$cens.t.grid)

  epsilon <- max(min(diff(sort(unique(time)))), 1e-5)

  # ---- flatten to long matrices: (N*T) x K ----
  event.Z.long <- matrix(NA_real_, nrow = N * event.n.time, ncol = event.k)
  for (j in seq_len(event.k)) event.Z.long[, j] <- c(event.Z[,,j])

  cens.Z.long <- matrix(NA_real_, nrow = N * cens.n.time, ncol = cens.k)
  for (j in seq_len(cens.k)) cens.Z.long[, j] <- c(cens.Z[,,j])

  # ---- observed values at each subject time (for updating the other side) ----
  event.Z.obs <- matrix(NA_real_, nrow = N, ncol = event.k)
  for (i in seq_len(N)) {
    for (j in seq_len(event.k)) {
      event.Z.obs[i, j] <- stats::approx(
        x = control$event.t.grid, y = event.Z[i, , j],
        xout = time[i], method = "constant", rule = 2, ties = mean
      )$y
    }
  }

  cens.Z.obs <- matrix(NA_real_, nrow = N, ncol = cens.k)
  for (i in seq_len(N)) {
    for (j in seq_len(cens.k)) {
      cens.Z.obs[i, j] <- stats::approx(
        x = c(-1, control$cens.t.grid), y = c(1, cens.Z[i, , j]),
        xout = time[i] - epsilon, method = "constant", rule = 2, ties = mean
      )$y
    }
  }

  # ---- long vectors ----
  obsWeights.event.long <- rep(obsWeights, event.n.time)
  obsWeights.cens.long  <- rep(obsWeights, cens.n.time)
  time.event.long <- rep(time, event.n.time)
  time.cens.long  <- rep(time, cens.n.time)
  event.event.long <- rep(event, event.n.time)
  event.cens.long  <- rep(event, cens.n.time)
  event.t.grid.long <- rep(control$event.t.grid, each = N)
  cens.t.grid.long  <- rep(control$cens.t.grid,  each = N)

  initWeightAlg <- get(control$initWeightAlg)

  obs.cens.vals <- NULL
  obs.event.vals <- NULL
  S.coef <- rep(0, event.k)
  G.coef <- rep(0, cens.k)

  # ---- Initialization ----
  if (control$initWeight == "censoring") {

    initFit <- initWeightAlg(
      time = time, event = 1 - event, X = X, newdata = X,
      new.times = time - epsilon, obsWeights = obsWeights, id = id
    )

    obs.cens.vals <- rep(diag(initFit$pred), length(control$event.t.grid))
    obs.cens.vals <- pmax(obs.cens.vals, 1e-4)

    # logloss needs G(t|X) on the grid which we don't have yet -> init with brier
    init_method <- if (metalearner == "logloss") "brier" else metalearner

    S.coef[!event.errorsInLibrary] <- .survcomputeCoef(
      time = time.event.long, event = event.event.long,
      t.vals = event.t.grid.long, cens.vals = obs.cens.vals,
      preds = event.Z.long[, !event.errorsInLibrary, drop = FALSE],
      obsWeights = obsWeights.event.long,
      method = init_method
    )

    obs.event.vals <- rep(drop(event.Z.obs %*% S.coef), length(control$cens.t.grid))
    obs.event.vals <- pmax(obs.event.vals, 1e-4)

  } else {

    initFit <- initWeightAlg(
      time = time, event = event, X = X, newdata = X,
      new.times = time, obsWeights = obsWeights, id = id
    )

    obs.event.vals <- rep(diag(initFit$pred), length(control$cens.t.grid))
    obs.event.vals <- pmax(obs.event.vals, 1e-4)
  }



  # ---- Iteration ----
  iter <- 1
  G_t_long <- NULL

  while (TRUE) {


    if (iter > control$max.SL.iter) {
      warning("Did not converge in ", control$max.SL.iter, " iterations")
      break
    }

    if (!is.null(obs.cens.vals)) obs.cens.vals.old <- obs.cens.vals
    if (!is.null(obs.event.vals)) obs.event.vals.old <- obs.event.vals

    # ---- Update censoring weights (G.coef) ----
    G.coef <- rep(0, cens.k)
    # IMPORTANT: Do NOT use logloss for censoring weight update.
    # Keep censoring update on Brier (Westling-style).
    G_method <- if (metalearner == "logloss") "brier" else metalearner

    G.coef[!cens.errorsInLibrary] <- .survcomputeCoef(
      time = time.cens.long, event = 1 - event.cens.long,
      t.vals = cens.t.grid.long, cens.vals = obs.event.vals,
      preds = cens.Z.long[, !cens.errorsInLibrary, drop=FALSE],
      obsWeights = obsWeights.cens.long,
      method = G_method
    )

    # ---- Update obs.cens.vals = G(tildeT|X) replicated across event grid ----
    obs.cens.vals <- rep(drop(cens.Z.obs %*% G.coef), length(control$event.t.grid))
    obs.cens.vals <- pmax(obs.cens.vals, 1e-4)

    # ---- If logloss: compute G_t_long = G(t|X) for each (i,t_event) row ----
    if (metalearner == "logloss") {

      if (all(cens.errorsInLibrary)) {
        stop("All censoring learners failed; cannot compute G(t|X) for logloss.")
      }

      # 1) ensemble on censoring grid using LONG matrix (2D) -> no conformability error
      G_cens_long <- drop(
        cens.Z.long[, !cens.errorsInLibrary, drop = FALSE] %*% G.coef[!cens.errorsInLibrary]
      )
      G_cens_long <- pmax(G_cens_long, 1e-4)

      # 2) reshape to N x cens.n.time
      G_cens_grid <- matrix(G_cens_long, nrow = N, ncol = cens.n.time)

      # 3) interpolate each row from censoring grid -> event grid, then flatten long
      G_event_grid <- matrix(NA_real_, nrow = N, ncol = event.n.time)
      for (i in seq_len(N)) {
        G_event_grid[i, ] <- stats::approx(
          x = control$cens.t.grid,
          y = G_cens_grid[i, ],
          xout = control$event.t.grid,
          method = "constant",
          rule = 2,
          ties = mean
        )$y
      }
      G_event_grid <- pmax(G_event_grid, 1e-4)
      G_t_long <- as.vector(G_event_grid)

      if (length(G_t_long) != length(event.t.grid.long)) {
        stop("Internal error: G_t_long length mismatch with event.t.grid.long.")
      }
    }

    # ---- Update event weights (S.coef) ----
    if (metalearner == "logloss") {
      if (is.null(G_t_long)) stop("G_t_long missing for logloss; check censoring library.")

      S.coef <- rep(0, event.k)
      S.coef[!event.errorsInLibrary] <- .survcomputeCoef(
        time = time.event.long, event = event.event.long,
        t.vals = event.t.grid.long,
        cens.vals = obs.cens.vals,      # G(tildeT|X) replicated across t
        G_t = G_t_long,                 # G(t|X) for each (i,t) row
        preds = event.Z.long[, !event.errorsInLibrary, drop = FALSE],
        obsWeights = obsWeights.event.long,
        method = "logloss"
      )

    } else {

      S.coef <- rep(0, event.k)
      S.coef[!event.errorsInLibrary] <- .survcomputeCoef(
        time = time.event.long, event = event.event.long,
        t.vals = event.t.grid.long, cens.vals = obs.cens.vals,
        preds = event.Z.long[, !event.errorsInLibrary, drop = FALSE],
        obsWeights = obsWeights.event.long,
        method = metalearner
      )
    }

    obs.event.vals <- rep(drop(event.Z.obs %*% S.coef), length(control$cens.t.grid))
    obs.event.vals <- pmax(obs.event.vals, 1e-4)



    if (verbose) {
      topK_S <- min(3, length(S.coef))
      topK_G <- min(3, length(G.coef))

      ordS <- order(S.coef, decreasing = TRUE)[seq_len(topK_S)]
      ordG <- order(G.coef, decreasing = TRUE)[seq_len(topK_G)]

      labS <- if (!is.null(names(S.coef))) names(S.coef)[ordS] else as.character(ordS)
      labG <- if (!is.null(names(G.coef))) names(G.coef)[ordG] else as.character(ordG)

      if (isTRUE(verbose)) {
        message(sprintf("[iter=%d] top S: %s | top G: %s",
                  iter,
                  paste(paste0(labS, "=", round(S.coef[ordS], 3)), collapse = ", "),
                  paste(paste0(labG, "=", round(G.coef[ordG], 3)), collapse = ", ")))
      }
      utils::flush.console()
    }

    # ---- Convergence ----
    # ---- after updating obs.cens.vals and obs.event.vals ----
    cens.delta  <- Inf
    event.delta <- Inf

    if (!is.null(obs.cens.vals.old) && !is.null(obs.event.vals.old)) {
      cens.delta  <- max(abs(obs.cens.vals  - obs.cens.vals.old))
      event.delta <- max(abs(obs.event.vals - obs.event.vals.old))
    }

    if (isTRUE(verbose)) {
      message(sprintf("[iter=%d] cens.delta=%.3e  event.delta=%.3e  sum=%.3e",
                iter, cens.delta, event.delta, cens.delta + event.delta))
      }

    tol <- if (!is.null(control$tol)) control$tol else 1e-5
    if (cens.delta + event.delta < tol) {
      if (verbose) message("Converged in ", iter, " iterations.")
      break
    }



    iter <- iter + 1
  }

  # ---- CV risks ----
  calc_risk <- function(preds, time, event, t.grid, cens_T, weights, method, G_t = NULL) {

    if (method == "brier") {
      Y_ipcw <- 1 - (as.numeric(time <= t.grid) * event / cens_T)
      return(apply(preds, 2, function(col) mean(weights * (Y_ipcw - col)^2, na.rm = TRUE)))
    }

    if (method == "entropy") {
      Y_ipcw <- 1 - (as.numeric(time <= t.grid) * event / cens_T)
      Y_clamp <- pmax(pmin(Y_ipcw, 1), 0)
      return(apply(preds, 2, function(col) {
        p <- pmax(pmin(col, 1 - 1e-15), 1e-15)
        loss <- -(weights * (Y_clamp * log(p) + (1 - Y_clamp) * log(1 - p)))
        mean(loss, na.rm = TRUE)
      }))
    }

    if (method == "logloss") {
      if (is.null(G_t)) stop("calc_risk(method='logloss') requires G_t.")
      G_T <- pmax(cens_T, 1e-4)
      G_t <- pmax(G_t, 1e-4)
      fail <- as.numeric(time <= t.grid) * event
      surv <- as.numeric(time >  t.grid)

      w_cap <- 100
      w_fail <- pmin(fail / G_T, w_cap)
      w_surv <- pmin(surv / G_t, w_cap)

      return(apply(preds, 2, function(col) {
        S <- pmin(pmax(col, 1e-10), 1 - 1e-10)
        loss <- -weights * (w_fail * log(1 - S) + w_surv * log(S))
        mean(loss, na.rm = TRUE)
      }))
    }

    stop("Unknown method in calc_risk")
  }

  # for risk calc, rebuild final G_t_long the same way if logloss
  if (metalearner == "logloss") {
    if (is.null(G_t_long)) stop("G_t_long missing at risk computation stage.")
    G_t_long_final <- G_t_long
  } else {
    G_t_long_final <- NULL
  }

  event.cvRisks <- calc_risk(
    preds = event.Z.long,
    time = time.event.long,
    event = event.event.long,
    t.grid = event.t.grid.long,
    cens_T = obs.cens.vals,
    weights = obsWeights.event.long,
    method = metalearner,
    G_t = G_t_long_final
  )

  # keep censoring risk as brier (same as Westling-style)
  cens.cvRisks <- calc_risk(
    preds = cens.Z.long,
    time = time.cens.long,
    event = event.cens.long,
    t.grid = cens.t.grid.long,
    cens_T = obs.event.vals,
    weights = obsWeights.cens.long,
    method = "brier"
  )

  list(
    event.coef = S.coef,
    cens.coef  = G.coef,
    event.cvRisks = event.cvRisks,
    cens.cvRisks  = cens.cvRisks
  )
}











#' Internal function to compute ensemble weights
#' @importFrom nnls nnls
#' @noRd
.survcomputeCoef <- function(time, event, t.vals, cens.vals, preds, obsWeights, method = "brier", G_t = NULL) {

  if (!method %in% c("brier","entropy","logloss")) {
    stop("Unknown method: ", method, ". Use 'brier', 'entropy', or 'logloss'.")
  }

  # Safety: Remove NA columns
  valid_cols <- !apply(preds, 2, anyNA)
  if(sum(valid_cols) == 0) return(rep(1/ncol(preds), ncol(preds)))

  preds_clean <- preds[, valid_cols, drop=FALSE]
  k_clean <- ncol(preds_clean)
  if(k_clean == 1) {
    full_w <- rep(0, ncol(preds))
    full_w[valid_cols] <- 1
    return(full_w)
  }

  cens.vals[cens.vals < 1e-4] <- 1e-4

  # ----------------------------------------------------------------------------
  # METHOD 1: Least Squares (Brier Score)
  # ----------------------------------------------------------------------------
  if (method == "brier") {
    # IPCW "Observed" Outcome (1 if alive, 0 if dead, weighted)
    out <- 1 - as.numeric(time <= t.vals) * event / cens.vals

    fit.nnls <- nnls::nnls(sqrt(obsWeights) * preds_clean, sqrt(obsWeights) * out)
    coef <- coef(fit.nnls)
    if(sum(coef) == 0) coef <- rep(1, length(coef))

    final_w <- coef / sum(coef)
  }


  # ----------------------------------------------------------------------------
  # METHOD 2: entropy (cross-entropy on IPCW pseudo-outcomes; simplex via softmax)
  # ----------------------------------------------------------------------------
  else if (method == "entropy") {

    # Raw IPCW pseudo-outcome
    Y_ipcw <- 1 - (as.numeric(time <= t.vals) * event / cens.vals)

    # Clamp target into [0,1] so cross-entropy is well-defined
    Y_clamp <- pmax(pmin(Y_ipcw, 1), 0)

    # Softmax maps unconstrained theta -> simplex weights
    softmax <- function(theta) {
      z <- theta - max(theta)
      w <- exp(z)
      w / sum(w)
    }

    # Loss as a function of weights w (same as your original idea)
    loss_w <- function(w) {
      P_ens <- drop(preds_clean %*% w)
      P_ens <- pmax(pmin(P_ens, 1 - 1e-10), 1e-10)

      loss <- -(obsWeights * (Y_clamp * log(P_ens) + (1 - Y_clamp) * log(1 - P_ens)))
      mean(loss)
    }

    # Optimize over theta (unconstrained), then transform via softmax
    loss_theta <- function(theta) loss_w(softmax(theta))

    opt <- try(stats::optim(par = rep(0, k_clean), fn = loss_theta, method = "BFGS"),
               silent = TRUE)

    if (inherits(opt, "try-error")) {
      final_w <- rep(1 / k_clean, k_clean)
    } else {
      final_w <- softmax(opt$par)
    }
  }

  # ----------------------------------------------------------------------------
  # METHOD 3: IPCW log-loss (simplex via softmax + optim)
  # ----------------------------------------------------------------------------
  else if (method == "logloss") {

    if (is.null(G_t)) {
      stop("method='logloss' requires G_t = G(t|X) aligned with the (i,t) long rows.")
    }

    # Floors for numerical stability
    G_T <- pmax(cens.vals, 1e-4)   # should be G(tildeT|X) replicated across t
    G_t <- pmax(G_t, 1e-4)         # should be G(t|X) for each (i,t) row

    fail <- as.numeric(time <= t.vals) * event      # Δ I(T<=t)
    surv <- as.numeric(time >  t.vals)              # I(T>t)

    # Softmax maps unconstrained theta -> simplex weights
    softmax <- function(theta) {
      z <- theta - max(theta)
      w <- exp(z)
      w / sum(w)
    }

    # Loss as a function of weights w
    loss_w <- function(w) {
      S_ens <- drop(preds_clean %*% w)
      S_ens <- pmin(pmax(S_ens, 1e-10), 1 - 1e-10)

      # Optional but recommended: cap IPCW multipliers
      w_cap <- 100
      w_fail <- pmin(fail / G_T, w_cap)
      w_surv <- pmin(surv / G_t, w_cap)

      loss <- -obsWeights * (w_fail * log(1 - S_ens) + w_surv * log(S_ens))
      mean(loss)
    }

    # Optimize over theta (unconstrained), then transform via softmax
    loss_theta <- function(theta) loss_w(softmax(theta))

    opt <- try(stats::optim(par = rep(0, k_clean), fn = loss_theta, method = "BFGS"),
               silent = TRUE)

    if (inherits(opt, "try-error")) {
      final_w <- rep(1 / k_clean, k_clean)
    } else {
      final_w <- softmax(opt$par)
    }
  }

  # Expand back to full vector
  full_w <- rep(0, ncol(preds))
  full_w[valid_cols] <- final_w
  return(full_w)
}

#' @keywords internal
#' @noRd
.make_design_matrix <- function(X) {
  X_df <- as.data.frame(X)

  # If user passed a matrix, preserve colnames
  if (is.matrix(X)) X_df <- as.data.frame(X)

  # model.matrix handles factors/characters safely via one-hot encoding
  mm <- stats::model.matrix(~ . - 1, data = X_df)

  # Ensure it's a plain numeric matrix
  mm <- as.matrix(mm)
  storage.mode(mm) <- "double"
  mm
}



#' Keep All Variables Screener
#' @param X Training covariate data.frame.
#' @param ... Additional ignored arguments.
#'
#' @return A logical vector of the same length as the number of columns in \code{X},
#'   indicating which variables passed the screening algorithm (\code{TRUE} to keep,
#'   \code{FALSE} to drop).
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:20, ]
#' x_cols <- grep("^x", names(dat))[1:5]
#' X <- dat[, x_cols, drop = FALSE]
#'
#' screen.all(X)
#' @export
screen.all <- function(X, ...) {
  whichVariable <- rep(TRUE, ncol(X))
  names(whichVariable) <- colnames(X)
  return(whichVariable)
}




#' Marginal Cox Regression Screening
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param obsWeights Observation weights.
#' @param minscreen Minimum number of covariates to return. Defaults to 2.
#' @param min.p Threshold p-value. Defaults to 0.1.
#' @param ... Additional ignored arguments.
#' @return A logical vector of the same length as the number of columns in \code{X},
#'   indicating which variables passed the screening algorithm (\code{TRUE} to keep,
#'   \code{FALSE} to drop).
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:40, ]
#' x_cols <- grep("^x", names(dat))[1:5]
#' X <- dat[, x_cols, drop = FALSE]
#'
#' screen.marg(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   minscreen = 2,
#'   min.p = 0.2
#' )
#' @export
screen.marg <- function(time, event, X, obsWeights = NULL,
                        minscreen = 2, min.p = 0.1, ...) {

  requireNamespace("survival", quietly = TRUE)

  X_mat <- .make_design_matrix(X)

  pvals <- vapply(seq_len(ncol(X_mat)), function(j) {
    colj <- X_mat[, j]

    fit_try <- try({
      if (is.null(obsWeights)) {
        survival::coxph(survival::Surv(time, event) ~ colj)
      } else {
        survival::coxph(survival::Surv(time, event) ~ colj, weights = obsWeights)
      }
    }, silent = TRUE)

    if (inherits(fit_try, "try-error")) return(1.0)

    smry <- summary(fit_try)
    # Single coefficient p-value
    p <- suppressWarnings(as.numeric(smry$coefficients[,"Pr(>|z|)"][1]))
    if (is.na(p)) p <- 1.0
    p
  }, numeric(1))

  whichVariable <- (pvals <= min.p)

  # Safety net: keep at least minscreen
  if (sum(whichVariable, na.rm = TRUE) < minscreen) {
    whichVariable <- rep(FALSE, ncol(X_mat))
    whichVariable[order(pvals)[seq_len(minscreen)]] <- TRUE
  }
  names(whichVariable) <- colnames(X_mat)

  # IMPORTANT: return mask in original X_mat space (design matrix columns)
  # This is OK if downstream uses the same design matrix builder.
  return(whichVariable)
}



#' GLMNET (Lasso) Screening
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param obsWeights Observation weights.
#' @param alpha Penalty exponent (1 = lasso).
#' @param minscreen Minimum number of covariates to return. Defaults to 2.
#' @param nfolds Number of CV folds.
#' @param nlambda Number of penalty parameters.
#' @param ... Additional ignored arguments.
#' @return A logical vector of the same length as the number of columns in \code{X},
#'   indicating which variables passed the screening algorithm (\code{TRUE} to keep,
#'   \code{FALSE} to drop).
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:40, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'
#'   screen.glmnet(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     alpha = 1,
#'     minscreen = 2,
#'     nfolds = 3,
#'     nlambda = 20
#'   )
#' }
#' @export
screen.glmnet <- function(time, event, X, obsWeights = NULL,
                          alpha = 1, minscreen = 2,
                          nfolds = 10, nlambda = 100, ...) {

  requireNamespace("glmnet", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)

  # ---- Fix 1: Cox family requires strictly positive times ----
  time <- pmax(time, 1e-5)

  # Build a safe numeric design matrix (same philosophy as your wrappers)
  X_df <- as.data.frame(X)
  X_mat <- stats::model.matrix(~ . - 1, data = X_df)
  X_mat <- as.matrix(X_mat)
  storage.mode(X_mat) <- "double"

  if (is.null(obsWeights)) obsWeights <- rep(1, nrow(X_mat))

  fit.glmnet <- suppressWarnings(
    glmnet::cv.glmnet(
      x = X_mat,
      y = survival::Surv(time, event),
      weights = obsWeights,
      family = "cox",
      alpha = alpha,
      nfolds = nfolds,
      nlambda = nlambda
    )
  )

  # ---- Fix 2: use exported coef() ----
  beta_min <- as.matrix(stats::coef(fit.glmnet, s = "lambda.min"))
  whichVariable <- as.vector(beta_min != 0)

  # Safety net: keep at least minscreen
  if (sum(whichVariable) < minscreen) {
    beta_path <- as.matrix(fit.glmnet$glmnet.fit$beta)  # p x nlambda
    nnz <- apply(beta_path, 2, function(b) sum(b != 0))

    idx <- which(nnz >= minscreen)
    if (length(idx) > 0) {
      use <- idx[1]
      whichVariable <- as.vector(beta_path[, use] != 0)
    } else {
      use <- which.max(nnz)
      whichVariable <- as.vector(beta_path[, use] != 0)
      if (sum(whichVariable) == 0) whichVariable <- rep(TRUE, ncol(X_mat))
    }
  }

  names(whichVariable) <- colnames(X_mat)

  return(whichVariable)
}




#' Random Survival Forest Screening Algorithm
#'
#' This screening algorithm uses the \code{randomForestSRC} package to select covariates
#' based on their Variable Importance (VIMP). It grows a fast forest and retains features
#' with a VIMP greater than zero.
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (1 = event, 0 = censored).
#' @param X Training covariate data.frame or matrix.
#' @param obsWeights Numeric vector of observation weights.
#' @param minscreen Integer. Minimum number of covariates to return. Defaults to 2.
#' @param ntree Integer. Number of trees to grow. Defaults to 100 for fast screening.
#' @param ... Additional arguments passed to \code{\link[randomForestSRC]{rfsrc}}.
#'
#' @return A logical vector of the same length as the number of columns in \code{X},
#'   indicating which variables passed the screening algorithm (\code{TRUE} to keep,
#'   \code{FALSE} to drop).
#' @examples
#' if (requireNamespace("randomForestSRC", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:40, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'
#'   screen.rfsrc(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     minscreen = 2,
#'     ntree = 10
#'   )
#' }
#' @export
screen.rfsrc <- function(time, event, X, obsWeights = NULL,
                         minscreen = 2, ntree = 100, ...) {

  requireNamespace("randomForestSRC", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)
  Surv <- survival::Surv

  X_df <- as.data.frame(X)

  if (is.null(obsWeights)) obsWeights <- rep(1, nrow(X_df))

  data_rf <- data.frame(time = time, event = event, X_df)

  fit_rf <- randomForestSRC::rfsrc(
    Surv(time, event) ~ .,
    data = data_rf,
    ntree = ntree,
    case.wt = obsWeights,
    importance = TRUE,
    nsplit = 2,
    ...
  )

  vimp <- fit_rf$importance
  if (is.null(vimp)) {
    # fallback: keep all if VIMP is unavailable
    return(rep(TRUE, ncol(X_df)))
  }

  # vimp is named by original X columns (not model.matrix columns)
  vimp <- as.numeric(vimp)
  whichVariable <- vimp > 0
  whichVariable[is.na(whichVariable)] <- FALSE

  if (sum(whichVariable) < minscreen) {
    whichVariable <- rep(FALSE, length(vimp))
    whichVariable[order(vimp, decreasing = TRUE)[seq_len(minscreen)]] <- TRUE
  }

  names(whichVariable) <- colnames(X_df)

  return(whichVariable)
}



#' High Variance Screening Algorithm (Unsupervised)
#'
#' An unsupervised screening algorithm that filters out low-variance features.
#' This is particularly useful for high-dimensional genomic or transcriptomic data
#' where many features remain relatively constant across all observations.
#'
#' @param time Numeric vector of observed follow-up times (Ignored internally).
#' @param event Numeric vector of event indicators (Ignored internally).
#' @param X Training covariate data.frame or matrix.
#' @param obsWeights Numeric vector of observation weights (Ignored internally).
#' @param keep_fraction Numeric value between 0 and 1. The fraction of highest-variance
#' features to retain. Defaults to 0.5 (keeps the top 50%).
#' @param minscreen Integer. Minimum number of covariates to return. Defaults to 2.
#' @param ... Additional ignored arguments.
#'
#' @return A logical vector of the same length as the number of columns in \code{X},
#'   indicating which variables passed the screening algorithm (\code{TRUE} to keep,
#'   \code{FALSE} to drop).
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:40, ]
#' x_cols <- grep("^x", names(dat))[1:6]
#' X <- dat[, x_cols, drop = FALSE]
#'
#' screen.var(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   keep_fraction = 0.5,
#'   minscreen = 2
#' )
#' @export
screen.var <- function(time, event, X, obsWeights = NULL,
                       keep_fraction = 0.5, minscreen = 2, ...) {

  X_mat <- .make_design_matrix(X)

  vars <- apply(X_mat, 2, function(col) stats::var(col, na.rm = TRUE))

  cutoff <- stats::quantile(vars, probs = 1 - keep_fraction, na.rm = TRUE)
  whichVariable <- (vars >= cutoff)
  whichVariable[is.na(whichVariable)] <- FALSE

  if (sum(whichVariable) < minscreen) {
    whichVariable <- rep(FALSE, length(vars))
    whichVariable[order(vars, decreasing = TRUE)[seq_len(minscreen)]] <- TRUE
  }
  names(whichVariable) <- colnames(X_mat)

  return(whichVariable)
}




#' Elastic Net Screening Algorithm
#'
#' This screening algorithm uses \code{\link[glmnet]{cv.glmnet}} to select covariates.
#' Unlike LASSO (\code{alpha = 1}), which drops correlated features, Elastic Net
#' (\code{alpha = 0.5} by default) shrinks correlated groups of features together,
#' making it ideal for selecting entire biological pathways.
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (1 = event, 0 = censored).
#' @param X Training covariate data.frame or matrix.
#' @param obsWeights Numeric vector of observation weights.
#' @param alpha Numeric penalty exponent for \code{glmnet}. Defaults to 0.5 (Elastic Net).
#' @param minscreen Integer. Minimum number of covariates to return. Defaults to 2.
#' @param nfolds Integer. Number of folds for cross-validation. Defaults to 10.
#' @param nlambda Integer. Number of penalty parameters to search over. Defaults to 100.
#' @param ... Additional arguments passed to \code{screen.glmnet}.
#'
#' @return A logical vector of the same length as the number of columns in \code{X},
#'   indicating which variables passed the screening algorithm (\code{TRUE} to keep,
#'   \code{FALSE} to drop).
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:40, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'
#'   screen.elasticnet(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     alpha = 0.5,
#'     minscreen = 2,
#'     nfolds = 3,
#'     nlambda = 20
#'   )
#' }
#' @export
screen.elasticnet <- function(time, event, X, obsWeights = NULL, alpha = 0.5, minscreen = 2, nfolds = 10, nlambda = 100, ...) {

  # We simply pass the arguments directly to our robust screen.glmnet wrapper,
  # forcing alpha = 0.5 instead of alpha = 1.

  whichVariable <- screen.glmnet(
    time = time,
    event = event,
    X = X,
    obsWeights = obsWeights,
    alpha = alpha,
    minscreen = minscreen,
    nfolds = nfolds,
    nlambda = nlambda,
    ...
  )

  return(whichVariable)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

# Our max follow-up is well beyond 200, so this grid is safe.
new.times <- seq(50, 200, by = 25) 

## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)

## ----evaluate-performance-----------------------------------------------------
# Evaluate performance directly using the fitted model and test data
performance_results <- eval_summary(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## ----plot-benchmark, fig.align='center', fig.width=9, fig.height=3------------
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## ----plot-calibration, fig.align='center'-------------------------------------
# Assess calibration specifically at Time = 150
plot_calibration(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_time = 150,
  bins = 5 # Group patients into 5 risk quintiles
)

# Define global variables for ggplot2 to satisfy R CMD check
utils::globalVariables(c(
  "C_Index", "Mean_Predicted", "Observed", "Survival_Prob",
  "Patient", "Feature", "Importance", "SHAP", "FeatureValue",
  "Direction", "Time", "Survival",
  # add these:
  "Tau", "ATE", "Observed_Time", "Predicted_RMST", "Event"
))

#' Plot Longitudinal Benchmark Metrics
#'
#' Generates time-dependent performance curves comparing the SuperSurv ensemble
#' against its base learners, or evaluates a single standalone learner.
#'
#' @param object A fitted SuperSurv object OR a fitted standalone learner.
#' @param newdata A data.frame of test covariates.
#' @param time Numeric vector of observed follow-up times for the test set.
#' @param event Numeric vector of event indicators for the test set.
#' @param eval_times Numeric vector of times at which to evaluate predictions.
#' @param metrics Character vector specifying which plots to return.
#'   Options: "brier", "auc", "cindex". Defaults to all three.
#' @param verbose Logical; if TRUE, progress messages are shown. Defaults to FALSE.
#' @return A combined patchwork ggplot object, or a single ggplot if only one metric is selected.
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:120, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   eval_times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = eval_times,
#'     event.library = c("surv.coxph", "surv.ranger"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   plot_benchmark(
#'     object = fit,
#'     newdata = X,
#'     time = dat$duration,
#'     event = dat$event,
#'     eval_times = eval_times,
#'     metrics = c("brier")
#'   )
#' }
#' @export
plot_benchmark <- function(object, newdata, time, event, eval_times,
                           metrics = c("brier", "auc", "cindex"),
                           verbose = FALSE) {

  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("patchwork", quietly = TRUE)

  if (isTRUE(verbose)) {message("Generating predictions for benchmark plots...") }

  preds <- predict(object, newdata = newdata, new.times = eval_times)

  get_metrics_df <- function(model_name, S_mat) {
    brier_vals <- eval_brier(time, event, S_mat, eval_times)$brier_scores
    auc_vals <- suppressMessages(eval_timeROC(time, event, S_mat, eval_times)$AUC_curve)

    cindex_vals <- vapply(eval_times, function(t) {
      eval_cindex(time, event, S_mat, eval_times, eval_time = t, method = "uno")
    }, numeric(1))

    data.frame(
      Time = eval_times,
      Model = model_name,
      Brier = brier_vals,
      CD_AUC = auc_vals,
      C_Index = cindex_vals,
      stringsAsFactors = FALSE
    )
  }

  if (isTRUE(verbose)) {message("Calculating time-dependent metrics...") }

  res_list <- list()

  # SMART EXTRACTION LOGIC
  if (is.list(preds) && !is.null(preds$event.predict)) {
    # SuperSurv ensemble
    k_models <- dim(preds$event.library.predict)[3]

    model_names <- object$event.libraryNames
    if (is.null(model_names) || length(model_names) != k_models) {
      # fall back to dimnames if present
      dn <- dimnames(preds$event.library.predict)[[3]]
      if (!is.null(dn) && length(dn) == k_models) {
        model_names <- dn
      } else {
        model_names <- paste0("Base_Learner_", seq_len(k_models))
      }
    }

    all_names <- c("SuperSurv_Ensemble", model_names)

    res_list[[1]] <- get_metrics_df("SuperSurv_Ensemble", preds$event.predict)
    for (i in seq_len(k_models)) {
      res_list[[i + 1]] <- get_metrics_df(model_names[i], preds$event.library.predict[, , i])
    }

  } else if (is.matrix(preds)) {
    # Standalone learner
    model_name <- class(object)[1]
    if (is.null(model_name) || model_name %in% c("matrix")) model_name <- "Standalone_Model"

    all_names <- model_name
    res_list[[1]] <- get_metrics_df(model_name, preds)

  } else {
    stop("Unrecognized prediction format. Must be a SuperSurv object or a valid base learner with matrix predictions.")
  }

  plot_df <- do.call(rbind, res_list)
  plot_df$Model <- factor(plot_df$Model, levels = all_names)

  survex_colors <- c("#e41a1c", "#377eb8", "#4daf4a", "#984ea3",
                     "#ff7f00", "#ffff33", "#a65628", "#f781bf")
  survex_colors <- rep(survex_colors, length.out = length(all_names))
  names(survex_colors) <- all_names

  plots <- list()

  if ("brier" %in% metrics) {
    plots$brier <- ggplot2::ggplot(plot_df, ggplot2::aes(x = Time, y = Brier, color = Model)) +
      ggplot2::geom_line(linewidth = 1.2) +
      ggplot2::geom_point(size = 3) +
      ggplot2::scale_color_manual(values = survex_colors) +
      ggplot2::theme_minimal() +
      ggplot2::labs(title = "IPCW Brier Score", y = "Brier Score (Lower is better)") +
      ggplot2::theme(
        plot.title = ggplot2::element_text(size = 20),
        axis.title.y = ggplot2::element_text(size = 18),
        axis.title.x = ggplot2::element_text(size = 18))
  }

  if ("auc" %in% metrics) {
    plots$auc <- ggplot2::ggplot(plot_df, ggplot2::aes(x = Time, y = CD_AUC, color = Model)) +
      ggplot2::geom_line(linewidth = 1.2) +
      ggplot2::geom_point(size = 3) +
      ggplot2::scale_color_manual(values = survex_colors) +
      ggplot2::theme_minimal() +
      ggplot2::labs(title = "Time-Dependent AUC", y = "AUC (Higher is better)")+
      ggplot2::theme(
        plot.title = ggplot2::element_text(size = 20),
        axis.title.y = ggplot2::element_text(size = 18),
        axis.title.x = ggplot2::element_text(size = 18))
  }

  if ("cindex" %in% metrics) {
    plots$cindex <- ggplot2::ggplot(plot_df, ggplot2::aes(x = Time, y = C_Index, color = Model)) +
      ggplot2::geom_line(linewidth = 1.2) +
      ggplot2::geom_point(size = 3) +
      ggplot2::scale_color_manual(values = survex_colors) +
      ggplot2::theme_minimal() +
      ggplot2::labs(title = "Uno's C-index", y = "C-index (Higher is better)")+
      ggplot2::theme(
        plot.title = ggplot2::element_text(size = 20),
        axis.title.y = ggplot2::element_text(size = 18),
        axis.title.x = ggplot2::element_text(size = 18))
  }

  if (length(plots) == 0) stop("No valid metrics selected.")
  if (length(plots) == 1) return(plots[[1]])

  patchwork::wrap_plots(plots, nrow = 1) +
    patchwork::plot_layout(guides = "collect") &
    ggplot2::theme(
      legend.position = "bottom",
      legend.direction = "horizontal",
      legend.title = ggplot2::element_text(size = 16),
      legend.text = ggplot2::element_text(size = 16)
    ) &
    ggplot2::guides(
      fill = ggplot2::guide_legend(nrow = 3, byrow = TRUE),
      color = ggplot2::guide_legend(nrow = 3, byrow = TRUE)
    )
}





#' Plot Survival Calibration Curve
#'
#' @param object A fitted SuperSurv object OR a standalone base learner.
#' @param newdata A data.frame of test covariates.
#' @param time Numeric vector of observed follow-up times for the test set.
#' @param event Numeric vector of event indicators for the test set.
#' @param eval_time Numeric. A single time point at which to assess calibration.
#' @param bins Integer. Defaults to 5.
#' @return A ggplot object.
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:120, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   eval_times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = eval_times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   plot_calibration(
#'     object = fit,
#'     newdata = X,
#'     time = dat$duration,
#'     event = dat$event,
#'     eval_time = 100,
#'     bins = 4
#'   )
#' }
#' @export
plot_calibration <- function(object, newdata, time, event, eval_time, bins = 5) {
  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)

  preds <- predict(object, newdata = newdata, new.times = eval_time)

  if (is.list(preds) && !is.null(preds$event.predict)) {
    S_pred <- preds$event.predict[, 1]
  } else if (is.matrix(preds)) {
    S_pred <- preds[, 1]
  } else {
    stop("Unrecognized model object or prediction output.")
  }

  df <- data.frame(time = time, event = event, pred = S_pred)

  # Tie-safe quantile binning
  qs <- stats::quantile(df$pred, probs = seq(0, 1, length.out = bins + 1), na.rm = TRUE)
  qs <- unique(qs)
  if (length(qs) <= 2) stop("Not enough unique prediction values to form calibration bins.")

  df$bin <- cut(df$pred, breaks = qs, include.lowest = TRUE, labels = FALSE)

  calib_data <- data.frame(Bin = integer(), Mean_Predicted = numeric(), Observed = numeric())

  for (i in sort(unique(df$bin))) {
    sub_df <- df[df$bin == i, , drop = FALSE]
    mean_pred <- mean(sub_df$pred, na.rm = TRUE)

    km <- survival::survfit(survival::Surv(time, event) ~ 1, data = sub_df)
    idx <- findInterval(eval_time, km$time)
    obs_surv <- ifelse(idx == 0, 1, km$surv[idx])

    calib_data <- rbind(calib_data, data.frame(
      Bin = i,
      Mean_Predicted = mean_pred,
      Observed = obs_surv
    ))
  }

  ggplot2::ggplot(calib_data, ggplot2::aes(x = Mean_Predicted, y = Observed)) +
    ggplot2::geom_point(size = 4, color = "#e41a1c") +
    ggplot2::geom_line(color = "#e41a1c", linewidth = 1) +
    ggplot2::geom_abline(slope = 1, intercept = 0, color = "black",
                         linetype = "dashed", linewidth = 1) +
    ggplot2::scale_x_continuous(limits = c(0, 1), breaks = seq(0, 1, 0.2)) +
    ggplot2::scale_y_continuous(limits = c(0, 1), breaks = seq(0, 1, 0.2)) +
    ggplot2::theme_minimal() +
    ggplot2::labs(
      title = sprintf("Calibration Curve at Time = %s", eval_time),
      subtitle = "Red line = Model Calibration | Dashed line = Perfect Calibration",
      x = "Predicted Survival Probability",
      y = "Observed Survival Probability (Kaplan-Meier)"
    )
}






#' Plot Predicted Survival Curves
#'
#' @param preds A list containing SuperSurv predictions OR a raw prediction matrix.
#' @param eval_times Numeric vector of times at which predictions were evaluated.
#' @param patient_idx Integer vector. Defaults to 1.
#' @return A ggplot object.
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:120, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   eval_times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = eval_times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   preds <- predict(fit, newdata = X, new.times = eval_times, type = "event")
#'
#'   plot_predict(
#'     preds = preds,
#'     eval_times = eval_times,
#'     patient_idx = c(1, 2)
#'   )
#' }
#' @export
plot_predict <- function(preds, eval_times, patient_idx = 1) {
  requireNamespace("ggplot2", quietly = TRUE)

  if (is.list(preds) && !is.null(preds$event.predict)) {
    S_mat <- preds$event.predict
  } else if (is.matrix(preds)) {
    S_mat <- preds
  } else {
    stop("preds must be a prediction list from SuperSurv or a prediction matrix.")
  }

  plot_df <- data.frame(Time = numeric(), Survival_Prob = numeric(), Patient = character(),
                        stringsAsFactors = FALSE)

  for (i in patient_idx) {
    temp_df <- data.frame(
      Time = eval_times,
      Survival_Prob = S_mat[i, ],
      Patient = paste("Patient", i)
    )
    plot_df <- rbind(plot_df, temp_df)
  }

  plot_df$Patient <- factor(plot_df$Patient, levels = paste("Patient", patient_idx))

  ggplot2::ggplot(plot_df, ggplot2::aes(x = Time, y = Survival_Prob, color = Patient)) +
    ggplot2::geom_step(linewidth = 1.2) +
    ggplot2::scale_y_continuous(limits = c(0, 1), breaks = seq(0, 1, 0.2)) +
    ggplot2::theme_minimal() +
    ggplot2::labs(
      title = "Predicted Patient Survival Curves",
      x = "Follow-up Time",
      y = "Predicted Survival Probability"
    ) +
    ggplot2::theme(
      legend.position = "bottom",
      plot.title = ggplot2::element_text(face = "bold")
    )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Standard Train/Test Split
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# Define a diverse library of base learners
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")

## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
# 1. The Ensemble Super Learner (Weighted Average)
fit_ensemble <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "ensemble",     # <-- Calculates fractional weights
  nFolds = 3
)

# 2. The 'Best' Super Learner (Winner-Take-All)
fit_best <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "best",         # <-- Selects the single best model
  nFolds = 3
)

## ----inspect-weights----------------------------------------------------------
cat("\n--- ENSEMBLE WEIGHTS (selection = 'ensemble') ---\n")
print(round(event_weights(fit_ensemble), 4))

cat("\n--- BEST MODEL WEIGHTS (selection = 'best') ---\n")
print(round(event_weights(fit_best), 4))

## ----evaluate-comparison------------------------------------------------------
# Evaluate the Ensemble
cat("Performance of the ENSEMBLE Super Learner:\n")
eval_summary(fit_ensemble, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)

cat("\nPerformance of the BEST MODEL Super Learner:\n")
eval_summary(fit_best, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)

#' Create a Tuning Grid of Survival Learners
#'
#' Dynamically generates custom wrapper functions for a specified base learner
#' across a grid of hyperparameters.
#'
#' @param base_learner Character string of the base learner function name (e.g., "surv.gbm").
#' @param grid_params List of numeric/character vectors containing hyperparameter values.
#' @return A character vector of class \code{"SuperSurv_grid"} containing the
#'   newly generated function names.
#' @export
#' @keywords internal
create_grid <- function(base_learner, grid_params) {

  param_grid <- expand.grid(grid_params, stringsAsFactors = FALSE)

  generated_learners <- character(nrow(param_grid))

  for (i in seq_len(nrow(param_grid))) {

    specific_params <- as.list(param_grid[i, , drop = FALSE])

    # Create descriptive learner name
    param_string <- paste(
      paste0(names(specific_params), specific_params),
      collapse = "_"
    )

    learner_name <- paste0(base_learner, "_", param_string)

    generated_learners[i] <- learner_name

    new_fun <- local({

      .base_learner <- base_learner
      .specific_params <- specific_params

      function(time, event, X, newdata, new.times, obsWeights = NULL, id = NULL, ...) {

        base_fun <- get(.base_learner, mode = "function")

        args_to_pass <- list(
          time = time,
          event = event,
          X = X,
          newdata = newdata,
          new.times = new.times,
          obsWeights = obsWeights,
          id = id
        )

        call_args <- c(args_to_pass, .specific_params, list(...))

        do.call(base_fun, call_args)
      }

    })

    assign(learner_name, new_fun, envir = parent.frame())
  }

  class(generated_learners) <- c("SuperSurv_grid", class(generated_learners))
  return(generated_learners)
}






#' Calculate Baseline Survival using Breslow Estimator
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (0=censored, 1=event).
#' @param risk_score Numeric vector of risk scores.
#' @param new.times Numeric vector of times at which to evaluate the baseline hazard.
#'
#' @return A numeric vector of cumulative baseline hazard #' computes \deqn{\hat{S}(t)}.
#' @keywords internal
#' @noRd
safe_breslow_step <- function(time, event, risk_score, new.times) {

  # 1. Sort data by time
  ord <- order(time, -event)
  time_sorted <- time[ord]
  event_sorted <- event[ord]

  # Safety: Clamp huge scores to avoid Inf.
  safe_score <- pmin(risk_score[ord], 700)
  risk_sorted <- exp(safe_score)

  # 2. Compute Risk Set Sums
  risk_pool <- rev(cumsum(rev(risk_sorted)))

  # 3. Compute Hazard contributions
  is_event <- event_sorted == 1
  if (!any(is_event)) return(rep(0, length(new.times)))

  event_t <- time_sorted[is_event]
  haz_contribution <- 1 / risk_pool[is_event]

  # 4. Aggregate ties
  unique_t <- unique(event_t)
  if (length(unique_t) < length(event_t)) {
    haz_contribution <- tapply(haz_contribution, event_t, sum)
    event_t <- as.numeric(names(haz_contribution))
  }

  # 5. Cumulative Hazard
  cum_haz <- cumsum(haz_contribution)

  # 6. Interpolate (Step Function)
  stats::approx(
    x = event_t,
    y = cum_haz,
    xout = new.times,
    method = "constant",
    yleft = 0,
    yright = max(cum_haz),
    f = 0,
    rule = 2,
    ties = mean
  )$y
}







#' Universal Risk Wrapper for SHAP
#'
#' Ensures "High Value = High Risk" orientation for ALL base learners and the
#' SuperSurv ensemble, allowing unified SHAP value calculations.
#'
#' @param object A fitted model object (e.g., from a single wrapper or a SuperSurv ensemble).
#' @param newdata A data.frame of new covariates to predict on.
#'
#' @return A numeric vector of risk scores of the same length as the number
#'   of rows in \code{newdata}, where higher values consistently indicate a higher
#'   risk of the event.
#' @keywords internal
#' @noRd
get_risk_universal <- function(object, newdata) {

  # --- 1. The Cox Family (High = Bad) ---
  if (inherits(object, "surv.coxph") || inherits(object, "coxph")) {
    # Extract native model to avoid predict.surv.coxph collision
    mod <- if(inherits(object, "surv.coxph")) object$object else object
    return(predict(mod, newdata = newdata, type = "lp"))
  }
  # --- REPLACE THIS BLOCK IN get_risk_universal ---
  else if (inherits(object, "surv.glmnet")) {
    mod <- if(inherits(object, "surv.glmnet")) object$object else object

    # glmnet REQUIRES a matrix, so we convert the SHAP background data
    newdata_df <- as.data.frame(newdata)
    newdata_mat <- stats::model.matrix(~ . - 1, data = newdata_df)

    # Predict the Linear Predictor (Risk)
    return(as.vector(predict(mod, newx = newdata_mat, s = "lambda.min", type = "link")))
  }
  else if (inherits(object, "surv.coxboost")) {
    mod <- object$object
    return(as.vector(predict(mod, newdata = as.matrix(newdata), type = "lp")))
  }

  # --- 2. The GAM ---
  else if (inherits(object, "surv.gam") || inherits(object, "gam")) {
    mod <- if(inherits(object, "surv.gam")) object$object else object
    return(as.vector(predict(mod, newdata = newdata, type = "link")))
  }

  # --- 3. The Machine Family (High = Bad) ---
  else if (inherits(object, "surv.gbm") || inherits(object, "gbm")) {
    mod <- if(inherits(object, "surv.gbm")) object$object else object
    best_iter <- if(inherits(object, "surv.gbm")) object$best.iter else mod$n.trees
    return(predict(mod, newdata = newdata, n.trees = best_iter, type = "link"))
  }
  else if (inherits(object, "surv.xgboost") || inherits(object, "xgb.Booster")) {
    mod <- if(inherits(object, "surv.xgboost")) object$object else object
    if(is.data.frame(newdata)) newdata <- as.matrix(newdata)
    return(predict(mod, newdata = newdata, outputmargin = TRUE))
  }
  else if (inherits(object, "surv.svm")) {
    mod <- object$object
    return(predict(mod, newdata = newdata)$predicted)
  }

  # --- 4. The Tree Family (High = Bad via Conversion) ---
  else if (inherits(object, "surv.rfsrc") || inherits(object, "rfsrc")) {
    mod <- if(inherits(object, "surv.rfsrc")) object$object else object
    return(predict(mod, newdata = newdata)$predicted)
  }
  else if (inherits(object, "surv.ranger") || inherits(object, "ranger")) {
    mod <- if(inherits(object, "surv.ranger")) object$object else object
    chf <- predict(mod, data = newdata)$chf
    mid_idx <- floor(ncol(chf)/2)
    return(chf[, mid_idx])
  }
  else if (inherits(object, "surv.bart")) {
    mod <- object$object
    surv_mat <- mod$surv.test.mean
    return(1 - rowMeans(surv_mat))
  }
  else if (inherits(object, "surv.aorsf") || inherits(object, "aorsf")) {
    mod <- if(inherits(object, "surv.aorsf")) object$object else object

    # 1. Get the Median Time Horizon (from training data)
    t_median <- median(mod$data[[1]], na.rm = TRUE)

    # 2. Predict Risk Directly (Fastest & Safest)
    risk <- predict(
      mod,
      new_data = newdata,
      pred_horizon = t_median,
      pred_type = "risk"
    )

    return(as.numeric(risk))
  }
  # Modify this line in get_risk_universal:
  else if (inherits(object, "surv.glmnet") || inherits(object, "surv.ridge")) {

    mod <- if(inherits(object, "surv.glmnet") || inherits(object, "surv.ridge")) object$object else object

    newdata_df <- as.data.frame(newdata)
    newdata_mat <- stats::model.matrix(~ . - 1, data = newdata_df)

    return(as.vector(predict(mod, newx = newdata_mat, s = "lambda.min", type = "link")))
  }

  # --- 5. The Parametric Family (High = GOOD -> FLIP SIGN!) ---
  else if (inherits(object, "surv.parametric") ||
           inherits(object, "survreg") ||
           inherits(object, "surv.weibull") ||
           inherits(object, "surv.exponential") ||
           inherits(object, "surv.lognormal") ||
           inherits(object, "surv.loglogistic")) {

    mod <- object$object
    # We use the linear predictor as the risk score for SHAP
    return(as.vector(predict(mod, newdata = as.data.frame(newdata), type = "linear")))
  }
  else if (inherits(object, "surv.rpart")) {
    mod <- object$object

    # rpart for survival returns the expected event rate.
    # We take the log (with a safety clamp) to get the linear predictor (risk score) for SHAP.
    rate_new <- predict(mod, newdata = as.data.frame(newdata))
    return(as.vector(log(pmax(rate_new, 1e-10))))
  }

  else {
    stop(paste("Unknown class in SHAP wrapper:", class(object)[1]))
  }
}

#' List Available Wrappers and Screeners in SuperSurv
#'
#' This function prints all built-in prediction algorithms and feature screening
#' algorithms available in the \code{SuperSurv} package.
#'
#' @param what Character string. If \code{"both"} (default), lists both prediction
#'   and screening functions. If \code{"surv"}, lists only prediction models.
#'   If \code{"screen"}, lists only screening algorithms. Otherwise, lists all exports.
#'
#' @return An invisible character vector containing the requested function names.
#' @examples
#' list_wrappers()
#' @export
list_wrappers <- function(what = "both") {

  # Grab all exported functions from the package
  everything <- sort(getNamespaceExports("SuperSurv"))

  # Identify models and screeners using regex
  models <- everything[grepl("^surv\\.", everything)]
  screens <- everything[grepl("^screen\\.", everything)]

  if (what == "both") {
    message("--- Prediction Models (surv.*) ---")
    print(models)
    message("\n--- Screening Algorithms (screen.*) ---")
    print(screens)
    invisible(c(models, screens))

  } else if (what == "surv") {
    message("--- Prediction Models (surv.*) ---")
    print(models)
    invisible(models)

  } else if (what == "screen") {
    message("--- Screening Algorithms (screen.*) ---")
    print(screens)
    invisible(screens)

  } else {
    message("--- All Exported Functions in SuperSurv ---")
    print(everything)
    invisible(everything)
  }
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
set.seed(123)

# Load built-in data
data("metabric", package = "SuperSurv")

# Define predictors and time grid
X <- metabric[, grep("^x", names(metabric))]
new.times <- seq(10, 150, by = 10)

## ----train-model--------------------------------------------------------------
fit <- SuperSurv(
  time = metabric$duration,
  event = metabric$event,
  X = X,
  newdata = X,
  new.times = new.times,
  event.library = c("surv.coxph", "surv.rfsrc"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE) 
)

## ----causal-effect------------------------------------------------------------
# Estimate the adjusted difference up to tau = 100 months
results <- estimate_marginal_rmst(
  fit = fit, 
  data = metabric, 
  trt_col = "x4", 
  times = new.times, 
  tau = 100
)



print(results$ATE_RMST)

## -----------------------------------------------------------------------------
rmst_results_inf <- estimate_marginal_rmst(
  fit = fit,
  data = metabric,
  trt_col = "x4",
  times = new.times,
  tau = 100,
  inference = TRUE,
  B = 100,
  seed = 123
)

rmst_results_inf$ATE_RMST
rmst_results_inf$SE_RMST
rmst_results_inf$CI_RMST
format.pval(rmst_results_inf$p_value, digits = 3, eps = 1e-16)

## ----plot-curve---------------------------------------------------------------
# Plot the Delta RMST across a sequence of tau values
tau_grid <- seq(20, 140, by = 30)
plot_marginal_rmst_curve(
  fit = fit, 
  data = metabric, 
  trt_col = "x4", 
  times = new.times, 
  tau_seq = tau_grid,
  inference = TRUE, 
  B = 100, 
  seed = 123, 
  ci_level = 0.95
)

## ----plot-obs-----------------------------------------------------------------
plot_rmst_vs_obs(
  fit = fit, 
  data = metabric, 
  time_col = "duration", 
  event_col = "event", 
  times = new.times, 
  tau = 350
)

library(SuperSurv)

data("metabric", package = "SuperSurv")
dat <- metabric[1:30, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
new_times <- seq(20, 100, by = 20)

expect_error_contains <- function(expr, pattern) {
  msg <- tryCatch(
    {
      force(expr)
      NULL
    },
    error = function(e) conditionMessage(e)
  )

  stopifnot(!is.null(msg), grepl(pattern, msg, fixed = TRUE))
}

km_grid <- create_grid("surv.km", list(dummy = 1))

fit_grid <- SuperSurv(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X[1:5, , drop = FALSE],
  new.times = new_times,
  event.library = km_grid,
  cens.library = "surv.km",
  nFolds = 2,
  verbose = FALSE
)

stopifnot(
  inherits(km_grid, "SuperSurv_grid"),
  is.character(km_grid),
  inherits(fit_grid, "SuperSurv")
)

expect_error_contains(
  SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X[1:5, , drop = FALSE],
    new.times = new_times,
    event.library = 1,
    cens.library = "surv.km",
    nFolds = 2,
    verbose = FALSE
  ),
  "`event.library` must be either a character vector of learner names"
)

expect_error_contains(
  SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X[1:5, , drop = FALSE],
    new.times = new_times,
    event.library = c("surv.km", ""),
    cens.library = "surv.km",
    nFolds = 2,
    verbose = FALSE
  ),
  "`event.library` contains missing or empty function names"
)

expect_error_contains(
  SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X[1:5, , drop = FALSE],
    new.times = new_times,
    event.library = "does.not.exist",
    cens.library = "surv.km",
    nFolds = 2,
    verbose = FALSE
  ),
  "`event.library` contains unknown function name(s): does.not.exist"
)

expect_error_contains(
  SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X[1:5, , drop = FALSE],
    new.times = new_times,
    event.library = "surv.km",
    cens.library = list(list("surv.km")),
    nFolds = 2,
    verbose = FALSE
  ),
  "`cens.library` element 1 must be a character vector"
)

#' Explain Predictions with Global SHAP (Kernel SHAP)
#'
#' @param model A fitted SuperSurv object OR a single wrapper output.
#' @param X_explain The dataset you want to explain (e.g., `X_test[1:10, ]`).
#' @param X_background The reference dataset for fastshap (e.g., `X_train[1:100, ]`).
#' @param nsim Number of simulations. Defaults to 20.
#' @param only_best Logical. If TRUE and model is SuperSurv, only explains the highest-weighted base learner.
#' @param verbose Logical; if \code{TRUE}, progress messages are shown.
#' @return A data.frame of class \code{c("explain", "data.frame")} containing the calculated SHAP values. The columns correspond to the covariates in \code{X_explain}.
#' @examples
#' if (requireNamespace("fastshap", quietly = TRUE) &&
#'     requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   shap_values <- explain_kernel(
#'     model = fit,
#'     X_explain = X[1:10, , drop = FALSE],
#'     X_background = X[11:40, , drop = FALSE],
#'     nsim = 5
#'   )
#'
#'   dim(shap_values)
#' }
#' @export
explain_kernel <- function(model, X_explain, X_background, nsim = 20,
                           only_best = FALSE,verbose = FALSE) {

  requireNamespace("fastshap", quietly = TRUE)

  # ------------------------------------------------------------------
  # PATH A: The SuperSurv Ensemble
  # ------------------------------------------------------------------
  if (inherits(model, "SuperSurv")) {
    weights <- event_weights(model)

    if (only_best) {
      active_indices <- which.max(weights)
      active_weights <- 1
    } else {
      active_indices <- which(weights > 0.001)
      active_weights <- weights[active_indices]
      active_weights <- active_weights / sum(active_weights) # Re-normalize
    }

    shap_list <- list()
    for (i in seq_along(active_indices)) {
      idx <- active_indices[i]
      model_fit <- model$event.fitLibrary[[idx]]
      weight <- active_weights[i]
      model_name <- names(model$event.fitLibrary)[idx]

      if (isTRUE(verbose)) { message(sprintf(" -> SHAP for %s (Weight: %.3f)", model_name, weight))  }

      s <- fastshap::explain(
        object = model_fit,
        X = X_explain,
        nsim = nsim,
        adjust = TRUE,
        baseline = mean(get_risk_universal(model_fit, X_background)),
        pred_wrapper = function(obj, newdata) { get_risk_universal(obj, newdata) }
      )
      shap_list[[i]] <- as.data.frame(s) * weight
    }

    final_shap <- Reduce("+", shap_list)
    class(final_shap) <- c("explain", "data.frame")
    return(final_shap)

    # ------------------------------------------------------------------
    # PATH B: A Single Base Learner
    # ------------------------------------------------------------------
  } else if (is.list(model) && !is.null(model$fit)) {

    model_to_explain <- model$fit
    if (isTRUE(verbose)) {  message(  " -> Calculating SHAP for single learner of class: ",  class(model_to_explain)[1]    ) }

    s <- fastshap::explain(
      object = model_to_explain,
      X = X_explain,
      nsim = nsim,
      adjust = TRUE,
      baseline = mean(get_risk_universal(model_to_explain, X_background)),
      pred_wrapper = function(obj, newdata) { get_risk_universal(obj, newdata) }
    )

    shap_out <- as.data.frame(s)
    class(shap_out) <- c("explain", "data.frame")
    return(shap_out)

  } else {
    stop("Input must be a fitted 'SuperSurv' object or a valid single learner wrapper output.")
  }
}




#' Create a Time-Dependent Survex Explainer
#'
#' Bridges a fitted SuperSurv ensemble or a single base learner to the
#' \code{survex} package for Time-Dependent SHAP and Model Parts.
#'
#' @param model A fitted SuperSurv object OR a single wrapper output.
#' @param data Covariate data for explanation (data.frame).
#' @param y The survival object (\code{Surv(time, event)}).
#' @param times The time grid for evaluation.
#' @param label Optional character string to name the explainer.
#' @return An explainer object of class \code{survex_explainer} created by
#'   \code{\link[survex]{explain_survival}}, which can be passed to DALEX and survex functions for further model diagnostics and plotting.
#' @examples
#' if (requireNamespace("survex", quietly = TRUE) &&
#'     requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   times <- seq(20, 120, by = 20)
#'   y <- survival::Surv(dat$duration, dat$event)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   explainer <- explain_survex(
#'     model = fit,
#'     data = X,
#'     y = y,
#'     times = times,
#'     label = "SuperSurv_demo"
#'   )
#'
#'   class(explainer)
#' }
#' @export
explain_survex <- function(model, data, y, times, label = NULL) {

  if (!requireNamespace("survex", quietly = TRUE)) stop("Please install the 'survex' package.")

  # 1. Determine Model & Label
  if (inherits(model, "SuperSurv")) {
    model_obj <- model
    if (is.null(label)) label <- "SuperSurv_Ensemble"
  } else if (is.list(model) && !is.null(model$fit)) {
    model_obj <- model
    if (is.null(label)) label <- class(model$fit)[1]
  } else {
    stop("Input must be a fitted 'SuperSurv' object or a valid single learner wrapper output 'surv.*'.")
  }

  # 2. Define the Custom Prediction Wrapper

  custom_predict <- function(mod, newdata, ...) {
    if (inherits(mod, "SuperSurv")) {
      preds <- predict(object = mod, newdata = newdata, new.times = times)
      return(preds$event.predict)
    }

    if (is.list(mod) && !is.null(mod$fit)) {
      return(predict(mod$fit, newdata = newdata, new.times = times))
    }

    stop("Unsupported model type supplied to custom_predict().")
  }

  # 3. Create the Explainer
  explainer <- survex::explain_survival(
    model = model_obj,
    data = as.data.frame(data),
    y = y,
    predict_survival_function = custom_predict,
    times = times,
    label = label,
    verbose = FALSE
  )

  return(explainer)
}





#' Plot Global Feature Importance for SuperSurv
#' @param title Plot title.
#' @param shap_values The output from \code{explain_kernel()}.
#' @param top_n Number of features to show (default 10)
#' @return A \code{ggplot} object visualizing the SHAP values.
#' @examples
#' if (requireNamespace("fastshap", quietly = TRUE) &&
#'     requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   shap_values <- explain_kernel(
#'     model = fit,
#'     X_explain = X[1:10, , drop = FALSE],
#'     X_background = X[11:40, , drop = FALSE],
#'     nsim = 5
#'   )
#'
#'   plot_global_importance(shap_values, top_n = 5)
#' }
#' @export
plot_global_importance <- function(shap_values, title = "SuperSurv: Ensemble Feature Importance", top_n = 10) {

  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("dplyr", quietly = TRUE)

  # 1. Calculate Mean |SHAP|
  # We use drop=FALSE and as.matrix to ensure it works with different data types
  importance <- colMeans(abs(as.matrix(shap_values)))

  # 2. Organize for ggplot
  df_imp <- data.frame(
    Feature = names(importance),
    Importance = as.numeric(importance)
  ) %>%
    dplyr::arrange(desc(Importance)) %>%
    dplyr::slice(1:top_n) # Take top N features

  # 3. Plot
  ggplot2::ggplot(df_imp, ggplot2::aes(x = stats::reorder(Feature, Importance), y = Importance)) +
    ggplot2::geom_col(fill = "steelblue", width = 0.7) +
    ggplot2::coord_flip() +
    ggplot2::labs(
      title = title,
      subtitle = paste("Based on Weighted Ensemble SHAP Values"),
      x = NULL,
      y = "Mean |SHAP| (Impact on Mortality Risk)"
    ) +
    ggplot2::theme_minimal(base_size = 14) +
    ggplot2::theme(panel.grid.minor = ggplot2::element_blank())
}




#' Beeswarm Summary Plot for SuperSurv SHAP
#'
#' @param shap_values The output from \code{explain_kernel()}.
#' @param data The covariate data used (X_explain)
#' @param top_n Number of features to display
#' @return A \code{ggplot} object visualizing the SHAP values.
#' @examples
#' if (requireNamespace("fastshap", quietly = TRUE) &&
#'     requireNamespace("ggforce", quietly = TRUE) &&
#'     requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   shap_values <- explain_kernel(
#'     model = fit,
#'     X_explain = X[1:20, , drop = FALSE],
#'     X_background = X[21:50, , drop = FALSE],
#'     nsim = 5
#'   )
#'
#'   plot_beeswarm(
#'     shap_values = shap_values,
#'     data = X[1:20, , drop = FALSE],
#'     top_n = 5
#'   )
#' }
#' @export
plot_beeswarm <- function(shap_values, data, top_n = 10) {

  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("tidyr", quietly = TRUE)
  requireNamespace("dplyr", quietly = TRUE)
  requireNamespace("ggforce", quietly = TRUE)

  # 1. Convert to standard data frame to avoid tibble issues
  shap_df <- as.data.frame(shap_values)
  data_df <- as.data.frame(data)

  # 2. Calculate overall importance to rank the Y-axis
  # We do this BEFORE pivoting to find the top features
  importance_df <- data.frame(
    Feature = names(shap_df),
    mean_abs = colMeans(abs(shap_df))
  ) %>%
    dplyr::arrange(desc(mean_abs)) %>%
    dplyr::slice(1:min(top_n, ncol(shap_df))) # EXPLICIT CALL

  top_features <- importance_df$Feature

  # 3. Reshape SHAP values to long format (only for top features)
  shap_long <- shap_df[, top_features, drop = FALSE] %>%
    tidyr::pivot_longer(cols = dplyr::everything(),
                        names_to = "Feature",
                        values_to = "SHAP")

  # 4. Reshape original data and scale (0 to 1) for color coding
  # We use a small epsilon (1e-9) to avoid division by zero
  data_long <- data_df[, top_features, drop = FALSE] %>%
    dplyr::mutate(dplyr::across(dplyr::everything(),
                                ~ (. - min(., na.rm=T)) / (max(., na.rm=T) - min(., na.rm=T) + 1e-9))) %>%
    tidyr::pivot_longer(cols = dplyr::everything(),
                        names_to = "Feature",
                        values_to = "FeatureValue")

  # 5. Merge
  df_plot <- data.frame(
    Feature = shap_long$Feature,
    SHAP = shap_long$SHAP,
    FeatureValue = data_long$FeatureValue
  )

  # Ensure the Y-axis follows the importance ranking
  df_plot$Feature <- factor(df_plot$Feature, levels = rev(top_features))

  # 6. Plot
  ggplot2::ggplot(df_plot, ggplot2::aes(x = SHAP, y = Feature, color = FeatureValue)) +
    ggforce::geom_sina(alpha = 0.7, size = 1.8, method = "counts", maxwidth = 0.6) +
    ggplot2::geom_vline(xintercept = 0, color = "black", linetype = "solid", alpha = 0.5) +
    # SHAP standard colors: Blue (Low) to Red (High)
    ggplot2::scale_color_gradient(low = "#1E88E5", high = "#FF0D57",
                                  breaks = c(0, 1), labels = c("Low", "High"),
                                  name = "Feature Value") +
    ggplot2::labs(
      title = "SuperSurv SHAP Summary",
      subtitle = paste("Top", length(top_features), "Features by Mean Absolute SHAP"),
      x = "SHAP Value (Impact on Mortality Risk)",
      y = NULL
    ) +
    ggplot2::theme_minimal(base_size = 14) +
    ggplot2::theme(
      legend.position = "right",
      panel.grid.minor = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_text(face = "bold",size = 16),
      plot.title = ggplot2::element_text(size = 20, face = "bold"),
      axis.title = ggplot2::element_text(size = 18),
      axis.text.x = ggplot2::element_text(size = 16)
    )

}




#' Waterfall Plot for an Individual Patient
#'
#' @param shap_values The output from \code{explain_kernel()}.
#' @param patient_index The row index of the patient to explain
#' @param top_n Number of features to show (default 10)
#' @return A \code{ggplot} object visualizing the SHAP values.
#' @examples
#' if (requireNamespace("fastshap", quietly = TRUE) &&
#'     requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   shap_values <- explain_kernel(
#'     model = fit,
#'     X_explain = X[1:10, , drop = FALSE],
#'     X_background = X[11:40, , drop = FALSE],
#'     nsim = 5
#'   )
#'
#'   plot_patient_waterfall(
#'     shap_values = shap_values,
#'     patient_index = 1,
#'     top_n = 5
#'   )
#' }
#' @export
plot_patient_waterfall <- function(shap_values, patient_index = 1, top_n = 10) {

  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("dplyr", quietly = TRUE)

  # 1. Extract the SHAP values for that specific patient
  row_vals <- as.data.frame(shap_values)[patient_index, ]

  df_p <- data.frame(
    Feature = names(row_vals),
    SHAP = as.numeric(row_vals)
  ) %>%
    dplyr::arrange(desc(abs(SHAP))) %>%
    dplyr::slice(1:top_n) %>%
    dplyr::mutate(Direction = ifelse(SHAP > 0, "Higher Risk", "Lower Risk"))

  # 2. Plot
  ggplot2::ggplot(df_p, ggplot2::aes(x = stats::reorder(Feature, SHAP), y = SHAP, fill = Direction)) +
    ggplot2::geom_col() +
    ggplot2::coord_flip() +
    ggplot2::scale_fill_manual(values = c("Higher Risk" = "#FF0D57", "Lower Risk" = "#1E88E5")) +
    ggplot2::labs(
      title = paste("Patient", patient_index, ": Risk Factor Contribution"),
      subtitle = "How individual features pushed this patient away from the average risk",
      x = NULL,
      y = "SHAP Value (Impact on Risk Score)"
    ) +
    ggplot2::theme_minimal(base_size = 14)
}







#' Plot SHAP Dependence for SuperSurv
#'
#' @param shap_values The output from \code{explain_kernel()}.
#' @param data The original covariate data used for the explanation (X_explain)
#' @param feature_name String name of the column to plot
#' @param title Optional custom title.
#' @return A \code{ggplot} object visualizing the SHAP values.
#' @examples
#' if (requireNamespace("fastshap", quietly = TRUE) &&
#'     requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   shap_values <- explain_kernel(
#'     model = fit,
#'     X_explain = X[1:20, , drop = FALSE],
#'     X_background = X[21:50, , drop = FALSE],
#'     nsim = 5
#'   )
#'
#'   plot_dependence(
#'     shap_values = shap_values,
#'     data = X[1:20, , drop = FALSE],
#'     feature_name = colnames(X)[1]
#'   )
#' }
#' @export
plot_dependence <- function(shap_values, data, feature_name, title = NULL) {

  requireNamespace("ggplot2", quietly = TRUE)
  if(is.null(title)) title <- paste("SHAP Dependence:", feature_name)

  df_plot <- data.frame(
    Feature_Value = data[[feature_name]],
    SHAP_Value = shap_values[[feature_name]]
  )

  ggplot2::ggplot(df_plot, ggplot2::aes(x = Feature_Value, y = SHAP_Value)) +
    # 1. Smooth line in background
    ggplot2::geom_smooth(method = "loess", formula = y ~ x, color = "black",
                         se = TRUE, fill = "gray80", linetype = "dashed") +
    # 2. Points on top
    ggplot2::geom_point(alpha = 0.7, color = "firebrick", size = 2.5) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dotted", color = "gray40") +
    ggplot2::labs(title = title, x = paste(feature_name, "Value"), y = "SHAP Value") +
    ggplot2::theme_minimal(base_size = 14)
}












#' Survival Probability Heatmap
#'
#' @param object A fitted SuperSurv object
#' @param newdata Test covariates (e.g., `X_te[1:50, ]`)
#' @param times The time grid to visualize
#' @return A \code{ggplot} object visualizing the SHAP values.
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   plot_survival_heatmap(
#'     object = fit,
#'     newdata = X[1:20, , drop = FALSE],
#'     times = times
#'   )
#' }
#' @export
plot_survival_heatmap <- function(object, newdata, times) {

  requireNamespace("ggplot2", quietly = TRUE)
  requireNamespace("tidyr", quietly = TRUE)
  requireNamespace("dplyr", quietly = TRUE)

  # 1. Generate Predictions
  preds <- predict(object, newdata = newdata, new.times = times)$event.predict

  # 2. Prepare Data: Sort patients by their final survival probability
  # This makes the heatmap look "organized" (High risk at top, Low risk at bottom)
  order_idx <- order(preds[, ncol(preds)])
  preds_sorted <- preds[order_idx, ]

  df_heatmap <- as.data.frame(preds_sorted)
  colnames(df_heatmap) <- times
  df_heatmap$PatientID <- factor(1:nrow(df_heatmap))

  # 3. Pivot to Long Format
  df_long <- df_heatmap %>%
    tidyr::pivot_longer(cols = -PatientID, names_to = "Time", values_to = "Survival") %>%
    dplyr::mutate(Time = as.numeric(Time))

  ggplot2::ggplot(df_long, ggplot2::aes(x = Time, y = PatientID, fill = Survival)) +
    ggplot2::geom_tile() +
    # Using the 'Plasma' palette to match the purple/red/yellow theme of survex
    ggplot2::scale_fill_viridis_c(option = "plasma", direction = 1, name = "S(t)") +
    ggplot2::labs(
      title = "SuperSurv: Patient Risk Trajectories",
      subtitle = "Sorted by predicted survival at final follow-up",
      x = "Time (Months)",
      y = "Individual Patients"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.y = ggplot2::element_blank(),
      panel.grid = ggplot2::element_blank(),
      # Match the dark purple text from your importance plots
      text = ggplot2::element_text(color = "#371ea3"),
      plot.title = ggplot2::element_text(face = "bold", size = 18)
    )
}

#' Calculate Restricted Mean Survival Time (RMST)
#' @param surv_matrix Matrix of survival probabilities (rows: patients, cols: time points)
#' @param times Vector of time points corresponding to the columns
#' @param tau The restriction time horizon
#' @return A vector of RMST values for each patient
#' @keywords internal
get_rmst <- function(surv_matrix, times, tau) {
  surv_matrix <- as.matrix(surv_matrix)
  ord <- order(times)
  times <- times[ord]
  surv_matrix <- surv_matrix[, ord, drop = FALSE]

  if (tau <= 0) return(rep(0, nrow(surv_matrix)))
  if (tau <= min(times)) return(rep(tau, nrow(surv_matrix)))  # assume S(t)=1 before first grid

  valid_idx <- which(times <= tau)
  t_valid <- times[valid_idx]
  S_valid <- surv_matrix[, valid_idx, drop = FALSE]

  t_calc <- c(0, t_valid, tau)
  dt <- diff(t_calc)

  S_calc <- cbind(1, S_valid, S_valid[, ncol(S_valid), drop = FALSE])
  S_left <- as.matrix(S_calc[, -ncol(S_calc), drop = FALSE])

  rmst <- as.vector(S_left %*% matrix(dt, ncol = 1))
  rmst
}




#' Estimate an Adjusted Marginal RMST Contrast
#'
#' Computes a covariate-adjusted marginal contrast on the restricted mean
#' survival time (RMST) scale using standardization (g-computation) based on a
#' fitted \code{SuperSurv} model.
#'
#' For a binary grouping variable \code{trt_col}, the function predicts
#' counterfactual survival curves under \code{A = 1} and \code{A = 0} for every
#' individual in the supplied dataset, integrates each curve up to the
#' restriction time \code{tau}, and averages the resulting individual-level
#' RMST differences. The resulting contrast is generally interpreted as an
#' adjusted marginal contrast. When \code{trt_col} corresponds to a manipulable
#' intervention and additional identification assumptions hold, the same
#' standardized procedure may also support a causal interpretation.
#'
#' If \code{inference = TRUE}, the function additionally performs a
#' perturbation-based inference procedure conditional on the fitted
#' \code{SuperSurv} model. In this implementation, the fitted learner library,
#' hyperparameters, base learners, and ensemble weights are held fixed, and
#' random positive weights are applied to the individual-level RMST contrasts to
#' estimate a perturbation-based standard error, Wald-type confidence interval,
#' and p-value.
#'
#' @param fit A fitted object of class \code{"SuperSurv"}.
#' @param data A \code{data.frame} containing the covariates used for
#'   standardization, including the binary grouping variable specified by
#'   \code{trt_col}.
#' @param trt_col Character string giving the name of the binary grouping
#'   variable in \code{data}. The variable is set to 1 and 0, respectively, to
#'   generate the two standardized prediction regimes.
#' @param times Numeric vector of prediction time points corresponding to the
#'   evaluation grid used for survival prediction.
#' @param tau Numeric scalar giving the restriction horizon for RMST. Must not
#'   exceed \code{max(times)}.
#' @param inference Logical; if \code{TRUE}, compute perturbation-based standard
#'   errors, confidence intervals, and a Wald-type p-value. Defaults to
#'   \code{FALSE}.
#' @param B Integer giving the number of perturbation replicates when
#'   \code{inference = TRUE}. Defaults to \code{200}.
#' @param seed Optional integer seed for reproducibility of the perturbation
#'   procedure.
#' @param ci_level Numeric scalar in \code{(0, 1)} specifying the confidence
#'   level for the Wald-type confidence interval. Defaults to \code{0.95}.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{ATE_RMST}: The estimated adjusted marginal RMST contrast
#'   \eqn{\widehat{\Delta}_{RMST}(\tau)}.
#'   \item \code{mean_RMST_Treated}: The average predicted RMST under
#'   \code{A = 1}.
#'   \item \code{mean_RMST_Control}: The average predicted RMST under
#'   \code{A = 0}.
#'   \item \code{tau}: The restriction horizon used for integration.
#'   \item \code{patient_rmst_treated}: Vector of individual-level predicted
#'   RMST values under \code{A = 1}.
#'   \item \code{patient_rmst_control}: Vector of individual-level predicted
#'   RMST values under \code{A = 0}.
#'   \item \code{patient_delta_rmst}: Vector of individual-level predicted RMST
#'   contrasts.
#'   \item \code{inference}: Logical indicator for whether perturbation-based
#'   inference was requested.
#'   \item \code{B}: Number of perturbation replicates used when
#'   \code{inference = TRUE}; otherwise \code{NULL}.
#'   \item \code{SE_RMST}: Perturbation-based standard error of the RMST
#'   contrast; otherwise \code{NULL}.
#'   \item \code{CI_RMST}: Wald-type confidence interval for the RMST contrast;
#'   otherwise \code{NULL}.
#'   \item \code{z_value}: Wald-type test statistic; otherwise \code{NULL}.
#'   \item \code{p_value}: Two-sided Wald-type p-value; otherwise \code{NULL}.
#'   \item \code{perturb_reps}: Vector of perturbation replicate estimates;
#'   otherwise \code{NULL}.
#' }
#'
#' @details
#' The function uses the empirical distribution of the observed covariates in
#' \code{data} as the standardization distribution. RMST is evaluated
#' numerically from the predicted survival matrix using a left Riemann sum over
#' the supplied grid \code{times}.
#'
#' The perturbation-based inference implemented here is conditional on the
#' fitted \code{SuperSurv} model. It does not re-tune hyperparameters, reselect
#' the learner library, or refit the base learners under each perturbation.
#' Instead, it perturbs the aggregation of the individual-level predicted RMST
#' contrasts. This yields a lightweight uncertainty quantification procedure for
#' the standardized RMST contrast given the final fitted ensemble.
#'
#' @examples
#' \dontrun{
#' data("metabric", package = "SuperSurv")
#' x_cols <- grep("^x", names(metabric), value = TRUE)
#' X <- metabric[, x_cols]
#' new.times <- seq(10, 150, by = 10)
#'
#' fit <- SuperSurv(
#'   time = metabric$duration,
#'   event = metabric$event,
#'   X = X,
#'   newdata = X,
#'   new.times = new.times,
#'   event.library = c("surv.coxph", "surv.rfsrc"),
#'   cens.library = c("surv.coxph"),
#'   control = list(saveFitLibrary = TRUE),
#'   nFolds = 3
#' )
#'
#' rmst_res <- estimate_marginal_rmst(
#'   fit = fit,
#'   data = metabric,
#'   trt_col = "x4",
#'   times = new.times,
#'   tau = 100,
#'   inference = TRUE,
#'   B = 200,
#'   seed = 123
#' )
#'
#' rmst_res$ATE_RMST
#' rmst_res$SE_RMST
#' rmst_res$CI_RMST
#' format.pval(rmst_res$p_value, digits = 3, eps = 1e-16)
#' }
#'
#' @export
estimate_marginal_rmst <- function(fit, data, trt_col, times, tau,
                                 inference = FALSE, B = 200,
                                 seed = NULL, ci_level = 0.95) {

  if (!inherits(fit, "SuperSurv")) {
    stop("'fit' must be a fitted 'SuperSurv' object.")
  }

  if (!is.data.frame(data)) {
    stop("'data' must be a data.frame.")
  }

  if (!trt_col %in% names(data)) {
    stop("'", trt_col, "' not found in 'data'.")
  }

  if (tau > max(times)) {
    stop("'tau' cannot be greater than the maximum value in 'times'.")
  }

  if (!is.logical(inference) || length(inference) != 1) {
    stop("'inference' must be TRUE or FALSE.")
  }

  if (!is.numeric(B) || length(B) != 1 || B <= 1) {
    stop("'B' must be a single integer greater than 1.")
  }

  if (!is.numeric(ci_level) || length(ci_level) != 1 ||
      ci_level <= 0 || ci_level >= 1) {
    stop("'ci_level' must be a single number in (0, 1).")
  }

  # Use the exact variables the model was trained on.
  model_vars <- training_variables(fit)

  missing_vars <- setdiff(model_vars, names(data))
  if (length(missing_vars) > 0) {
    stop("The following training variables are missing from 'data': ",
         paste(missing_vars, collapse = ", "))
  }

  # -----------------------------
  # Counterfactual predictions: A = 1
  # -----------------------------
  data_trt1 <- data
  data_trt1[[trt_col]] <- 1
  X_trt1 <- data_trt1[, model_vars, drop = FALSE]

  pred_trt1_obj <- predict(fit, newdata = X_trt1, new.times = times)
  surv_trt1 <- pred_trt1_obj$event.predict

  rmst_1 <- get_rmst(surv_matrix = surv_trt1, times = times, tau = tau)

  # -----------------------------
  # Counterfactual predictions: A = 0
  # -----------------------------
  data_trt0 <- data
  data_trt0[[trt_col]] <- 0
  X_trt0 <- data_trt0[, model_vars, drop = FALSE]

  pred_trt0_obj <- predict(fit, newdata = X_trt0, new.times = times)
  surv_trt0 <- pred_trt0_obj$event.predict

  rmst_0 <- get_rmst(surv_matrix = surv_trt0, times = times, tau = tau)

  # -----------------------------
  # Point estimate
  # -----------------------------
  delta_i <- rmst_1 - rmst_0
  ATE <- mean(delta_i)

  # -----------------------------
  # Optional perturbation inference
  # -----------------------------
  perturb_reps <- NULL
  SE_RMST <- NULL
  CI_RMST <- NULL
  p_value <- NULL
  z_value <- NULL

  if (isTRUE(inference)) {
    if (!is.null(seed)) set.seed(seed)

    n <- length(delta_i)
    perturb_reps <- numeric(B)

    for (b in seq_len(B)) {
      # Exponential perturbation weights with mean ~1 after normalization
      w <- stats::rexp(n, rate = 1)
      w <- w / mean(w)

      perturb_reps[b] <- sum(w * delta_i) / sum(w)
    }

    SE_RMST <- stats::sd(perturb_reps)

    alpha <- 1 - ci_level
    zcrit <- stats::qnorm(1 - alpha / 2)

    CI_RMST <- c(
      ATE - zcrit * SE_RMST,
      ATE + zcrit * SE_RMST
    )
    names(CI_RMST) <- c("lower", "upper")

    if (!is.na(SE_RMST) && SE_RMST > 0) {
      z_value <- ATE / SE_RMST
      p_value <- 2 * stats::pnorm(-abs(z_value))
    } else {
      z_value <- NA_real_
      p_value <- NA_real_
    }
  }

  res <- list(
    ATE_RMST = ATE,
    mean_RMST_Treated = mean(rmst_1),
    mean_RMST_Control = mean(rmst_0),
    tau = tau,
    patient_rmst_treated = rmst_1,
    patient_rmst_control = rmst_0,
    patient_delta_rmst = delta_i,
    inference = inference,
    B = if (isTRUE(inference)) B else NULL,
    SE_RMST = SE_RMST,
    CI_RMST = CI_RMST,
    z_value = z_value,
    p_value = p_value,
    # p_value = base::format.pval(p_value, digits = 3, eps = 1e-16),
    perturb_reps = perturb_reps
  )

  msg <- sprintf("Adjusted Delta RMST at tau = %s: %s time units",
                 tau, round(ATE, 3))

  if (isTRUE(inference) && !is.null(SE_RMST)) {
    msg <- paste0(
      msg,
      " | SE = ", round(SE_RMST, 3),
      " | ", round(ci_level * 100), "% CI = [",
      round(CI_RMST[1], 3), ", ", round(CI_RMST[2], 3), "]"
    )
  }

  message(msg)

  return(res)
}








#' Plot Adjusted Marginal RMST Contrast Over Time
#'
#' Generates a curve showing how the adjusted marginal restricted mean survival
#' time (RMST) contrast evolves across a sequence of restriction times.
#'
#' If \code{inference = TRUE}, the function additionally displays perturbation-based
#' Wald confidence intervals at each value of \code{tau}.
#'
#' @param fit A fitted \code{SuperSurv} ensemble object.
#' @param data A \code{data.frame} containing the covariates and the binary grouping variable.
#' @param trt_col Character string. The exact name of the binary grouping variable in \code{data}.
#' @param times Numeric vector of time points matching the prediction grid.
#' @param tau_seq Numeric vector. A sequence of restriction times (\code{tau}) to evaluate and plot.
#' @param inference Logical; if \code{TRUE}, compute perturbation-based confidence intervals.
#'   Defaults to \code{FALSE}.
#' @param B Integer. Number of perturbation replicates used when \code{inference = TRUE}.
#'   Defaults to \code{200}.
#' @param seed Optional integer seed for reproducibility.
#' @param ci_level Numeric scalar in \code{(0,1)} specifying the confidence level for the
#'   confidence interval. Defaults to \code{0.95}.
#'
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:80, ]
#' x_cols <- grep("^x", names(dat), value = TRUE)[1:5]
#' X <- dat[, x_cols, drop = FALSE]
#' new.times <- seq(20, 120, by = 20)
#'
#' fit <- SuperSurv(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = X,
#'   new.times = new.times,
#'   event.library = c("surv.coxph", "surv.glmnet"),
#'   cens.library = c("surv.coxph"),
#'   control = list(saveFitLibrary = TRUE)
#' )
#'
#' tau_grid <- seq(40, 120, by = 20)
#' plot_marginal_rmst_curve(
#'   fit = fit,
#'   data = dat,
#'   trt_col = "x4",
#'   times = new.times,
#'   tau_seq = tau_grid,
#'   inference = TRUE,
#'   B = 100,
#'   seed = 123
#' )
#'
#' @return A \code{ggplot} object visualizing the adjusted marginal RMST contrast curve.
#' @export
plot_marginal_rmst_curve <- function(fit, data, trt_col, times, tau_seq,
                                     inference = FALSE, B = 200,
                                     seed = NULL, ci_level = 0.95) {
  requireNamespace("ggplot2", quietly = TRUE)

  results <- lapply(seq_along(tau_seq), function(i) {
    t <- tau_seq[i]

    # optional varying seed across tau values for reproducibility
    seed_i <- if (!is.null(seed)) seed + i - 1 else NULL

    res <- estimate_marginal_rmst(
      fit = fit,
      data = data,
      trt_col = trt_col,
      times = times,
      tau = t,
      inference = inference,
      B = B,
      seed = seed_i,
      ci_level = ci_level
    )

    out <- data.frame(
      Tau = t,
      Delta_RMST = if (!is.null(res$Delta_RMST)) res$Delta_RMST else res$ATE_RMST,
      stringsAsFactors = FALSE
    )

    if (isTRUE(inference)) {
      out$SE_RMST <- res$SE_RMST
      out$lower <- res$CI_RMST["lower"]
      out$upper <- res$CI_RMST["upper"]
      out$p_value <- res$p_value
    }

    out
  })

  res_df <- do.call(rbind, results)

  p <- ggplot2::ggplot(res_df, ggplot2::aes(x = Tau, y = Delta_RMST))

  if (isTRUE(inference)) {
    p <- p +
      ggplot2::geom_ribbon(
        ggplot2::aes(ymin = lower, ymax = upper),
        alpha = 0.4,
        fill = "#e63946"
      )
  }

  p <- p +
    ggplot2::geom_line(color = "#e63946", linewidth = 1) +
    ggplot2::geom_point(color = "#1d3557", size = 2) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black") +
    ggplot2::theme_minimal() +
    ggplot2::labs(
      title = "Adjusted Marginal RMST Contrast Over Time",
      subtitle = if (isTRUE(inference)) {
        paste0("Difference in RMST with ", round(ci_level * 100), "% perturbation-based confidence intervals")
      } else {
        "Difference in Restricted Mean Survival Time between groups"
      },
      x = "Restriction Time (Tau)",
      y = expression(Delta ~ "RMST")
    ) +
    ggplot2::theme(
      legend.position = "right",
      panel.grid.minor = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_text(face = "bold", size = 16),
      plot.title = ggplot2::element_text(size = 20, face = "bold"),
      axis.title = ggplot2::element_text(size = 18),
      axis.text.x = ggplot2::element_text(size = 16)
    )

  return(p)
}


#' Plot Predicted RMST vs. Observed Survival Times
#'
#' Evaluates the calibration of the causal RMST estimator by plotting the model's
#' predicted RMST for each patient against their actual observed follow-up time.
#'
#' @param fit A fitted \code{SuperSurv} ensemble object.
#' @param data A \code{data.frame} containing the patient covariates, times, and events.
#' @param time_col Character string. The exact name of the observed follow-up time column in \code{data}.
#' @param event_col Character string. The exact name of the event indicator column in \code{data} (e.g., 1 for event, 0 for censored).
#' @param times Numeric vector of time points matching the prediction grid.
#' @param tau Numeric. A single truncation time limit up to which the RMST is calculated.
#'
#' @return A \code{ggplot} object comparing predicted RMST to observed outcomes.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:80, ]
#' x_cols <- grep("^x", names(dat))[1:5]
#' X <- dat[, x_cols, drop = FALSE]
#' new.times <- seq(20, 120, by = 20)
#'
#' fit <- SuperSurv(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = X,
#'   new.times = new.times,
#'   event.library = c("surv.coxph", "surv.glmnet"),
#'   cens.library = c("surv.coxph"),
#'   control = list(saveFitLibrary = TRUE)
#' )
#'
#' plot_rmst_vs_obs(
#'   fit = fit,
#'   data = dat,
#'   time_col = "duration",
#'   event_col = "event",
#'   times = new.times,
#'   tau = 350
#' )
#' @export
plot_rmst_vs_obs <- function(fit, data, time_col, event_col, times, tau) {
  requireNamespace("ggplot2", quietly = TRUE)
  model_vars <- training_variables(fit)
  X_obs <- data[, model_vars, drop = FALSE]

  # Using the standardized 'newdata' argument
  pred_obs_obj <- predict(fit, newdata = X_obs, new.times = times)
  surv_obs <- pred_obs_obj$event.predict
  rmst_obs <- get_rmst(surv_obs, times, tau)

  plot_df <- data.frame(
    Observed_Time = data[[time_col]],
    Predicted_RMST = rmst_obs,
    Event = as.factor(data[[event_col]])
  )

  ggplot2::ggplot(plot_df, ggplot2::aes(x = Observed_Time, y = Predicted_RMST, color = Event)) +
    ggplot2::geom_point(alpha = 0.6, size = 2) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "black") +
    ggplot2::geom_hline(yintercept = tau, linetype = "dotted", color = "red") +
    ggplot2::theme_minimal() +
    ggplot2::scale_color_manual(values = c("0" = "#457b9d", "1" = "#e63946"),
                                labels = c("0" = "Censored", "1" = "Event")) +
    ggplot2::labs(title = sprintf("Predicted RMST vs. Observed Time (Tau = %s)", tau),
                  x = "Observed Survival Time", y = "Predicted RMST", color = "Status") +
   ggplot2::theme(
    legend.position = "right",
    panel.grid.minor = ggplot2::element_blank(),
    axis.text.y = ggplot2::element_text(face = "bold",size = 16),
    plot.title = ggplot2::element_text(size = 20, face = "bold"),
    axis.title = ggplot2::element_text(size = 18),
    axis.text.x = ggplot2::element_text(size = 16)
  )
}

#' Extract SuperSurv ensemble coefficients
#'
#' Returns the optimized convex-combination weights from a fitted
#' \code{SuperSurv} object.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param type Character string specifying which weights to return. Use
#'   \code{"event"} for the event-time ensemble, \code{"censoring"} for the
#'   censoring ensemble, or \code{"both"} for both sets of weights.
#' @param ... Additional arguments ignored.
#'
#' @return For \code{type = "event"} or \code{type = "censoring"}, a named
#'   numeric vector of ensemble weights. For \code{type = "both"}, a list with
#'   elements \code{event} and \code{censoring}.
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   coef(fit)
#'   coef(fit, type = "both")
#' }
#' @method coef SuperSurv
#' @export
coef.SuperSurv <- function(object, type = c("event", "censoring", "both"), ...) {
  type <- match.arg(type)

  switch(
    type,
    event = object$event.coef,
    censoring = object$cens.coef,
    both = list(
      event = object$event.coef,
      censoring = object$cens.coef
    )
  )
}

#' Access SuperSurv ensemble weights
#'
#' Extracts the fitted event or censoring ensemble weights from a
#' \code{SuperSurv} object.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param ... Additional arguments ignored.
#'
#' @return A named numeric vector of ensemble weights.
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   event_weights(fit)
#'   censor_weights(fit)
#' }
#' @export
event_weights <- function(object, ...) {
  UseMethod("event_weights")
}

#' @rdname event_weights
#' @method event_weights SuperSurv
#' @export
event_weights.SuperSurv <- function(object, ...) {
  coef(object, type = "event")
}

#' @rdname event_weights
#' @export
censor_weights <- function(object, ...) {
  UseMethod("censor_weights")
}

#' @rdname event_weights
#' @method censor_weights SuperSurv
#' @export
censor_weights.SuperSurv <- function(object, ...) {
  coef(object, type = "censoring")
}

#' Access SuperSurv learner names
#'
#' Returns the fitted learner names from a \code{SuperSurv} object.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param type Character string specifying whether to return event learner
#'   names, censoring learner names, or both.
#' @param ... Additional arguments ignored.
#'
#' @return For \code{type = "event"} or \code{type = "censoring"}, a character
#'   vector of learner names. For \code{type = "both"}, a list with elements
#'   \code{event} and \code{censoring}.
#'
#' @export
learner_names <- function(object, ...) {
  UseMethod("learner_names")
}

#' @rdname learner_names
#' @method learner_names SuperSurv
#' @export
learner_names.SuperSurv <- function(object,
                                    type = c("event", "censoring", "both"),
                                    ...) {
  type <- match.arg(type)

  event_names <- names(event_weights(object))
  censor_names <- names(censor_weights(object))

  switch(
    type,
    event = event_names,
    censoring = censor_names,
    both = list(event = event_names, censoring = censor_names)
  )
}

#' Access SuperSurv prediction evaluation times
#'
#' Returns the time grid used for the fitted \code{SuperSurv} predictions.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param ... Additional arguments ignored.
#'
#' @return A numeric vector of prediction evaluation times.
#'
#' @export
eval_times <- function(object, ...) {
  UseMethod("eval_times")
}

#' @rdname eval_times
#' @method eval_times SuperSurv
#' @export
eval_times.SuperSurv <- function(object, ...) {
  times <- object$eval.times
  if (is.null(times)) {
    pred_dimnames <- dimnames(object$event.predict)
    times <- suppressWarnings(as.numeric(pred_dimnames[[2L]]))
  }
  if (is.null(times) || length(times) == 0L || anyNA(times)) {
    stop("Prediction evaluation times are not available in this SuperSurv object.",
         call. = FALSE)
  }

  times
}

#' Access SuperSurv training variable names
#'
#' Returns the covariate names used to fit a \code{SuperSurv} object.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param ... Additional arguments ignored.
#'
#' @return A character vector of training variable names.
#'
#' @export
training_variables <- function(object, ...) {
  UseMethod("training_variables")
}

#' @rdname training_variables
#' @method training_variables SuperSurv
#' @export
training_variables.SuperSurv <- function(object, ...) {
  vars <- object$varNames
  if (is.null(vars)) {
    stop("Training variable names are not available in this SuperSurv object.", call. = FALSE)
  }

  vars
}

#' Access variables selected by SuperSurv screeners
#'
#' Returns the variables retained by a screening step for one or more fitted
#' event or censoring learners.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param type Character string specifying whether to inspect event or censoring
#'   learners.
#' @param learner Optional learner index or learner name. If omitted, selected
#'   variables are returned for every learner of the requested type.
#' @param ... Additional arguments ignored.
#'
#' @return A named list of character vectors, or a single character vector when
#'   \code{learner} has length one.
#'
#' @export
selected_variables <- function(object, ...) {
  UseMethod("selected_variables")
}

#' @rdname selected_variables
#' @method selected_variables SuperSurv
#' @export
selected_variables.SuperSurv <- function(object,
                                         type = c("event", "censoring"),
                                         learner = NULL,
                                         ...) {
  type <- match.arg(type)
  vars <- training_variables(object)

  screen <- switch(
    type,
    event = object$event.whichScreen,
    censoring = object$cens.whichScreen
  )

  names_by_learner <- learner_names(object, type = type)
  screen <- .SuperSurv_screen_by_learner(object, screen, type)

  if (is.null(learner)) {
    learner_idx <- seq_along(names_by_learner)
  } else if (is.character(learner)) {
    learner_idx <- match(learner, names_by_learner)
    if (anyNA(learner_idx)) {
      stop("Unknown learner name: ",
           paste(learner[is.na(learner_idx)], collapse = ", "),
           call. = FALSE)
    }
  } else {
    learner_idx <- as.integer(learner)
    if (anyNA(learner_idx) || any(learner_idx < 1L) ||
        any(learner_idx > length(names_by_learner))) {
      stop("'learner' must be a valid learner index or name.", call. = FALSE)
    }
  }

  out <- lapply(learner_idx, function(i) vars[as.logical(screen[i, ])])
  names(out) <- names_by_learner[learner_idx]

  if (length(out) == 1L) out[[1L]] else out
}

#' Print a SuperSurv fit
#'
#' Prints a concise description of a fitted \code{SuperSurv} object.
#'
#' @param x A fitted object of class \code{"SuperSurv"}.
#' @param digits Number of significant digits to use for displayed weights.
#' @param ... Additional arguments ignored.
#'
#' @return The input object \code{x}, invisibly.
#'
#' @method print SuperSurv
#' @export
print.SuperSurv <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  cat("SuperSurv fit\n")
  cat("  Selection:", .SuperSurv_selection(x), "\n")
  cat("  Event learners:", .SuperSurv_n_learners(x$event.libraryNames), "\n")
  cat("  Censoring learners:", .SuperSurv_n_learners(x$cens.libraryNames), "\n")

  pred_dim <- dim(x$event.predict)
  if (length(pred_dim) == 2L) {
    cat("  Predictions:", pred_dim[1L], "observations x", pred_dim[2L], "times\n")
  }
  cat("  Evaluation times:", .SuperSurv_range_label(eval_times(x)), "\n")

  event_weights <- event_weights(x)
  active <- event_weights[event_weights > 0]
  if (length(active) > 0L) {
    active <- sort(active, decreasing = TRUE)
    cat("  Nonzero event weights:\n")
    print(round(active, digits))
  } else {
    cat("  Nonzero event weights: none\n")
  }

  invisible(x)
}

#' Summarize a SuperSurv fit
#'
#' Summarizes the fitted event and censoring ensembles, including learner names,
#' ensemble weights, cross-validated risks, error flags, prediction dimensions,
#' and recorded timing information.
#'
#' @param object A fitted object of class \code{"SuperSurv"}.
#' @param x A summary object produced by \code{summary.SuperSurv()}.
#' @param digits Number of significant digits to use for displayed weights and
#'   risks.
#' @param ... Additional arguments ignored.
#'
#' @return An object of class \code{"summary.SuperSurv"}, a list containing the
#'   matched call, selection mode, event and censoring learner summaries,
#'   prediction dimensions, and timing information.
#'
#' @method summary SuperSurv
#' @export
summary.SuperSurv <- function(object, ...) {
  out <- list(
    call = object$call,
    selection = .SuperSurv_selection(object),
    event = .SuperSurv_learner_summary(
      learner_names = object$event.libraryNames,
      weights = event_weights(object),
      cv_risks = object$event.cvRisks,
      cv_errors = object$event.errorsInCVLibrary,
      fit_errors = object$event.errorsInLibrary
    ),
    censoring = .SuperSurv_learner_summary(
      learner_names = object$cens.libraryNames,
      weights = censor_weights(object),
      cv_risks = object$cens.cvRisks,
      cv_errors = object$cens.errorsInCVLibrary,
      fit_errors = object$cens.errorsInLibrary
    ),
    prediction_dimensions = list(
      event = dim(object$event.predict),
      censoring = dim(object$cens.predict),
      event_library = dim(object$event.library.predict),
      censoring_library = dim(object$cens.library.predict)
    ),
    eval_times = eval_times(object),
    times = object$times
  )

  class(out) <- "summary.SuperSurv"
  out
}

#' @rdname summary.SuperSurv
#' @method print summary.SuperSurv
#' @export
print.summary.SuperSurv <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {
  cat("Summary of SuperSurv fit\n")
  cat("  Selection:", x$selection, "\n\n")

  cat("Call:\n")
  print(x$call)
  cat("\n")

  cat("Event ensemble:\n")
  print(.SuperSurv_format_summary_table(x$event, digits = digits), row.names = FALSE)
  cat("\nCensoring ensemble:\n")
  print(.SuperSurv_format_summary_table(x$censoring, digits = digits), row.names = FALSE)

  event_dim <- x$prediction_dimensions$event
  if (length(event_dim) == 2L) {
    cat("\nPredictions:", event_dim[1L], "observations x", event_dim[2L], "times\n")
  }
  if (!is.null(x$eval_times)) {
    cat("Evaluation times:", .SuperSurv_range_label(x$eval_times), "\n")
  }

  if (!is.null(x$times)) {
    elapsed <- vapply(x$times, function(z) unname(z[["elapsed"]]), numeric(1L))
    cat("Elapsed time (seconds):\n")
    print(round(elapsed, digits))
  }

  invisible(x)
}

.SuperSurv_selection <- function(object) {
  if (!is.null(object$call) && !is.null(object$call$selection)) {
    selection <- object$call$selection
    if (is.character(selection)) return(selection)
    return(paste(deparse(selection), collapse = " "))
  }

  "ensemble"
}

.SuperSurv_n_learners <- function(learner_names) {
  n <- nrow(learner_names)
  if (is.null(n)) length(learner_names) else n
}

.SuperSurv_learner_summary <- function(learner_names, weights, cv_risks,
                                       cv_errors, fit_errors) {
  learner_names <- as.data.frame(learner_names, stringsAsFactors = FALSE)
  if (!all(c("predAlgorithm", "screenAlgorithm") %in% names(learner_names))) {
    names(learner_names)[seq_len(min(2L, ncol(learner_names)))] <-
      c("predAlgorithm", "screenAlgorithm")[seq_len(min(2L, ncol(learner_names)))]
  }

  out <- data.frame(
    learner = names(weights),
    predAlgorithm = learner_names$predAlgorithm,
    screenAlgorithm = learner_names$screenAlgorithm,
    weight = unname(weights),
    risk = unname(cv_risks),
    status = .SuperSurv_learner_status(cv_errors, fit_errors),
    cv_error = as.logical(cv_errors),
    fit_error = as.logical(fit_errors),
    stringsAsFactors = FALSE
  )

  out[order(out$risk, decreasing = FALSE, na.last = TRUE), , drop = FALSE]
}

.SuperSurv_format_summary_table <- function(x, digits) {
  x$weight <- round(x$weight, digits)
  x$risk <- round(x$risk, digits)
  x[, c("learner", "weight", "risk", "status"), drop = FALSE]
}

.SuperSurv_range_label <- function(x) {
  if (length(x) == 1L) return(format(x))
  paste0(length(x), " values from ", format(min(x)), " to ", format(max(x)))
}

.SuperSurv_learner_status <- function(cv_errors, fit_errors) {
  status <- rep("ok", length(cv_errors))
  status[as.logical(cv_errors)] <- "CV error"
  status[as.logical(fit_errors)] <- "fit error"
  status[as.logical(cv_errors) & as.logical(fit_errors)] <- "CV and fit error"
  status
}

.SuperSurv_screen_by_learner <- function(object, screen, type) {
  if (is.null(dim(screen))) {
    screen <- matrix(as.logical(screen), nrow = 1L)
  }

  n_learners <- length(learner_names(object, type = type))
  if (nrow(screen) == n_learners) {
    return(screen)
  }

  library_rows <- switch(
    type,
    event = object$event.library$library$rowScreen,
    censoring = object$cens.library$library$rowScreen
  )

  screen[library_rows, , drop = FALSE]
}

#' METABRIC Breast Cancer Dataset
#'
#' A subset of the Molecular Taxonomy of Breast Cancer International Consortium
#' (METABRIC) dataset to demonstrate the SuperSurv package. 9 Covariates: 4 gene indicators (MKI67, EGFR, PGR, and ERBB2) and 5 clinical features (age at diagnosis, and indicators for hormone treatment, radiotherapy, and chemotherapy)
#'
#' @format A data frame with the clinical and genomic variables:
#' \describe{
#'   \item{duration}{Survival time.}
#'   \item{event}{Event indicator (1 = event, 0 = censored).}
#'   \item{x0}{Feature x0.}
#'   \item{x1}{Feature x1.}
#'   \item{x2}{Feature x2.}
#'   \item{x3}{Feature x3.}
#'   \item{x4}{Feature x4.}
#'   \item{x5}{Feature x5.}
#'   \item{x6}{Feature x6.}
#'   \item{x7}{Feature x7.}
#'   \item{x8}{Feature x8.}
#' }
"metabric"

#' IPCW Brier Score and Integrated Brier Score (IBS)
#'
#' Calculates the Inverse Probability of Censoring Weighted (IPCW) Brier Score
#' over a grid of times, and computes the Integrated Brier Score (IBS) using
#' trapezoidal integration.
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (1 = event, 0 = censored).
#' @param S_mat A numeric matrix of predicted survival probabilities
#'   (rows = observations, columns = time points).
#' @param times Numeric vector of evaluation times matching the columns of \code{S_mat}.
#' @param tmin Numeric. Lower bound for IBS integration. Defaults to \code{min(times)}.
#' @param tmax Numeric. Upper bound for IBS integration. Defaults to \code{max(times)}.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{brier_scores}: A numeric vector of Brier scores at each time point.
#'   \item \code{ibs}: The Integrated Brier Score over the range `\code{tmin}, \code{tmax}`.
#'   \item \code{times}: The time grid used.
#' }
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:40, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:10, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.coxph(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' eval_brier(
#'   time = dat$duration[1:10],
#'   event = dat$event[1:10],
#'   S_mat = fit[["pred"]],
#'   times = times
#' )
#' @export
eval_brier <- function(time, event, S_mat, times, tmin = min(times), tmax = max(times)) {

  n <- length(time)
  T_len <- length(times)
  S_mat <- as.matrix(S_mat)

  if (nrow(S_mat) != n || ncol(S_mat) != T_len) {
    stop("S_mat dimensions (", nrow(S_mat), "x", ncol(S_mat),
         ") do not match n (", n, ") and length of times (", T_len, ").")
  }

  # Clamp probabilities to avoid exact 0/1 floating issues
  epsProb <- 1e-6
  epsG <- 1e-6
  S_mat[S_mat < epsProb] <- epsProb
  S_mat[S_mat > 1 - epsProb] <- 1 - epsProb

  # Helper: KM Censoring Survival G(t)
  cens <- 1 - event
  fit_G <- survival::survfit(survival::Surv(time, cens) ~ 1)
  t_fit <- fit_G$time
  s_fit <- fit_G$surv

  Gfun <- function(t) {
    idx <- findInterval(t, t_fit, rightmost.closed = TRUE)
    out <- ifelse(idx == 0, 1, s_fit[idx])
    pmax(out, epsG)
  }

  # Calculate Brier Score at each time t
  bs <- numeric(T_len)
  G_Ti <- Gfun(time) # G evaluated at each observed time

  for (j in seq_len(T_len)) {
    t <- times[j]
    S_t <- S_mat[, j]

    Y <- as.integer(time > t)
    G_t <- Gfun(t)

    # Graf IPCW Weights
    w <- ifelse(time <= t & event == 1, 1 / G_Ti,
                ifelse(time > t, 1 / G_t, 0))

    bs[j] <- mean(w * (Y - S_t)^2)
  }

  # Calculate Integrated Brier Score (IBS) via Trapezoidal Rule
  valid_idx <- which(times >= tmin & times <= tmax)
  if (length(valid_idx) < 2) {
    ibs <- NA
  } else {
    t_sub <- times[valid_idx]
    b_sub <- bs[valid_idx]
    diffs <- diff(t_sub)
    heights <- (b_sub[-1] + b_sub[-length(b_sub)]) / 2
    ibs <- sum(diffs * heights) / (max(t_sub) - min(t_sub))
  }

  return(list(brier_scores = bs, ibs = ibs, times = times))
}






#' Calculate Concordance Index (Harrell's or Uno's)
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (1 = event, 0 = censored).
#' @param S_mat A numeric matrix of predicted survival probabilities.
#' @param times Numeric vector of evaluation times matching the columns of \code{S_mat}.
#' @param eval_time Numeric. The specific time point at which to extract predictions.
#' @param method Character. Either "harrell" or "uno". Defaults to "uno".
#'
#' @return A numeric value representing the chosen C-index.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:40, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:10, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.coxph(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' eval_cindex(
#'   time = dat$duration[1:10],
#'   event = dat$event[1:10],
#'   S_mat = fit[["pred"]],
#'   times = times,
#'   eval_time = 100,
#'   method = "uno"
#' )
#' @export
eval_cindex <- function(time, event, S_mat, times, eval_time, method = "uno") {

  requireNamespace("survival", quietly = TRUE)

  # Find the closest time column in the matrix
  t_idx <- which.min(abs(times - eval_time))

  # survival::concordance expects a "survival" score (higher = lives longer)
  # So we pass the survival probability DIRECTLY.
  surv_score <- S_mat[, t_idx]

  if (method == "uno") {
    # timewt = "n/G2" applies Uno's IPCW correction
    cfit <- survival::concordance(survival::Surv(time, event) ~ surv_score, timewt = "n/G2")
  } else {
    # Default Harrell's C
    cfit <- survival::concordance(survival::Surv(time, event) ~ surv_score)
  }

  return(cfit$concordance)
}






#' Time-Dependent AUC and Integrated AUC
#'
#' Evaluates the cumulative/dynamic time-dependent AUC and integrated AUC (iAUC)
#' using inverse probability of censoring weighting (IPCW).
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (1 = event, 0 = censored).
#' @param S_mat A numeric matrix of predicted survival probabilities.
#' @param times Numeric vector of evaluation times matching the columns of \code{S_mat}.
#'
#' @return A list containing the \code{AUC_curve} at each time point, the
#'   \code{times}, and the integrated AUC \code{iAUC}.
#' @examples
#'  data("metabric", package = "SuperSurv")
#'  dat <- metabric[1:40, ]
#'  x_cols <- grep("^x", names(dat))[1:3]
#'  X <- dat[, x_cols, drop = FALSE]
#'  newX <- X[1:10, , drop = FALSE]
#'  times <- seq(50, 150, by = 50)
#'
#'  fit <- surv.coxph(
#'    time = dat$duration,
#'    event = dat$event,
#'    X = X,
#'    newdata = newX,
#'    new.times = times,
#'    obsWeights = rep(1, nrow(dat)),
#'    id = NULL
#'  )
#'
#'  eval_timeROC(
#'    time = dat$duration[1:10],
#'    event = dat$event[1:10],
#'    S_mat = fit[["pred"]],
#'    times = times
#'  )
#' @export
eval_timeROC <- function(time, event, S_mat, times) {

  time <- as.numeric(time)
  event <- as.numeric(event)
  S_mat <- as.matrix(S_mat)
  times <- as.numeric(times)

  n <- length(time)
  if (nrow(S_mat) != n) {
    stop("Number of rows in S_mat must equal length(time).")
  }
  if (ncol(S_mat) != length(times)) {
    stop("Number of columns in S_mat must equal length(times).")
  }

  # Clamp probabilities
  eps_prob <- 1e-6
  S_mat[S_mat < eps_prob] <- eps_prob
  S_mat[S_mat > 1 - eps_prob] <- 1 - eps_prob

  # Estimate censoring survival G(t) using KM for censoring
  cens <- 1 - event
  fit_G <- survival::survfit(survival::Surv(time, cens) ~ 1)
  t_fit <- fit_G$time
  s_fit <- fit_G$surv
  eps_G <- 1e-6

  Gfun <- function(t) {
    idx <- findInterval(t, t_fit, rightmost.closed = TRUE)
    out <- ifelse(idx == 0, 1, s_fit[idx])
    pmax(out, eps_G)
  }

  auc_vals <- rep(NA_real_, length(times))

  for (j in seq_along(times)) {
    t0 <- times[j]

    # Use risk at the same evaluation time
    risk_score <- 1 - S_mat[, j]

    # Cumulative/dynamic definition
    is_case <- (time <= t0 & event == 1)
    is_ctrl <- (time > t0)

    if (sum(is_case) == 0 || sum(is_ctrl) == 0) {
      auc_vals[j] <- NA_real_
      next
    }

    # IPCW weights
    w_case <- rep(0, n)
    w_ctrl <- rep(0, n)

    w_case[is_case] <- 1 / Gfun(time[is_case])
    w_ctrl[is_ctrl] <- 1 / Gfun(t0)

    case_idx <- which(is_case)
    ctrl_idx <- which(is_ctrl)

    # Weighted pairwise AUC:
    # P(score_case > score_ctrl) + 0.5 P(tie)
    num <- 0
    den <- 0

    for (ii in case_idx) {
      for (jj in ctrl_idx) {
        wij <- w_case[ii] * w_ctrl[jj]
        den <- den + wij

        if (risk_score[ii] > risk_score[jj]) {
          num <- num + wij
        } else if (risk_score[ii] == risk_score[jj]) {
          num <- num + 0.5 * wij
        }
      }
    }

    auc_vals[j] <- if (den > 0) num / den else NA_real_
  }

  valid_idx <- which(!is.na(auc_vals))
  if (length(valid_idx) < 2) {
    iauc <- NA_real_
  } else {
    t_sub <- times[valid_idx]
    a_sub <- auc_vals[valid_idx]
    diffs <- diff(t_sub)
    heights <- (a_sub[-1] + a_sub[-length(a_sub)]) / 2
    iauc <- sum(diffs * heights) / (max(t_sub) - min(t_sub))
  }

  list(
    AUC_curve = auc_vals,
    times = times,
    iAUC = iauc
  )
}





#' Evaluate SuperSurv predictions on test data
#'
#' Computes the integrated Brier score (IBS), Uno C-index, and integrated area
#' under the curve (iAUC) for the SuperSurv ensemble and all individual base
#' learners.
#'
#' @param object A fitted \code{SuperSurv} object.
#' @param newdata A data.frame of test covariates.
#' @param time Numeric vector of observed follow-up times for the test set.
#' @param event Numeric vector of event indicators for the test set.
#' @param eval_times Numeric vector of times at which to evaluate survival
#'   predictions.
#' @param risk_time Numeric. The specific time horizon used when extracting risk
#'   scores for Uno C-index. Defaults to the median of \code{eval_times}.
#' @param verbose Logical; if \code{TRUE}, progress messages are shown.
#'
#' @return An object of class \code{"SuperSurv_eval"} containing benchmark
#'   metrics for the ensemble and base learners.
#'
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:80, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#'
#' fit <- SuperSurv(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = dat[, x_cols, drop = FALSE],
#'   new.times = seq(50, 200, by = 50),
#'   event.library = c("surv.coxph", "surv.km"),
#'   cens.library = c("surv.coxph", "surv.km")
#' )
#'
#' res <- eval_summary(
#'   object = fit,
#'   newdata = dat[, x_cols, drop = FALSE],
#'   time = dat$duration,
#'   event = dat$event,
#'   eval_times = seq(50, 200, by = 50)
#' )
#'
#' res
#'
#' @export
eval_summary <- function(object, newdata, time, event, eval_times,
                         risk_time = stats::median(eval_times),
                         verbose = FALSE) {

  if (isTRUE(verbose)) {
    message("Generating predictions on test data...")
  }
  preds <- predict(object, newdata = newdata, new.times = eval_times)

  k_models <- dim(preds$event.library.predict)[3]
  model_names <- dimnames(preds$event.library.predict)[[3]]

  if (is.null(model_names)) {
    model_names <- paste0("Base_Learner_", seq_len(k_models))
  }

  all_names <- c("SuperSurv_Ensemble", model_names)

  results <- data.frame(
    Model = all_names,
    IBS = numeric(length(all_names)),
    Uno_C = numeric(length(all_names)),
    iAUC = numeric(length(all_names)),
    stringsAsFactors = FALSE
  )

  process_matrix <- function(S_mat) {
    ibs_res <- eval_brier(time, event, S_mat, eval_times)$ibs

    c_res <- eval_cindex(
      time,
      event,
      S_mat,
      eval_times,
      eval_time = risk_time,
      method = "uno"
    )

    iauc_res <- suppressMessages(
      eval_timeROC(time, event, S_mat, eval_times)$iAUC
    )

    c(ibs_res, c_res, iauc_res)
  }

  if (isTRUE(verbose)) {
    message("Evaluating SuperSurv ensemble...")
  }
  results[1, 2:4] <- process_matrix(preds$event.predict)

  for (i in seq_len(k_models)) {
    if (isTRUE(verbose)) {
      message(sprintf(
        "Evaluating base learner %d/%d: %s",
        i, k_models, model_names[i]
      ))
    }
    results[i + 1, 2:4] <- process_matrix(preds$event.library.predict[, , i])
  }

  results$IBS <- round(results$IBS, 4)
  results$Uno_C <- round(results$Uno_C, 4)
  results$iAUC <- round(results$iAUC, 4)

  attr(results, "eval_times") <- eval_times
  attr(results, "risk_time") <- risk_time
  class(results) <- c("SuperSurv_eval", class(results))

  results
}

# 1. Appease CRAN checks for unquoted variables in ggplot2 and dplyr
# 1. Appease CRAN checks for unquoted variables in ggplot2 and dplyr
utils::globalVariables(c(
  "Brier", "CD_AUC", "Direction", "Feature", "FeatureValue",
  "Feature_Value", "Importance", "Model", "PatientID", "SHAP",
  "SHAP_Value", "Survival", "Time", "mean_abs", "var",
  "Delta_RMST", "lower", "upper"
))

# 2. Explicitly import base R stats functions and dplyr/magrittr operators
#' @importFrom stats as.formula coef median predict quantile var
#' @importFrom dplyr desc
#' @importFrom magrittr %>%
NULL

#' Wrapper function for Random Survival Forests (RFSRC)
#'
#' Final Production Wrapper for RFSRC (Tunable & Robust).
#' Estimates a survival random forest using \code{\link[randomForestSRC]{rfsrc}}.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Currently ignored.
#' @param ntree Number of trees to grow (default: 1000).
#' @param nodesize Minimum number of deaths in terminal nodes (default: 15).
#' @param mtry Number of variables randomly selected as candidates for splitting a node.
#' @param ... Additional arguments passed to \code{\link[randomForestSRC]{rfsrc}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("randomForestSRC", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.rfsrc(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     ntree = 10,
#'     nodesize = 3
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.rfsrc <- function(time, event, X, newdata = NULL, new.times, obsWeights = NULL, id = NULL,
                       ntree = 1000, nodesize = 15, mtry = NULL, ...) {

  requireNamespace("randomForestSRC", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)
  Surv <- survival::Surv

  # 1. Handle Defaults
  if(is.null(obsWeights)) obsWeights <- rep(1, length(time))
  if(is.null(newdata)) newdata <- X

  # 2. Prepare Data (RFSRC strictly needs data.frames)
  X_df <- as.data.frame(X)
  newdata_df <- as.data.frame(newdata)
  data <- data.frame(time = time, event = event, X_df)

  # 3. Setup Arguments
  # FIX: Removed "survival::" prefix. Using as.formula protects the parser.
  args_list <- list(
    formula = as.formula("Surv(time, event) ~ ."),
    data = data,
    case.wt = obsWeights,
    ntree = ntree,
    nodesize = nodesize,
    importance = "none" # Faster fitting
  )

  # Safely append any additional arguments passed via ... (like from create_grid)
  extra_args <- list(...)
  if (length(extra_args) > 0) {
    args_list <- c(args_list, extra_args)
  }

  # Only add mtry if explicitly provided, otherwise let rfsrc calculate default
  if (!is.null(mtry)) args_list$mtry <- mtry

  # 4. Fit Model
  fit.rfsrc <- do.call(randomForestSRC::rfsrc, args_list)

  # 5. Predict on newdata (Training Grid)
  survs <- predict(fit.rfsrc, newdata = newdata_df, importance = "none")$survival
  train_times <- fit.rfsrc$time.interest

  # 6. Interpolate to new.times
  # Uses constant interpolation for step-function survival curves
  pred <- t(apply(survs, 1, function(y) {
    stats::approx(train_times, y, xout = new.times, method = "constant", rule = 2, ties = mean)$y
  }))

  if (is.null(dim(pred))) {
    pred <- matrix(pred, nrow = nrow(newdata_df), ncol = length(new.times))
  }

  # 7. Safety: Monotonicity and Clamping
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    # Ensure survival curve never goes up
    pred <- t(apply(pred, 1, cummin))
  }

  fit <- list(object = fit.rfsrc)
  class(fit) <- c("surv.rfsrc")
  list(pred = pred, fit = fit)
}



#' Prediction function for survival random forest (RFSRC)
#'
#' Obtains predicted survivals from a fitted \code{surv.rfsrc} object.
#'
#' @param object Fitted \code{surv.rfsrc} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("randomForestSRC", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.rfsrc(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     ntree = 10,
#'     nodesize = 3
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
# 1. Change the argument here to 'newdata'
predict.surv.rfsrc <- function(object, newdata, new.times, ...) {


  fit <- object$object

  # 3. Predict on native time grid (ensure you use newdata here!)
  survs <- predict(fit, newdata = as.data.frame(newdata), importance = "none")$survival
  train_times <- fit$time.interest

  # 4. Vectorized interpolation to new.times
  pred <- t(apply(survs, 1, function(y) {
    stats::approx(train_times, y, xout = new.times, method = "constant", rule = 2, ties = mean)$y
  }))

  # 5. Safety Check: Dimensions
  if (is.null(dim(pred))) {
    pred <- matrix(pred, nrow = nrow(newdata), ncol = length(new.times))
  }

  # 6. Safety Check: Clamp and Monotonicity
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  return(pred)
}




############################################################
## XGBoost (Cox) — survSuperLearner wrapper + predict method
############################################################

#' Wrapper for XGBoost (Robust CV-Tuned + Safe Prediction)
#'
#' Estimates a Cox proportional hazards model via XGBoost.
#' Incorporates safe Breslow hazard calculation and matrix alignment to prevent C++ crashes.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param nrounds Max number of boosting iterations (default: 1000).
#' @param early_stopping_rounds Rounds with no improvement to trigger early stopping (default: 10).
#' @param eta Learning rate (default: 0.05).
#' @param max_depth Maximum tree depth (default: 2).
#' @param min_child_weight Minimum sum of instance weight in a child (default: 5).
#' @param lambda L2 regularization term on weights (default: 10).
#' @param subsample Subsample ratio of the training instances (default: 0.7).
#' @param ... Additional arguments passed to \code{\link[xgboost]{xgb.train}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("xgboost", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.xgboost(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     nrounds = 5,
#'     early_stopping_rounds = 2,
#'     max_depth = 1
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.xgboost <- function(time, event, X, newdata = NULL,  new.times, obsWeights, id,
                         nrounds = 1000, early_stopping_rounds = 10,
                         eta = 0.05, max_depth = 2, min_child_weight = 5,
                         lambda = 10, subsample = 0.7, ...) {

  requireNamespace("xgboost", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)

  if(is.null(newdata)) newdata <- X

  # 1. Prepare Matrices
  X_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(X))
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  # CRITICAL FIX: Matrix Alignment for CV
  # If newdata is missing a factor level present in X, we must pad it with zeros
  missing_cols <- setdiff(colnames(X_mat), colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  # Ensure columns are in the exact same order
  newdata_mat <- newdata_mat[, colnames(X_mat), drop = FALSE]

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 2. Format for XGBoost Cox
  y_xgb <- ifelse(event == 1, time, -time)
  dtrain <- xgboost::xgb.DMatrix(data = X_mat, label = y_xgb, weight = obsWeights)

  # 3. Fit XGBoost
  fit <- xgboost::xgb.train(
    params = list(
      objective = "survival:cox",
      eval_metric = "cox-nloglik",
      eta = eta,
      max_depth = max_depth,
      min_child_weight = min_child_weight,
      lambda = lambda,
      subsample = subsample
    ),
    data = dtrain,
    nrounds = nrounds,
    watchlist = list(train = dtrain),
    early_stopping_rounds = early_stopping_rounds,
    verbose = 0,
    ...
  )

  # 4. Center Training Scores (To stabilize hazard)
  lp_train <- predict(fit, newdata = X_mat)
  tr_mean <- mean(lp_train)
  lp_train_centered <- lp_train - tr_mean

  # 5. Calculate Baseline Hazard (using your custom safe function)
  bh <- safe_breslow_step(
    time = time, event = event,
    risk_score = lp_train_centered, new.times = new.times
  )

  # 6. Predict on New Data
  lp_new <- predict(fit, newdata = newdata_mat)
  lp_new_centered <- lp_new - tr_mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 7. Safety Clamp
  pred[pred < 0] <- 0; pred[pred > 1] <- 1


  fit_obj <- list(
    object  = fit,
    basehaz = bh,
    times   = new.times,
    stats   = list(mean = tr_mean, features = colnames(X_mat))
  )
  class(fit_obj) <- c("surv.xgboost")


  list(pred = pred, fit = fit_obj)
}





#' Prediction function for XGBoost wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.xgboost} object.
#'
#' @param object Fitted \code{surv.xgboost} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("xgboost", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.xgboost(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     nrounds = 5,
#'     early_stopping_rounds = 2,
#'     max_depth = 1
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.xgboost <- function(object, newdata, new.times, ...) {


  # 1. Faster/Safer Interpolation using stepfun
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  bh <- bh_fun(new.times)

  # 2. Format Matrix and Align Columns
  # Must use the exact features the XGBoost model expects
  expected_cols <- object$stats$features
  if (is.null(expected_cols)) {
    expected_cols <- object$object$feature_names
  }
  if (is.null(expected_cols)) stop("Cannot determine expected feature names for xgboost model.")


  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(expected_cols, colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, expected_cols, drop = FALSE]

  # 3. Predict Linear Predictor
  lp_new <- predict(object$object, newdata = newdata_mat)

  # 4. Center and Clamp
  lp_new_centered <- lp_new - object$stats$mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  # 5. Survival Formula
  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 6. Safety Clamping
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1

  return(pred)
}







############################################################
## Survival SVM — Wrapper
## Pattern: Utility Score -> Negate -> Cox Offset -> Survival
############################################################

#' Wrapper for Survival Support Vector Machine (survivalsvm)
#'
#' Final Production Wrapper for SVM (Tunable & Robust).
#' Estimates a survival SVM and calibrates the raw utility scores into
#' survival probabilities using a univariate Cox proportional hazards model.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param gamma.mu Regularization parameter for the SVM (default: 0.1).
#' @param type Type of SVM implementation (default: "vanbelle2").
#' @param kernel Kernel type for the SVM (default: "lin_kernel").
#' @param opt.meth Optimization method (default: "quadprog").
#' @param ... Additional arguments passed to \code{\link[survivalsvm]{survivalsvm}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("survivalsvm", quietly = TRUE) &&
#'  requireNamespace("quadprog", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:25, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.svm(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.svm <- function(time, event, X, newdata, new.times, obsWeights, id,
                     gamma.mu = 0.1, type = "vanbelle2",
                     kernel = "lin_kernel", opt.meth = "quadprog", ...) {

  requireNamespace("survivalsvm", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 1. Prepare Data
  Y <- survival::Surv(time, event)
  df_train <- data.frame(Y = Y, X)

  # 2. Fit Survival SVM
  makediff3 <- utils::getFromNamespace("makediff3", "survivalsvm")
  fit <- survivalsvm::survivalsvm(
    formula = Y ~ .,
    data = df_train,
    type = type,
    diff.meth = "makediff3",
    gamma.mu = gamma.mu,
    kernel = kernel,
    opt.meth = opt.meth,
    ...
  )

  # 3. Extract Raw Utility Scores and Negate them to represent Risk
  p_tr <- predict(fit, newdata = X)
  lp_raw <- -1 * as.numeric(p_tr$predicted)

  # 4. Fit Calibrator Cox Model
  calib_data <- data.frame(time = time, event = event, score = lp_raw)
  calib_fit <- survival::coxph(
    survival::Surv(time, event) ~ score,
    data = calib_data,
    weights = obsWeights
  )

  # 5. Extract Baseline Hazard from Calibrator
  bh_df <- survival::basehaz(calib_fit, centered = FALSE)
  bh <- stats::approx(
    bh_df$time, bh_df$hazard, xout = new.times,
    method = "constant", f = 0, rule = 2, ties = mean
  )$y
  bh <- cummax(replace(bh, is.na(bh), 0))

  # 6. Predict and Calibrate on New Data
  p_te <- predict(fit, newdata = newdata)
  lp_new_raw <- -1 * as.numeric(p_te$predicted)

  # Use R's native predict.coxph to get perfectly centered linear predictors
  lp_new_calibrated <- predict(
    calib_fit,
    newdata = data.frame(score = lp_new_raw),
    type = "lp"
  )

  lp_new_calibrated <- pmin(lp_new_calibrated, 700)

  # 7. Convert to Survival Probabilities
  pred <- outer(lp_new_calibrated, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # Safety Clamp
  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  fit_obj <- list(
    object = fit,
    calibrator = calib_fit,
    basehaz = bh,
    times = new.times
  )
  class(fit_obj) <- c("surv.svm")

  list(pred = pred, fit = fit_obj)
}

#' Prediction function for SVM wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.svm} object.
#' Applies the learned Cox calibration to the new raw utility scores.
#'
#' @param object Fitted \code{surv.svm} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("survivalsvm", quietly = TRUE) &&
#'   requireNamespace("quadprog", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:25, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.svm(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.svm <- function(object, newdata, new.times, ...) {

  # 1. Faster/Safer Interpolation using stepfun
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  bh <- bh_fun(new.times)

  # 2. Get Raw Predictions from SVM
  p_te <- predict(object$object, newdata = newdata)
  lp_new_raw <- -1 * as.numeric(p_te$predicted)

  # 3. Apply Calibration using the saved Cox model
  lp_new_calibrated <- predict(
    object$calibrator,
    newdata = data.frame(score = lp_new_raw),
    type = "lp"
  )

  # 4. Convert to Survival Probabilities S(t)
  pred <- outer(lp_new_calibrated, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 5. Safety Clamp
  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  return(pred)
}









#' Wrapper for Survival Regression Trees (rpart)
#'
#' Final Production Wrapper for single decision trees.
#' Uses \code{\link[rpart]{rpart}} with method="exp" and calculates
#' survival probabilities using the Breslow estimator.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param cp Complexity parameter (default: 0.01).
#' @param minsplit Minimum number of observations to attempt a split (default: 20).
#' @param maxdepth Maximum depth of any node of the final tree (default: 30).
#' @param ... Additional arguments passed to \code{\link[rpart]{rpart.control}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("rpart", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.rpart(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     cp = 0.01,
#'     minsplit = 5,
#'     maxdepth = 3
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.rpart <- function(time, event, X, newdata, new.times, obsWeights, id,
                       cp = 0.01, minsplit = 20, maxdepth = 30, ...) {

  requireNamespace("rpart", quietly = TRUE)

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 1. Safety Clamp for Time
  time <- pmax(time, 1e-5)

  # 2. Prepare Data
  X_df <- as.data.frame(X)
  newdata_df <- as.data.frame(newdata)
  dat <- data.frame(time = time, event = event, X_df)

  # 3. Fit the Survival Tree
  fit <- rpart::rpart(
    survival::Surv(time, event) ~ .,
    data = dat,
    weights = obsWeights,
    method = "exp", # Survival method for rpart
    control = rpart::rpart.control(
      cp = cp,
      minsplit = minsplit,
      maxdepth = maxdepth,
      ...
    )
  )

  # 4. Predict Expected Event Rates and Convert to Linear Predictor
  # If a leaf has 0 events, the rate is 0. log(0) = -Inf, which crashes Breslow.
  # We use pmax to safely clamp the minimum rate before taking the log.
  rate_train <- predict(fit, newdata = X_df)
  lp_train <- log(pmax(rate_train, 1e-10))

  # 5. Center Training Scores
  tr_mean <- mean(lp_train)
  lp_train_centered <- lp_train - tr_mean

  # 6. Baseline Hazard Calculation
  bh <- safe_breslow_step(
    time = time,
    event = event,
    risk_score = lp_train_centered,
    new.times = new.times
  )

  # 7. Predict on newdata
  rate_new <- predict(fit, newdata = newdata_df)
  lp_new <- log(pmax(rate_new, 1e-10))

  lp_new_centered <- lp_new - tr_mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  # 8. Calculate Survival S(t)
  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata_df), ncol = length(new.times))

  # 9. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  fit_obj <- list(
    object = fit,
    basehaz = bh,
    times = new.times,
    stats = list(mean = tr_mean)
  )
  class(fit_obj) <- c("surv.rpart")

  list(pred = pred, fit = fit_obj)
}


#' Prediction function for rpart wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.rpart} object.
#'
#' @param object Fitted \code{surv.rpart} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("rpart", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.rpart(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     cp = 0.01,
#'     minsplit = 5,
#'     maxdepth = 3
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.rpart <- function(object, newdata, new.times, ...) {

  # 1. Reconstruct Baseline Hazard
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  bh <- bh_fun(new.times)

  # 2. Predict Event Rate and Convert to Centered LP
  rate_new <- predict(object$object, newdata = as.data.frame(newdata))
  lp_new <- log(pmax(rate_new, 1e-10))

  lp_new_centered <- lp_new - object$stats$mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  # 3. Calculate Survival S(t)
  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 4. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  return(pred)
}











#' Wrapper for Ridge Regression (Penalized Cox)
#'
#' Final Production Wrapper for Ridge Regression (Tunable & Robust).
#' Estimates a penalized Cox model using a pure Ridge penalty (alpha = 0).
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param nfolds Number of folds for internal cross-validation to select lambda. Default is 10.
#' @param ... Additional arguments passed to \code{\link[glmnet]{cv.glmnet}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.ridge(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     nfolds = 3
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.ridge <- function(time, event, X, newdata, new.times, obsWeights = NULL, id = NULL, nfolds = 10, ...) {

  requireNamespace("glmnet", quietly = TRUE)

  # We clamp the minimum time to a tiny positive number.
  time <- pmax(time, 1e-5)

  if (is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 1. Prepare Matrices & Align Columns
  X_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(X))
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(colnames(X_mat), colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, colnames(X_mat), drop = FALSE]

  # 2. Fit Ridge (Hardcoded alpha = 0 for pure Ridge)
  fit <- glmnet::cv.glmnet(
    x = X_mat,
    y = survival::Surv(time, event),
    family = "cox",
    alpha = 0,               # Enforce Ridge penalty
    weights = obsWeights,
    nfolds = nfolds,
    ...
  )

  # 3. Predict Risk Scores & Center them
  lp_train <- as.numeric(predict(fit, newx = X_mat, s = "lambda.min", type = "link"))
  tr_mean <- mean(lp_train)
  lp_train_centered <- lp_train - tr_mean

  # 4. Baseline Hazard (Using our robust helper)
  bh <- safe_breslow_step(
    time = time,
    event = event,
    risk_score = lp_train_centered,
    new.times = new.times
  )

  # 5. Predict on newdata
  lp_new <- as.numeric(predict(fit, newx = newdata_mat, s = "lambda.min", type = "link"))
  lp_new_centered <- lp_new - tr_mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # Safety Clamp
  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  fit_obj <- list(
    object = fit,
    basehaz = bh,
    times = new.times,
    stats = list(mean = tr_mean, features = colnames(X_mat))
  )
  class(fit_obj) <- c("surv.ridge")

  list(pred = pred, fit = fit_obj)
}


#' Prediction function for Ridge wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.ridge} object.
#'
#' @param object Fitted \code{surv.ridge} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.ridge(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     nfolds = 3
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.ridge <- function(object, newdata, new.times, ...) {

  # 1. Faster/Safer Interpolation
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  bh <- bh_fun(new.times)

  # 2. Format Matrix and Align Columns
  expected_cols <- object$stats$features
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(expected_cols, colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, expected_cols, drop = FALSE]

  # 3. Predict Linear Predictor
  lp_new <- as.numeric(predict(object$object, newx = newdata_mat, s = "lambda.min", type = "link"))

  # 4. Center and Clamp
  lp_new_centered <- lp_new - object$stats$mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  # 5. Survival Formula
  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  pred[pred < 0] <- 0
  pred[pred > 1] <- 1

  return(pred)
}






############################################################
## Ranger (Random Forest) — wrapper + predict method
## Pattern: Native Probability Output + Interpolation
############################################################

#' Wrapper function for Ranger Random Survival Forest
#'
#' Final Production Wrapper for Ranger (Tunable & Fast).
#' Uses the \code{\link[ranger]{ranger}} C++ implementation to estimate survival curves.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param num.trees Number of trees (default: 500).
#' @param mtry Number of variables to split at each node. Defaults to \code{sqrt(p)}.
#' @param min.node.size Minimum node size (default: 15 for survival).
#' @param ... Additional arguments passed to \code{\link[ranger]{ranger}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("ranger", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.ranger(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     num.trees = 10,
#'     min.node.size = 3
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.ranger <- function(time, event, X, newdata, new.times, obsWeights, id,
                        num.trees = 500, mtry = NULL, min.node.size = NULL, ...) {

  requireNamespace("ranger", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 1. Prepare Data
  dat <- data.frame(time = time, status = event, as.data.frame(X))

  # 2. Setup Arguments
  args_list <- list(
    formula = survival::Surv(time, status) ~ .,
    data = dat,
    num.trees = num.trees,
    case.weights = obsWeights,
    probability = FALSE, # Survival uses probability=FALSE in ranger
    verbose = FALSE,
    ...
  )

  if (!is.null(mtry)) args_list$mtry <- mtry
  if (!is.null(min.node.size)) args_list$min.node.size <- min.node.size

  # 3. Fit Model
  fit <- do.call(ranger::ranger, args_list)

  # 4. Predict on newdata (Training Grid)
  p_obj <- predict(fit, data = as.data.frame(newdata))
  surv_probs <- p_obj$survival
  train_times <- p_obj$unique.death.times

  # 5. Vectorized Interpolation to new.times
  pred <- t(apply(surv_probs, 1, function(y) {
    stats::approx(x = train_times, y = y, xout = new.times, method = "constant", rule = 2, ties = mean)$y
  }))

  # 6. Safety Clamp and Monotonicity
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  fit_obj <- list(object = fit, times = new.times)
  class(fit_obj) <- c("surv.ranger")

  list(pred = pred, fit = fit_obj)
}




#' Prediction function for Ranger wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.ranger} object.
#'
#' @param object Fitted \code{surv.ranger} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("ranger", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.ranger(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     num.trees = 10,
#'     min.node.size = 3
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.ranger <- function(object, newdata, new.times, ...) {

  # 1. Predict using saved model
  p_obj <- predict(object$object, data = as.data.frame(newdata))
  surv_probs  <- p_obj$survival
  train_times <- p_obj$unique.death.times

  # 2. Vectorized Interpolation
  pred <- t(apply(surv_probs, 1, function(y) {
    stats::approx(x = train_times, y = y, xout = new.times, method = "constant", rule = 2, ties = mean)$y
  }))

  # 3. Dimensions Safety Check
  if (is.null(dim(pred))) {
    pred <- matrix(pred, nrow = nrow(newdata), ncol = length(new.times))
  }

  # 4. Safety Clamp and Monotonicity
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  return(pred)
}







#' Universal Parametric Survival Wrapper
#'
#' Final Production Wrapper for AFT Models (Weibull, Exponential, LogNormal, LogLogistic).
#' Replaces individual wrappers with one robust, vectorized function.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param dist Distribution for the AFT model (default: "weibull").
#' @param ... Additional arguments passed to \code{\link[survival]{survreg}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.parametric(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL,
#'   dist = "weibull"
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.parametric <- function(time, event, X, newdata, new.times, obsWeights, id, dist = "weibull", ...) {
  requireNamespace("survival", quietly = TRUE)

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 1. Handle Time <= 0 issue for AFT models (log(0) = -Inf)
  time <- pmax(time, 1e-5)

  # 2. Fit Model
  # CRITICAL FIX: Use data.frame() to prevent cbind from destroying factor variables
  dat <- data.frame(time = time, event = event, as.data.frame(X))

  fit <- survival::survreg(
    survival::Surv(time, event) ~ .,
    data = dat,
    weights = obsWeights,
    dist = dist,
    ...
  )

  # 3. Fast Prediction (Closed Form)
  # predict.survreg uses type = "linear" to return the linear predictor
  lp <- predict(fit, newdata = as.data.frame(newdata), type = "linear")
  scale <- fit$scale

  # 4. Vectorized Survival Calculation
  if (dist %in% c("exponential", "weibull")) {
    pred <- outer(lp, new.times, function(eta, t) {
      exp( - (t / exp(eta))^(1/scale) )
    })

  } else if (dist == "lognormal") {
    pred <- outer(lp, new.times, function(eta, t) {
      1 - stats::pnorm((log(t) - eta) / scale)
    })

  } else if (dist == "loglogistic") {
    pred <- outer(lp, new.times, function(eta, t) {
      1 / (1 + (t / exp(eta))^(1/scale))
    })

  } else {
    stop("Distribution not supported in fast wrapper")
  }

  # 5. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  fit_obj <- list(object = fit, dist = dist)
  class(fit_obj) <- c("surv.parametric")

  list(pred = pred, fit = fit_obj)
}





#' Parametric Survival Prediction Wrapper (Exponential)
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Cluster identification variable.
#' @param ... Additional ignored arguments.
#'
#' @return A list containing the fitted model and predictions.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.exponential(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.exponential <- function(time, event, X, newdata, new.times, obsWeights, id, ...) {
  surv.parametric(time = time, event = event, X = X, newdata = newdata,
                  new.times = new.times, obsWeights = obsWeights, id = id,
                  dist = "exponential", ...)
}



#' Parametric Survival Prediction Wrapper (Log-Logistic)
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Cluster identification variable.
#' @param ... Additional ignored arguments.
#'
#' @return A list containing the fitted model and predictions.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.loglogistic(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.loglogistic <- function(time, event, X, newdata, new.times, obsWeights, id, ...) {
  surv.parametric(time = time, event = event, X = X, newdata = newdata,
                  new.times = new.times, obsWeights = obsWeights, id = id,
                  dist = "loglogistic", ...)
}



#' Parametric Survival Prediction Wrapper (Log-Normal)
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Cluster identification variable.
#' @param ... Additional ignored arguments.
#'
#' @return A list containing the fitted model and predictions.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.lognormal(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.lognormal <- function(time, event, X, newdata, new.times, obsWeights, id, ...) {
  surv.parametric(time = time, event = event, X = X, newdata = newdata,
                  new.times = new.times, obsWeights = obsWeights, id = id,
                  dist = "lognormal", ...)
}

#' Parametric Survival Prediction Wrapper (Weibull)
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Cluster identification variable.
#' @param ... Additional ignored arguments.
#'
#' @return A list containing the fitted model and predictions.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.weibull(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.weibull <- function(time, event, X, newdata, new.times, obsWeights, id, ...) {
  surv.parametric(time = time, event = event, X = X, newdata = newdata,
                  new.times = new.times, obsWeights = obsWeights, id = id,
                  dist = "weibull", ...)
}






#' Prediction function for Universal Parametric Wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.parametric} object
#' using exact closed-form equations.
#'
#' @param object Fitted \code{surv.parametric} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.parametric(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL,
#'   dist = "weibull"
#' )
#'
#' pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#' dim(pred)
#' @noRd
#' @export
predict.surv.parametric <- function(object, newdata, new.times, ...) {

  fit <- object$object
  dist <- object$dist
  scale <- fit$scale

  # Extract linear predictor
  lp <- predict(fit, newdata = as.data.frame(newdata), type = "linear")

  # Vectorized Survival Calculation
  if (dist %in% c("exponential", "weibull")) {
    pred <- outer(lp, new.times, function(eta, t) {
      exp( - (t / exp(eta))^(1/scale) )
    })
  } else if (dist == "lognormal") {
    pred <- outer(lp, new.times, function(eta, t) {
      1 - stats::pnorm((log(t) - eta) / scale)
    })
  } else if (dist == "loglogistic") {
    pred <- outer(lp, new.times, function(eta, t) {
      1 / (1 + (t / exp(eta))^(1/scale))
    })
  }

  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  return(pred)
}







#' Kaplan-Meier Prediction Algorithm
#'
#' This prediction algorithm ignores all covariates and computes the marginal
#' Kaplan-Meier survival estimator using the \code{\link[survival]{survfit}} function.
#'
#' @param time Numeric vector of observed follow-up times.
#' @param event Numeric vector of event indicators (1 = event, 0 = censored).
#' @param X Training covariate data.frame (Ignored by KM).
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Numeric vector of times at which to predict survival.
#' @param obsWeights Numeric vector of observation weights.
#' @param id Optional vector indicating subject/cluster identities.
#' @param ... Additional ignored arguments.
#'
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: A list containing the fitted \code{\link[survival]{survfit}} object.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at \code{new.times}.
#' }
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.km(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.km <- function(time, event, X, newdata, new.times, obsWeights, id, ...) {

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # Fit the weighted KM curve
  fit.km <- survival::survfit(
    survival::Surv(time, event) ~ 1,
    weights = obsWeights
  )

  # Efficient prediction using stepfun
  sfun <- stats::stepfun(fit.km$time, c(1, fit.km$surv), right = FALSE)
  surv_probs <- sfun(new.times)

  # Repeat for every patient (KM gives the exact same curve for everyone)
  pred <- matrix(surv_probs,
                 nrow = nrow(newdata),
                 ncol = length(new.times),
                 byrow = TRUE)

  fit <- list(object = fit.km)
  class(fit) <- c("surv.km")

  return(list(pred = pred, fit = fit))
}



#' Predict Method for Kaplan-Meier Wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.km} object.
#'
#' @param object A fitted object of class \code{surv.km}.
#' @param newdata New covariate data.frame for which to obtain predictions (Ignored).
#' @param new.times Numeric vector of times at which to predict survival.
#' @param ... Additional ignored arguments.
#'
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation times.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.km(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#' dim(pred)
#' @noRd
#' @export
predict.surv.km <- function(object, newdata, new.times, ...) {

  # Extract the original fitted KM object
  fit.km <- object$object

  # Reconstruct the step function
  sfun <- stats::stepfun(fit.km$time, c(1, fit.km$surv), right = FALSE)
  surv_probs <- sfun(new.times)

  # Replicate across all new patients
  pred_matrix <- matrix(surv_probs,
                        nrow = nrow(newdata),
                        ncol = length(new.times),
                        byrow = TRUE)

  return(pred_matrix)
}








#' Wrapper function for Penalized Cox Regression (GLMNET)
#'
#' Final Production Wrapper for GLMNET (Tunable & Robust).
#' Estimates a penalized Cox model (Lasso, Ridge, or Elastic Net) with automatic lambda selection.
#' Uses the Breslow estimator with a step-function approach for the baseline hazard.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param alpha The elasticnet mixing parameter (0 = Ridge, 1 = Lasso). Default is 1.
#' @param nfolds Number of folds for internal cross-validation to select lambda. Default is 10.
#' @param ... Additional arguments passed to \code{\link[glmnet]{cv.glmnet}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }

#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.glmnet(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     alpha = 1,
#'     nfolds = 3
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.glmnet <- function(time, event, X, newdata, new.times, obsWeights, id,
                        alpha = 1, nfolds = 10, ...) {

  requireNamespace("glmnet", quietly = TRUE)

  # We clamp the minimum time to a tiny positive number.
  time <- pmax(time, 1e-5)

  # 1. Convert factors to dummy variables (glmnet requires matrices)
  X_mat <- stats::model.matrix(~ . - 1, data = X)
  newdata_mat <- stats::model.matrix(~ . - 1, data = newdata)

  missing_cols <- setdiff(colnames(X_mat), colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, colnames(X_mat), drop = FALSE]

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 2. Fit the Penalized Model
  # We use standard argument passing (cleaner than do.call here)
  fit <- glmnet::cv.glmnet(
    x = X_mat,
    y = survival::Surv(time, event),
    family = "cox",
    weights = obsWeights,
    alpha = alpha,
    nfolds = nfolds,
    ...
  )

  beta_min <- as.matrix(stats::coef(fit, s = "lambda.min"))
  s_use <- "lambda.min"

  if (sum(beta_min != 0) == 0) {
    beta_path <- as.matrix(fit$glmnet.fit$beta)
    nnz_path <- colSums(beta_path != 0)

    idx_nonzero <- which(nnz_path > 0)
    if (length(idx_nonzero) > 0) {
      s_use <- fit$glmnet.fit$lambda[idx_nonzero[1]]
    }
  }

  # 3. Calibrate Baseline Hazard (Breslow Estimator)
  lp_train <- as.numeric(predict(fit, newx = X_mat, s = s_use, type = "link"))
  cox_off <- survival::coxph(
    survival::Surv(time, event) ~ offset(lp_train),
    weights = obsWeights,
    ties = "breslow"
  )

  bh_df <- survival::basehaz(cox_off, centered = FALSE)
  bh <- stats::approx(
    bh_df$time, bh_df$hazard, xout = new.times,
    method = "constant", f = 0, rule = 2, ties = mean
  )$y
  bh <- cummax(replace(bh, is.na(bh), 0))

  # 4. Predict on newdata
  lp_new <- as.numeric(predict(fit, newx = newdata_mat, s = s_use, type = "link"))
  pred <- outer(lp_new, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 5. Safety Clamp
  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  fit_obj <- list(
    object = fit,
    basehaz = bh,
    times = new.times,
    stats = list(
      features = colnames(X_mat),
      s_use = s_use
    )
  )
  class(fit_obj) <- c("surv.glmnet")

  list(pred = pred, fit = fit_obj)
}




#' Prediction function for GLMNET wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.glmnet} object.
#'
#' @param object Fitted \code{surv.glmnet} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.glmnet(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     alpha = 1,
#'     nfolds = 3
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.glmnet <- function(object, newdata, new.times, ...) {

  # 1. Align Baseline Hazard (Step Function)
  if (identical(all.equal(new.times, object$times), TRUE)) {
    bh <- object$basehaz
  } else {
    bh <- stats::approx(
      object$times, object$basehaz, xout = new.times,
      method = "constant", f = 0, rule = 2, ties = mean
    )$y
  }
  bh <- cummax(replace(bh, is.na(bh), 0))

  # 2. Convert newdata to matrix safely
  # model.matrix removes NAs, so we ensure the dataframe is intact

  expected_cols <- object$stats$features
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(expected_cols, colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, expected_cols, drop = FALSE]


  # 3. Predict Linear Predictor
  s_use <- object$stats$s_use
  if (is.null(s_use)) s_use <- "lambda.min"

  lp_new <- as.numeric(predict(
    object$object,
    newx = newdata_mat,
    s = s_use,
    type = "link"
  ))


  # 4. Convert LP to Survival Probability Matrix
  pred <- outer(lp_new, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 5. Safety Clamp
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1

  return(pred)
}






#' Wrapper function for Gradient Boosting (GBM) prediction algorithm
#'
#' Final Production Wrapper for GBM (Tunable & Robust).
#' Estimates a Cox proportional hazards model via gradient boosting.
#' Uses the Breslow estimator with a step-function approach for the baseline hazard.
#' Includes internal safeguards against C++ crashes and small cross-validation folds.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param n.trees Integer specifying the total number of trees to fit (default: 1000).
#' @param interaction.depth Maximum depth of variable interactions (default: 2).
#' @param shrinkage A shrinkage parameter applied to each tree (default: 0.01).
#' @param cv.folds Number of cross-validation folds to perform internally for optimal tree selection (default: 5).
#' @param n.minobsinnode Minimum number of observations in the trees terminal nodes (default: 10).
#' @param ... Additional arguments passed to \code{\link[gbm]{gbm}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("gbm", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.gbm(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     n.trees = 20,
#'     interaction.depth = 1,
#'     shrinkage = 0.05,
#'     cv.folds = 0,
#'     n.minobsinnode = 3
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.gbm <- function(time, event, X, newdata, new.times, obsWeights, id,
                     n.trees = 1000, interaction.depth = 2, shrinkage = 0.01,
                     cv.folds = 5, n.minobsinnode = 10, ...) {

  requireNamespace("gbm", quietly = TRUE)
  requireNamespace("survival", quietly = TRUE)

  # 1. CRITICAL FIX: Prevent C++ Crashes (Data Types)
  if(is.matrix(X)) X <- as.data.frame(X)
  if(is.matrix(newdata)) newdata <- as.data.frame(newdata)

  # Force characters to factors
  X[] <- lapply(X, function(x) if(is.character(x)) as.factor(x) else x)
  newdata[] <- lapply(newdata, function(x) if(is.character(x)) as.factor(x) else x)

  # Align factor levels in newdata to match training
  for (nm in intersect(names(X), names(newdata))) {
    if (is.factor(X[[nm]])) {
      newdata[[nm]] <- factor(newdata[[nm]], levels = levels(X[[nm]]))
    }
  }


  # 2. CRITICAL FIX: Safety for Small Folds
  # If a CV fold is tiny, n.minobsinnode=10 will crash GBM.
  n_obs <- nrow(X)
  safe_min_node <- max(3, floor(n_obs * 0.05)) # Ensure at least 3 obs per node
  n.minobsinnode <- min(n.minobsinnode, safe_min_node)

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))
  dat <- data.frame(time = time, status = event, X)
  n.cores <- .safe_detect_cores()

  # 3. Fit GBM Model
  # We use explicit arguments so create_surv_grid can tune them perfectly
  fit <- gbm::gbm(
    formula = survival::Surv(time, status) ~ .,
    data = dat,
    distribution = "coxph",
    n.trees = n.trees,
    interaction.depth = interaction.depth,
    shrinkage = shrinkage,
    cv.folds = cv.folds,
    n.minobsinnode = n.minobsinnode,
    weights = obsWeights,
    verbose = FALSE,
    keep.data = FALSE,
    n.cores = n.cores,
    ...
  )

  # 4. Get Best Iteration
  method_perf <- if(cv.folds > 0) "cv" else "OOB"
  best.iter <- gbm::gbm.perf(fit, method = method_perf, plot.it = FALSE)

  # 5. Calibration (Breslow Estimator)
  # Get Risk Scores (Linear Predictor) on Training Data
  lp_train <- predict(fit, newdata = X, n.trees = best.iter, type = "link")

  cox_df <- data.frame(time = time, event = event, lp = lp_train)
  cox_off <- survival::coxph(
    survival::Surv(time, event) ~ offset(lp),
    data = cox_df,
    weights = obsWeights,
    ties = "breslow"
  )

  bh_df <- survival::basehaz(cox_off, centered = FALSE)

  # Interpolate to new.times (Step Function)
  bh <- stats::approx(
    bh_df$time, bh_df$hazard, xout = new.times,
    method = "constant", f = 0, rule = 2, ties = mean
  )$y
  bh[is.na(bh)] <- 0
  bh <- cummax(bh)

  # 6. Predict on New Data
  lp_new <- predict(fit, newdata = newdata, n.trees = best.iter, type = "link")

  # S(t) = exp(-H0(t) * exp(lp))
  pred <- outer(lp_new, bh, function(lp, h) exp(-exp(lp) * h))

  # 7. Safety Clamp
  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  fit_obj <- list(object = fit, basehaz = bh, times = new.times, best.iter = best.iter)
  class(fit_obj) <- c("surv.gbm")

  list(pred = pred, fit = fit_obj)
}

.safe_detect_cores <- function(detect = parallel::detectCores) {
  n_cores <- tryCatch(detect(), error = function(e) NA_integer_)
  if (length(n_cores) != 1L ||
      !is.numeric(n_cores) ||
      is.na(n_cores) ||
      !is.finite(n_cores) ||
      n_cores < 1) {
    return(1L)
  }

  as.integer(n_cores)
}


#' Prediction function for GBM wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.gbm} object.
#' Uses a step function to align the Breslow baseline hazard with the requested times.
#'
#' @param object Fitted \code{surv.gbm} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("gbm", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.gbm(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     n.trees = 20,
#'     interaction.depth = 1,
#'     shrinkage = 0.05,
#'     cv.folds = 0,
#'     n.minobsinnode = 3
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.gbm <- function(object, newdata, new.times, ...) {

  # 1. Safer Interpolation using stepfun
  # Prepend 0 to handle times before the first event safely
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  bh <- bh_fun(new.times)

  # 2. Linear Predictor
  lp_new <- predict(
    object$object,
    newdata = as.data.frame(newdata),
    n.trees = object$best.iter,
    type = "link"
  )

  # 3. Survival Formula S(t) = exp(-H0(t) * exp(lp))
  pred <- outer(lp_new, bh, function(lp, h) exp(-exp(lp) * h))

  # 4. Safety Clamping
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1

  return(pred)
}








#' Wrapper for Generalized Additive Cox Regression (GAM)
#'
#' Final Production Wrapper for GAM (Tunable & Robust).
#' Uses \code{\link[mgcv]{gam}} to fit an additive combination of smooth
#' and linear functions.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain predicted survivals.
#' @param obsWeights Observation weights (Note: Ignored, as mgcv uses weights for the event indicator).
#' @param id Optional cluster/individual ID indicator.
#' @param cts.num Cutoff of unique values at which a numeric covariate receives a smooth term (s).
#' @param ... Additional arguments passed to \code{\link[mgcv]{gam}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("mgcv", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.gam(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     cts.num = 5
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.gam <- function(time, event, X, newdata, new.times, obsWeights, id, cts.num = 5, ...) {

  requireNamespace("mgcv", quietly = TRUE)

  # 1. Formula Construction (Safe Spline Assignment)
  X_df <- as.data.frame(X)
  newdata_df <- as.data.frame(newdata)

  # Only apply s() if the column is numeric AND has enough unique values
  terms <- sapply(names(X_df), function(var) {
    if (is.numeric(X_df[[var]]) && length(unique(X_df[[var]])) > cts.num) {
      return(paste0("s(", var, ")"))
    } else {
      return(var)
    }
  })

  form_str <- paste("time ~", paste(terms, collapse = " + "))
  gam.formula <- as.formula(form_str)

  # 2. Prepare Data
  # mgcv::cox.ph requires 'time' in the data, and the event indicator passed as 'weights'
  dat <- data.frame(time = time, X_df)

  # 3. Fit GAM
  fit <- mgcv::gam(
    gam.formula,
    family = mgcv::cox.ph(),
    data = dat,
    weights = event, # Critical for mgcv cox models
    ...
  )

  # 4. Predict on newdata
  # mgcv natively returns S(t) if type="response" and newdata contains a 'time' column.
  n_new <- nrow(newdata_df)
  n_t   <- length(new.times)

  # Duplicate each row of newdata for every time point: P1, P1, P1, P2, P2, P2...
  long_newdata <- newdata_df[rep(1:n_new, each = n_t), , drop = FALSE]
  # Add the matching times: T1, T2, T3, T1, T2, T3...
  long_newdata$time <- rep(new.times, times = n_new)

  pred_vec <- predict(fit, newdata = long_newdata, type = "response")

  # Fill matrix by row to match the P1(T1,T2,T3) expansion structure
  pred <- matrix(pred_vec, nrow = n_new, ncol = n_t, byrow = TRUE)

  # 5. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  fit_obj <- list(object = fit, times = new.times)
  class(fit_obj) <- c("surv.gam")

  list(pred = pred, fit = fit_obj)
}



#' Prediction function for GAM wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.gam} object.
#'
#' @param object Fitted \code{surv.gam} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("mgcv", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.gam(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     cts.num = 5
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.gam <- function(object, newdata, new.times, ...) {

  newdata_df <- as.data.frame(newdata)
  n_new <- nrow(newdata_df)
  n_t   <- length(new.times)

  # 1. Expand newdata to match mgcv's required format
  long_newdata <- newdata_df[rep(1:n_new, each = n_t), , drop = FALSE]
  long_newdata$time <- rep(new.times, times = n_new)

  # 2. Predict S(t)
  pred_vec <- predict(object$object, newdata = long_newdata, type = "response")
  pred <- matrix(pred_vec, nrow = n_new, ncol = n_t, byrow = TRUE)

  # 3. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  return(pred)
}






#' Wrapper for standard Cox Proportional Hazards
#'
#' Final Production Wrapper for CoxPH.
#' Uses partial maximum likelihood and the Breslow estimator.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param ... Additional arguments passed to \code{\link[survival]{coxph}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.coxph(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' dim(fit[["pred"]])
#' @export
surv.coxph <- function(time, event, X, newdata, new.times, obsWeights, id, ...) {
  requireNamespace("survival", quietly = TRUE)

  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 1. Safety Clamp for Time
  time <- pmax(time, 1e-5)

  # 2. Clean Data Frame (Prevents '.' from swallowing external variables)
  dat <- data.frame(time = time, event = event, as.data.frame(X))

  # 3. Fit Cox Model safely passing 'id' only if it exists
  if(!missing(id) && !is.null(id)) {
    fit.coxph <- survival::coxph(
      survival::Surv(time, event) ~ .,
      data = dat,
      weights = obsWeights,
      id = id,
      x = TRUE,
      ...
    )
  } else {
    fit.coxph <- survival::coxph(
      survival::Surv(time, event) ~ .,
      data = dat,
      weights = obsWeights,
      x = TRUE,
      ...
    )
  }

  # 4. Get Baseline Hazard (Uncentered, evaluated at X=0)
  bh_clean <- survival::basehaz(fit.coxph, centered = FALSE)

  # 5. Fast Step Function Interpolation to new.times
  bh_fun <- stats::stepfun(bh_clean$time, c(0, bh_clean$hazard), right = FALSE)
  bh_interp <- bh_fun(new.times)

  # 6. Get Risk Scores on newdata (Must align with X=0 reference!)
  lp_new <- predict(
    fit.coxph,
    newdata = as.data.frame(newdata),
    type = "lp",
    reference = "zero"
  )

  # 7. Calculate Survival Probabilities
  pred <- outer(lp_new, bh_interp, function(eta, H0) exp(-exp(eta) * H0))

  # 8. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  # Save the raw basehaz table so predict.surv.coxph can evaluate any future time points
  fit <- list(
    object = fit.coxph,
    basehaz = bh_clean$hazard,
    times = bh_clean$time
  )
  class(fit) <- c("surv.coxph")

  list(pred = pred, fit = fit)
}




#' Prediction function for Cox regression wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.coxph} object.
#'
#' @param object Fitted \code{surv.coxph} object.
#' @param newdata New covariate data.frame for which to obtain predictions.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' data("metabric", package = "SuperSurv")
#' dat <- metabric[1:30, ]
#' x_cols <- grep("^x", names(dat))[1:3]
#' X <- dat[, x_cols, drop = FALSE]
#' newX <- X[1:5, , drop = FALSE]
#' times <- seq(50, 150, by = 50)
#'
#' fit <- surv.coxph(
#'   time = dat$duration,
#'   event = dat$event,
#'   X = X,
#'   newdata = newX,
#'   new.times = times,
#'   obsWeights = rep(1, nrow(dat)),
#'   id = NULL
#' )
#'
#' pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#' dim(pred)
#' @noRd
#' @export
predict.surv.coxph <- function(object, newdata, new.times, ...) {

  # 1. Get Linear Predictor (Risk Score) relative to X=0
  lp <- predict(
    object$object,
    newdata = as.data.frame(newdata),
    type = "lp",
    reference = "zero"
  )

  # 2. Reconstruct Baseline Hazard Step Function from saved raw data
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  H0 <- bh_fun(new.times)

  # 3. Calculate Survival S(t)
  pred <- outer(lp, H0, function(eta, h) exp(-exp(eta) * h))

  # 4. Safety Clamps
  pred[pred < 0] <- 0; pred[pred > 1] <- 1
  if (ncol(pred) > 1) {
    pred <- t(apply(pred, 1, cummin))
  }

  return(pred)
}






#' Wrapper function for Component-Wise Boosting (CoxBoost)
#'
#' Final Production Wrapper for CoxBoost (Tunable & Robust).
#' Estimates a Cox model via component-wise likelihood based boosting.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights (Note: CoxBoost does not natively support weights, so these are ignored).
#' @param id Optional cluster/individual ID indicator.
#' @param stepno Number of boosting steps (default: 100).
#' @param penalty Penalty value for the update (default: 100).
#' @param ... Additional arguments passed to \code{\link[CoxBoost]{CoxBoost}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' if (requireNamespace("CoxBoost", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.coxboost(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     stepno = 10,
#'     penalty = 50
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.coxboost <- function(time, event, X, newdata, new.times, obsWeights, id,
                          stepno = 100, penalty = 100, ...) {

  requireNamespace("CoxBoost", quietly = TRUE)

  # 1. Prepare Matrices & Align Columns
  X_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(X))
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(colnames(X_mat), colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, colnames(X_mat), drop = FALSE]

  # 2. Fit CoxBoost
  # Note: CoxBoost automatically standardizes and selects variables step-by-step
  fit <- CoxBoost::CoxBoost(
    time = time,
    status = event,
    x = X_mat,
    stepno = stepno,
    penalty = penalty,
    ...
  )

  # 3. Predict Risk Scores (Linear Predictors)
  # CoxBoost returns a matrix for 'lp' if multiple steps are requested,
  # but by default returns the final step. We ensure it's a vector.
  lp_train <- as.numeric(predict(fit, newdata = X_mat, type = "lp"))

  # 4. Center Training Scores (To stabilize hazard calculation)
  tr_mean <- mean(lp_train)
  lp_train_centered <- lp_train - tr_mean

  # 5. Baseline Hazard (Using our robust helper)
  # Ensure safe_breslow_step is loaded in your environment!
  bh <- safe_breslow_step(
    time = time,
    event = event,
    risk_score = lp_train_centered,
    new.times = new.times
  )

  # 6. Predict on New Data
  lp_new <- as.numeric(predict(fit, newdata = newdata_mat, type = "lp"))
  lp_new_centered <- lp_new - tr_mean

  # SAFETY CLAMP: Prevent Inf * 0 = NaN
  lp_new_centered <- pmin(lp_new_centered, 700)

  # 7. Convert to Survival Probabilities
  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  pred[pred < 0] <- 0; pred[pred > 1] <- 1

  # Save the column names so the predict method knows exactly what to expect
  fit_obj <- list(
    object = fit,
    basehaz = bh,
    times = new.times,
    stats = list(mean = tr_mean, features = colnames(X_mat))
  )
  class(fit_obj) <- c("surv.coxboost")

  list(pred = pred, fit = fit_obj)
}


#' Prediction function for CoxBoost wrapper
#'
#' Obtains predicted survivals from a fitted \code{surv.coxboost} object.
#'
#' @param object Fitted \code{surv.coxboost} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("CoxBoost", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.coxboost(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     stepno = 10,
#'     penalty = 50
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.coxboost <- function(object, newdata, new.times, ...) {

  # 1. Faster/Safer Interpolation using stepfun
  bh_fun <- stats::stepfun(object$times, c(0, object$basehaz), right = FALSE)
  bh <- bh_fun(new.times)

  # 2. Format Matrix and Align Columns
  expected_cols <- object$stats$features
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(expected_cols, colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, expected_cols, drop = FALSE]

  # 3. Predict Linear Predictor
  lp_new <- as.numeric(predict(object$object, newdata = newdata_mat, type = "lp"))

  # 4. Center and Clamp
  lp_new_centered <- lp_new - object$stats$mean
  lp_new_centered <- pmin(lp_new_centered, 700)

  # 5. Survival Formula
  pred <- outer(lp_new_centered, bh, function(lp, h) exp(-exp(lp) * h))
  pred <- matrix(as.vector(pred), nrow = nrow(newdata), ncol = length(new.times))

  # 6. Safety Clamping
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1

  return(pred)
}








#' Wrapper for BART (Bayesian Additive Regression Trees)
#'
#' Final Production Wrapper for BART (Tunable & Robust).
#' Uses the \code{\link[BART]{mc.surv.bart}} function.
#' Automatically reshapes the flat output vector into a survival matrix
#' and interpolates the predictions to the requested new.times.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction.
#' @param new.times Times at which to obtain predicted survivals.
#' @param obsWeights Observation weights (Note: BART does not natively support weights).
#' @param id Optional cluster/individual ID indicator.
#' @param ntree Number of trees (default: 50).
#' @param ndpost Number of posterior draws (default: 1000).
#' @param nskip Number of burn-in draws (default: 250).
#' @param ... Additional arguments passed to \code{\link[BART]{mc.surv.bart}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }
#' @examples
#' \dontrun{
#' if (.Platform$OS.type != "windows" &&
#'   requireNamespace("BART", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:20, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.bart(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     ntree = 3,
#'     ndpost = 5,
#'     nskip = 5
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' }
#' @export
surv.bart <- function(time, event, X, newdata = NULL, new.times, obsWeights = NULL, id = NULL,
                      ntree = 10, ndpost = 30, nskip = 10, ...) {

  if (.Platform$OS.type == "windows") {
    stop(
      "surv.bart() is not supported on Windows because ",
      "BART::mc.surv.bart() requires multicore forking."
    )
  }

  requireNamespace("BART", quietly = TRUE)

  # 0) Defaults (important for SL internals)
  if (is.null(newdata)) newdata <- X

  # 1) Time safety (avoid exact 0)
  time <- pmax(time, 1e-5)

  # 2) Design matrices + column alignment
  X_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(X))
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(colnames(X_mat), colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, colnames(X_mat), drop = FALSE]

  # 3) Fit + predict (BART predicts x.test during fitting)
  fit <- BART::mc.surv.bart(
    x.train  = X_mat,
    times    = time,
    delta    = event,
    x.test   = newdata_mat,
    ntree    = ntree,
    ndpost   = ndpost,
    nskip    = nskip,
    mc.cores = 1,
    seed     = 99,
    ...
  )

  # 4) Extract BART-native time grid
  unique_times <- fit$times
  unique_times <- sort(unique_times)
  unique_times <- unique_times[unique_times > 0]

  n_new <- nrow(newdata_mat)
  n_t   <- length(unique_times)

  # 5) Reshape survival output robustly
  surv_vec <- fit$surv.test.mean
  if (length(surv_vec) != n_new * n_t) {
    stop(
      "BART returned an unexpected prediction length: ",
      length(surv_vec), " (expected ", n_new * n_t, ")."
    )
  }

  surv_raw <- matrix(surv_vec, nrow = n_new, ncol = n_t, byrow = TRUE)

  # 6) Interpolate to requested new.times (step-function style)
  #    yleft=1 ensures S(t)=1 before first event time.
  pred <- t(apply(surv_raw, 1, function(y) {
    stats::approx(
      x = unique_times, y = y, xout = new.times,
      method = "constant", f = 0, rule = 2, ties = mean,
      yleft = 1
    )$y
  }))

  # 7) Safety clamps + monotonicity
  pred[pred < 0] <- 0
  pred[pred > 1] <- 1
  if (ncol(pred) > 1) pred <- t(apply(pred, 1, cummin))

  fit_obj <- list(
    object = fit,
    times  = unique_times,
    stats  = list(features = colnames(X_mat), n_t = n_t)
  )
  class(fit_obj) <- c("surv.bart")

  list(pred = pred, fit = fit_obj)
}



#' Prediction function for BART
#'
#' Obtains predicted survivals from a fitted \code{surv.bart} object.
#' Manually expands the time grid to bypass the 10-column expectation.
#'
#' @param object Fitted \code{surv.bart} object.
#' @param newdata New covariate data.frame to predict on.
#' @param new.times Times to predict.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' \dontrun{
#' if (.Platform$OS.type != "windows" &&
#'   requireNamespace("BART", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:20, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.bart(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     ntree = 3,
#'     ndpost = 5,
#'     nskip = 5
#'   )
#'
#'   pred <- fit[["pred"]]
#'   dim(pred)
#' }
#' }
#' @noRd
#' @export
predict.surv.bart <- function(object, newdata, new.times, ...) {

  # 1) Align features
  expected_cols <- object$stats$features
  newdata_mat <- stats::model.matrix(~ . - 1, data = as.data.frame(newdata))

  missing_cols <- setdiff(expected_cols, colnames(newdata_mat))
  if (length(missing_cols) > 0) {
    pad_mat <- matrix(0, nrow = nrow(newdata_mat), ncol = length(missing_cols))
    colnames(pad_mat) <- missing_cols
    newdata_mat <- cbind(newdata_mat, pad_mat)
  }
  newdata_mat <- newdata_mat[, expected_cols, drop = FALSE]

  # 2) Predict survival on the BART-native time grid
  #    (This is the least fragile approach; avoid manual "(t, X)" expansion.)
  p_obj <- tryCatch(
    {
      # Many BART objects support x.test style
      predict(object$object, x.test = newdata_mat)
    },
    error = function(e) NULL
  )

  if (is.null(p_obj)) {
    stop(
      "BART survival predictions on new data are not supported for this fitted object. ",
      "In SuperSurv, the BART wrapper is intended primarily for CV-time predictions ",
      "(via x.test during model fitting). If you need out-of-sample predictions, ",
      "consider using other learners or refitting BART with x.test=newdata inside the wrapper."
    )
  }

  # 3) Extract time grid + survival
  # Try common fields (these vary by version)
  if (!is.null(p_obj$times)) {
    bart_times <- p_obj$times
  } else if (!is.null(object$times)) {
    bart_times <- object$times
  } else {
    stop("Cannot find BART prediction time grid.")
  }

  # Survival output candidates
  surv_raw <- NULL
  if (!is.null(p_obj$surv.test.mean)) {
    # common
    surv_raw <- p_obj$surv.test.mean
  } else if (!is.null(p_obj$surv.mean)) {
    surv_raw <- p_obj$surv.mean
  } else if (!is.null(p_obj$surv)) {
    surv_raw <- p_obj$surv
  }

  if (is.null(surv_raw)) stop("Cannot find survival predictions in BART predict output.")

  # Ensure matrix shape: n_new x n_time
  n_new <- nrow(newdata_mat)
  n_t   <- length(bart_times)
  if (is.null(dim(surv_raw))) {
    surv_raw <- matrix(surv_raw, nrow = n_new, ncol = n_t, byrow = TRUE)
  }

  # 4) Interpolate to requested new.times (step function, survival monotone)
  pred <- t(apply(surv_raw, 1, function(y) {
    stats::stepfun(bart_times, c(1, y), right = FALSE)(new.times)
  }))

  pred[pred < 0] <- 0
  pred[pred > 1] <- 1
  if (ncol(pred) > 1) pred <- t(apply(pred, 1, cummin))

  pred
}









#' Wrapper for AORSF (Oblique Random Survival Forest)
#'
#' Final Production Wrapper for AORSF (Tunable & Robust).
#'
#' @param time Observed follow-up time; i.e. minimum of the event and censoring times.
#' @param event Observed event indicator; i.e, whether the follow-up time corresponds to an event or censoring.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame to use for prediction. Should have the same variable names and structure as \code{X}.
#' @param new.times Times at which to obtain the predicted survivals.
#' @param obsWeights Observation weights.
#' @param id Optional cluster/individual ID indicator.
#' @param n_tree Number of trees to grow (default: 500).
#' @param leaf_min_events Minimum number of events in a leaf node (default: 5).
#' @param mtry Number of predictors evaluated at each node.
#' @param ... Additional arguments passed to \code{\link[aorsf]{orsf}}.
#' @return A list containing:
#' \itemize{
#'   \item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
#'     If the model fails to fit, this may be an object of class \code{try-error}.
#'   \item \code{pred}: A numeric matrix of cross-validated survival predictions
#'     evaluated at the specified \code{new.times} grid.
#' }

#' @examples
#' if (requireNamespace("aorsf", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.aorsf(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     n_tree = 10,
#'     leaf_min_events = 2
#'   )
#'
#'   dim(fit[["pred"]])
#' }
#' @export
surv.aorsf <- function(time, event, X, newdata, new.times, obsWeights, id,
                       n_tree = 500, leaf_min_events = 5, mtry = NULL, ...) {

  requireNamespace("aorsf", quietly = TRUE)

  # 1. Prepare Data
  dat <- data.frame(time = time, status = event, X)

  # Handle weights safely
  if(missing(obsWeights) || is.null(obsWeights)) obsWeights <- rep(1, length(time))

  # 2. Fit Model
  # We construct a list of arguments for aorsf::orsf
  # We use this structure so that if mtry is NULL, aorsf uses its native default
  args_list <- list(
    data = dat,
    formula = time + status ~ .,
    weights = obsWeights,
    n_tree = n_tree,
    leaf_min_events = leaf_min_events,
    oobag_pred_type = "surv"  # Forced to 'surv' for SuperLearner compatibility
  )

  # Only add mtry if it was explicitly provided, otherwise let aorsf decide
  if (!is.null(mtry)) {
    args_list$mtry <- mtry
  }

  # Append any extra '...' arguments
  args_list <- c(args_list, list(...))

  # Execute the call safely
  fit <- do.call(aorsf::orsf, args_list)

  # 3. Predict Directly at new.times
  preds_surv <- predict(
    fit,
    new_data = newdata,
    pred_horizon = new.times,
    pred_type = "surv"
  )

  # 4. Format Output & Safety (Monotonicity)
  pred_mat <- as.matrix(preds_surv)
  if (ncol(pred_mat) > 1) {
    pred_mat <- t(apply(pred_mat, 1, cummin))
  }

  fit_obj <- list(object = fit)
  class(fit_obj) <- c("surv.aorsf")

  list(pred = pred_mat, fit = fit_obj)
}


#' Prediction function for AORSF
#'
#' Obtains predicted survivals from a fitted \code{surv.aorsf} object.
#' Uses the native \code{aorsf} prediction engine to calculate survival directly at requested times.
#'
#' @param object Fitted \code{surv.aorsf} object.
#' @param newdata New covariate data.frame.
#' @param new.times Times at which to obtain predicted survivals.
#' @param ... Additional ignored arguments.
#' @return A numeric matrix of predicted survival probabilities, where rows correspond
#'   to the observations in \code{newdata} and columns correspond to the evaluation
#'   times in \code{new.times}.
#' @examples
#' if (requireNamespace("aorsf", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:30, ]
#'   x_cols <- grep("^x", names(dat))[1:3]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:5, , drop = FALSE]
#'   times <- seq(50, 150, by = 50)
#'
#'   fit <- surv.aorsf(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = newX,
#'     new.times = times,
#'     obsWeights = rep(1, nrow(dat)),
#'     id = NULL,
#'     n_tree = 10,
#'     leaf_min_events = 2
#'   )
#'
#'   pred <- predict(fit[["fit"]], newdata = newX, new.times = times)
#'   dim(pred)
#' }
#' @noRd
#' @export
predict.surv.aorsf <- function(object, newdata, new.times, ...) {

  # 1. Predict directly at the requested new.times
  # object$object is the actual 'orsf' model
  preds_surv <- predict(
    object$object,
    new_data = newdata,
    pred_horizon = new.times,
    pred_type = "surv"
  )

  # 2. Format as Matrix
  out <- as.matrix(preds_surv)

  # 3. Safety Check: Dimensions
  # If new.times was a single number, ensure it's a column matrix
  if (is.null(dim(out))) {
    out <- matrix(out, nrow = nrow(newdata), ncol = length(new.times))
  }

  out[out < 0] <- 0
  out[out > 1] <- 1

  # 4. Safety Check: Monotonicity
  # Ensure probability never increases over time (S(t) must go down)
  if (ncol(out) > 1) {
    out <- t(apply(out, 1, cummin))
  }

  return(out)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

new.times <- seq(50, 200, by = 25) 

## ----standalone-rf------------------------------------------------------------
# 1. Fit the standalone wrapper
rf_standalone <- surv.rfsrc(
  time = train$duration,
  event = train$event,
  X = X_tr,
  new.times = new.times 
)

# 2. Extract the fitted model object and prediction matrix
rf_fit <- rf_standalone[["fit"]]
rf_pred_matrix <- rf_standalone[["pred"]]

## ----plot-standalone, fig.align='center'--------------------------------------
# Plot the first 3 patients in our training set
plot_predict(preds = rf_pred_matrix, eval_times = new.times, patient_idx = 1:3)

## ----eval-standalone, fig.align='center', fig.height=4, fig.width= 9----------
# The function automatically detects this is a single model and plots it!
plot_benchmark(
  object = rf_fit,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

## -----------------------------------------------------------------------------

# plot_calibration(
#   object   = rf_fit,
#   newdata  = X_te,
#   time     = test$duration,
#   event    = test$event,
#   eval_time = 150,
#   bins     = 2
# )


## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
my_library <- c("surv.coxph", "surv.weibull", "surv.rfsrc")

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = c("surv.coxph"), 
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)

## ----plot-ensemble-benchmark, fig.align='center', fig.height=9----------------
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

#' Predict method for SuperSurv fits
#'
#' Obtains predicted survival probabilities from a fitted SuperSurv ensemble.
#'
#' @param object A fitted object of class \code{SuperSurv}.
#' @param newdata A data.frame of new covariate values.
#' @param new.times A numeric vector of times at which to predict survival.
#' @param type Character string specifying the prediction output. Use
#'   \code{"event"} for the event survival matrix, \code{"censoring"} for the
#'   censoring survival matrix, or \code{"both"} for the full list of outputs.
#' @param onlySL Logical. If TRUE, only uses models with weights > threshold.
#' @param threshold Numeric. The weight threshold for onlySL.
#' @param ... Additional ignored arguments.
#' @return If \code{type = "event"} or \code{type = "censoring"}, a numeric
#'   matrix with rows corresponding to observations and columns corresponding to
#'   \code{new.times}. If \code{type = "both"}, a list containing:
#' \itemize{
#'   \item \code{event.predict}: A numeric matrix of final event survival predictions.
#'   \item \code{event.library.predict}: A 3D numeric array of event learner predictions.
#'   \item \code{cens.predict}: A numeric matrix of final censoring survival predictions.
#'   \item \code{cens.library.predict}: A 3D numeric array of censoring learner predictions.
#' }
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   newX <- X[1:10, , drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   pred_event <- predict(
#'     object = fit,
#'     newdata = newX,
#'     new.times = new.times,
#'     type = "event"
#'   )
#'
#'   dim(pred_event)
#' }
#' @export
predict.SuperSurv <- function (object, newdata, new.times,
                               type = c("both", "event", "censoring"),
                               onlySL = FALSE, threshold = 1e-4, ...) {
  type <- match.arg(type)

  # 1. Return training predictions if no new data is provided
  if (missing(newdata)) {
    out <- list(
      event.predict      = object$event.predict,
      cens.predict       = object$cens.predict,
      event.library.predict = object$event.library.predict,
      cens.library.predict  = object$cens.library.predict
    )
    return(.predict_SuperSurv_output(out, type = type))
  }

  if (missing(new.times)) stop("new.times must be specified for new predictions.")

  # 2. Safety Check
  if (!object$control$saveFitLibrary) {
    stop("This SuperSurv fit was created using control$saveFitLibrary = FALSE. New predictions cannot be made.")
  }

  # 3. Setup Arrays
  event.k <- nrow(object$event.libraryNames)
  event.pred <- array(0, dim=c(nrow(newdata), length(new.times), event.k))
  dimnames(event.pred)[[3]] <- apply(object$event.libraryNames, 1, paste, collapse = '_')
  dimnames(event.pred)[[2]] <- new.times

  cens.k <- nrow(object$cens.libraryNames)
  cens.pred <- array(0, dim=c(nrow(newdata), length(new.times), cens.k))
  dimnames(cens.pred)[[3]] <- apply(object$cens.libraryNames, 1, paste, collapse = '_')
  dimnames(cens.pred)[[2]] <- new.times

  # 4. Filter by Threshold (onlySL)
  if (onlySL) {
    event.whichLibrary <- which(object$event.coef > threshold)
    if(length(event.whichLibrary) == 0) event.whichLibrary <- which(object$event.coef > 0) # Safety
    event.coef <- object$event.coef
    event.coef[-event.whichLibrary] <- 0
    event.coef <- event.coef / sum(event.coef)

    cens.whichLibrary <- which(object$cens.coef > threshold)
    if(length(cens.whichLibrary) == 0) cens.whichLibrary <- which(object$cens.coef > 0) # Safety
    cens.coef <- object$cens.coef
    cens.coef[-cens.whichLibrary] <- 0
    cens.coef <- cens.coef / sum(cens.coef)
  } else {
    event.whichLibrary <- seq(event.k)
    event.coef <- object$event.coef
    cens.whichLibrary <- seq(cens.k)
    cens.coef <- object$cens.coef
  }

  # ----------------------------------------------------------------------------
  # 5. Predict Event Models
  # ----------------------------------------------------------------------------
  for (mm in event.whichLibrary) {
    # Apply Variable Screening (if any)
    newdataMM <- subset(newdata, select = object$event.whichScreen[object$event.library$library[mm, 2], ])

    # Call our specific base learner prediction wrappers
    event.pred[, , mm] <- do.call("predict", list(
      object    = object$event.fitLibrary[[mm]],
      newdata      = newdataMM,
      new.times = new.times,
      ...
    ))
  }

  # Combine Event Learners (Using the 2D matrix force fix!)
  event.predict <- matrix(NA_real_, nrow = nrow(newdata), ncol = length(new.times))
  K_event <- length(event.coef)

  for (j in seq_along(new.times)) {
    tmp_mat <- matrix(event.pred[, j, , drop = FALSE], nrow = nrow(newdata), ncol = K_event)
    event.predict[, j] <- tmp_mat %*% event.coef
  }

  # ----------------------------------------------------------------------------
  # 6. Predict Censoring Models
  # ----------------------------------------------------------------------------
  for (mm in cens.whichLibrary) {
    # Apply Variable Screening (if any)
    newdataMM <- subset(newdata, select = object$cens.whichScreen[object$cens.library$library[mm, 2], ])

    # Call our specific base learner prediction wrappers
    cens.pred[, , mm] <- do.call("predict", list(
      object    = object$cens.fitLibrary[[mm]],
      newdata      = newdataMM,
      new.times = new.times,
      ...
    ))
  }

  # Combine Censoring Learners
  cens.predict <- matrix(NA_real_, nrow = nrow(newdata), ncol = length(new.times))
  K_cens <- length(cens.coef)

  for (j in seq_along(new.times)) {
    tmp_mat <- matrix(cens.pred[, j, , drop = FALSE], nrow = nrow(newdata), ncol = K_cens)
    cens.predict[, j] <- tmp_mat %*% cens.coef
  }

  # Ensure strict probability bounds to fix floating-point drift
  event.predict <- pmax(pmin(event.predict, 1), 0)
  cens.predict <- pmax(pmin(cens.predict, 1), 0)

  # ----------------------------------------------------------------------------
  # 7. Output
  # ----------------------------------------------------------------------------
  out <- list(
    event.predict = event.predict,
    event.library.predict = event.pred,
    cens.predict = cens.predict,
    cens.library.predict = cens.pred
  )
  return(.predict_SuperSurv_output(out, type = type))
}

.predict_SuperSurv_output <- function(out, type) {
  switch(
    type,
    both = out,
    event = out$event.predict,
    censoring = out$cens.predict
  )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Create 100 columns of random noise (simulating irrelevant genes/biomarkers)
n_patients <- nrow(metabric)
noise_data <- matrix(rnorm(n_patients * 20), nrow = n_patients)
colnames(noise_data) <- paste0("noise_", 1:20)
# Combine the real data with the noise
high_dim_data <- cbind(metabric, noise_data)

# Standard Train/Test Split
train_idx <- sample(1:nrow(high_dim_data), 0.7 * nrow(high_dim_data))
train <- high_dim_data[train_idx, ]
test  <- high_dim_data[-train_idx, ]

X_tr <- train[, grep("^x|^noise", names(high_dim_data))]
X_te <- test[, grep("^x|^noise", names(high_dim_data))]
new.times <- seq(50, 200, by = 25)

## ----define-library-----------------------------------------------------------
# We pass a list of vectors. The first element is the prediction model, 
# and the second element is the screening algorithm.

my_screen_library <- list(
  c("surv.coxph", "screen.all"),         # Baseline: Cox model with ALL variables
  c("surv.coxph", "screen.marg"),        # Screen by marginal association, then fit Cox
  c("surv.weibull", "screen.elasticnet"),# Screen via Elastic Net, then fit Weibull
  c("surv.rpart", "screen.var")          # Drop zero-variance noise, then fit a Tree
)

# For the censoring library, we use an unscreened approach 
cens_library <- c("surv.coxph")

## ----fit-screen, results='hide', message=FALSE, warning=FALSE-----------------
fit_highdim <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_screen_library, # Pass our screened library
  cens.library = cens_library,
  control = list(saveFitLibrary = TRUE),
  verbose = T,
  nFolds = 3
)

## ----evaluate-screen----------------------------------------------------------
# Check the integrated performance
screened_performance <- eval_summary(
  object = fit_highdim,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

# Plot the benchmark
# plot_benchmark(fit_highdim, newdata = X_te, time = test$duration, event = test$event, eval_times = new.times)

## ----extract-features---------------------------------------------------------
# Get the variables retained by the second event learner.
selected_features <- selected_variables(fit_highdim, learner = 2)

cat("Total features evaluated:", ncol(X_tr), "\n")
cat("Features retained by Elastic Net:", length(selected_features), "\n\n")

cat("Selected Features:\n")
print(selected_features)

#' Super Learner for conditional survival functions
#'
#' Orchestrates the cross-validation, metalearner optimization, and prediction
#' for an ensemble of survival base learners.
#'
#' @details
#' \strong{Extending the learner library.} Custom base learners can be added by
#' defining a wrapper function and passing its name in \code{event.library} or
#' \code{cens.library}. A learner wrapper should accept \code{time},
#' \code{event}, \code{X}, \code{newdata}, \code{new.times},
#' \code{obsWeights}, \code{id}, and \code{...}, and should return a list with
#' \code{pred}, a numeric survival-probability matrix with
#' \code{nrow(newdata)} rows and \code{length(new.times)} columns, and
#' \code{fit}, the fitted object used for future prediction. If saved fits are
#' needed, give \code{fit} a class and provide a corresponding
#' \code{predict.<class>()} method that returns the same matrix shape.
#'
#' Screening methods can also be supplied by name. A screener should accept the
#' training inputs and return a logical vector aligned with the columns of
#' \code{X}. See \code{vignette("extending-supersurv", package = "SuperSurv")}
#' for a practical custom learner and screener example.
#'
#' @param time Observed follow-up time.
#' @param event Observed event indicator.
#' @param X Training covariate data.frame.
#' @param newdata Test covariate data.frame for prediction (defaults to X).
#' @param new.times Times at which to obtain predicted survivals.
#' @param event.library Character vector of prediction algorithms for the event.
#' @param cens.library Character vector of prediction algorithms for censoring.
#' @param id Cluster identification variable.
#' @param verbose Logical. If TRUE, prints progress messages.
#' @param control List of control parameters for the Super Learner.
#' @param cvControl List of control parameters for cross-validation.
#' @param obsWeights Observation weights.
#' @param metalearner Character string specifying the optimizer (e.g., "brier" or "logloss").
#' @param parallel Logical. If TRUE, uses future.apply for parallel execution.
#' @param selection Character. Specifies how the meta-learner combines the base models.
#'   Use \code{"ensemble"} (default) to calculate a weighted average (convex combination)
#'   of the base learners. Use \code{"best"} to act as a Discrete Super Learner, which
#'   assigns a weight of 1.0 to the single model with the lowest cross-validated risk.
#' @param nFolds Number of cross-validation folds (default: 10).
#' @return A list of class \code{SuperSurv} containing:
#' \itemize{
#'   \item \code{call}: The matched function call.
#'   \item \code{event.predict}: Matrix of in-sample cross-validated survival predictions.
#'   \item \code{cens.predict}: Matrix of in-sample cross-validated censoring predictions.
#'   \item \code{eval.times}: Numeric vector of prediction evaluation times.
#'   \item \code{event.coef}: Numeric vector of optimized ensemble weights for the event.
#'   \item \code{cens.coef}: Numeric vector of optimized ensemble weights for censoring.
#'   \item \code{event.library.predict}: 3D array of cross-validated predictions from individual event learners.
#'   \item \code{event.libraryNames}: Data frame detailing the algorithms and screeners used.
#'   \item \code{event.fitLibrary}: List of the fitted base learner models (if \code{saveFitLibrary = TRUE}).
#'   \item \code{times}: The time grid used for evaluation.
#' }
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   data("metabric", package = "SuperSurv")
#'   dat <- metabric[1:80, ]
#'   x_cols <- grep("^x", names(dat))[1:5]
#'   X <- dat[, x_cols, drop = FALSE]
#'   new.times <- seq(20, 120, by = 20)
#'
#'   fit <- SuperSurv(
#'     time = dat$duration,
#'     event = dat$event,
#'     X = X,
#'     newdata = X,
#'     new.times = new.times,
#'     event.library = c("surv.coxph", "surv.ridge"),
#'     cens.library = c("surv.coxph"),
#'     control = list(saveFitLibrary = TRUE)
#'   )
#'
#'   fit
#'   event_weights(fit)
#' }
#' @export
SuperSurv <- function(time, event, X, newdata = NULL, new.times,
                      event.library, cens.library,
                      id = NULL, verbose = FALSE,
                      control = list(), cvControl = list(),
                      obsWeights = NULL, metalearner = "brier",
                      selection = "ensemble", nFolds = 10,parallel = FALSE )  {

  # ----------------------------------------------------------------------------
  # 1. Setup & Checks
  # ----------------------------------------------------------------------------
  time_start <- proc.time()

  if (!is.null(dim(time)) && ncol(time) > 0) stop("time must be an (n x 1) numeric vector.")
  if (!is.null(dim(event)) && ncol(event) > 0) stop("event must be an (n x 1) numeric vector.")
  time <- as.numeric(time)
  event <- as.numeric(event)
  if (is.null(obsWeights)) obsWeights <- rep(1, length(time))
  if (is.null(newdata)) newdata <- X
  metalearner <- match.arg(metalearner, choices = c("brier", "logloss", "entropy"))

  varNames <- colnames(X)
  N <- dim(X)[1L]
  p <- dim(X)[2L]

  # Internal Checker (from internals.R)
  .checkInputs(time=time, event=event, X=X, newdata=newdata, id=id, obsWeights=obsWeights, verbose=verbose)

  # ----------------------------------------------------------------------------
  # 2. Parallel Setup
  # ----------------------------------------------------------------------------
  if (parallel) {
    if (!requireNamespace("future.apply", quietly = TRUE)) {
      stop("Package 'future.apply' is required for parallel = TRUE. Please install it.")
    }
    lapply_fun <- future.apply::future_lapply
    parallel_args <- list(future.seed = TRUE)
  } else {
    lapply_fun <- lapply
    parallel_args <- list()
  }

  # ----------------------------------------------------------------------------
  # 3. Time Grid Setup (Event) - With Memory Fix
  # ----------------------------------------------------------------------------
  if (is.null(control$event.t.grid)) {
    control$event.t.grid <- seq(0, max(time[event == 1]), length.out = 250)
  } else {
    if (!is.null(dim(control$event.t.grid)) && ncol(control$event.t.grid) > 0)
      stop("control$event.t.grid must be a numeric vector.")
    control$event.t.grid <- sort(unique(as.numeric(control$event.t.grid)))
    if(any(is.na(control$event.t.grid))) stop("No missing values allowed in event.t.grid")
    if (any(control$event.t.grid < 0)) stop("Values in event.t.grid must be non-negative")
    if (any(control$event.t.grid > max(time))) stop("Values in event.t.grid must not exceed max(time)")
  }

  if(length(control$event.t.grid) > 500) {
    if(verbose) warning("event.t.grid > 500 points. Reducing to quantiles for memory safety.")
    control$event.t.grid <- quantile(control$event.t.grid, probs = seq(0, 1, length.out = 500))
  }

  # ----------------------------------------------------------------------------
  # 4. Time Grid Setup (Censoring) - With Left-Continuity & Memory Fix
  # ----------------------------------------------------------------------------
  if (is.null(control$cens.t.grid)) {
    control$cens.t.grid <- seq(0, max(time[event == 0]), length.out = 250)
    epsilon <- max(min(diff(sort(unique(time)))) / 2, 1e-5)
    control$cens.t.grid <- control$cens.t.grid - epsilon
    control$cens.t.grid <- c(0, control$cens.t.grid[control$cens.t.grid > 0])
  } else {
    control$cens.t.grid <- sort(unique(as.numeric(control$cens.t.grid)))
    if(any(is.na(control$cens.t.grid))) stop("No missing values allowed in cens.t.grid")
    if (any(control$cens.t.grid < 0)) stop("Values in cens.t.grid must be non-negative")
    if (any(control$cens.t.grid > max(time))) stop("Values in cens.t.grid must not exceed max(time)")
  }

  if(length(control$cens.t.grid) > 500) {
    if(verbose) warning("cens.t.grid > 500 points. Reducing to quantiles for memory safety.")
    control$cens.t.grid <- quantile(control$cens.t.grid, probs = seq(0, 1, length.out = 500))
  }

  # ----------------------------------------------------------------------------
  # 5. Libraries & Folds
  # ----------------------------------------------------------------------------
  if (missing(cvControl)) cvControl <- list()
  cvControl$V <- nFolds

  control <- do.call("SuperSurv.control", control)
  cvControl <- do.call("SuperSurv.CV.control", cvControl)

  .validate_library_argument(event.library, "event.library")
  .validate_library_argument(cens.library, "cens.library")
  event.library <- .createLibrary(event.library)
  cens.library <- .createLibrary(cens.library)

  event.k <- nrow(event.library$library)
  cens.k <- nrow(cens.library$library)
  event.kScreen <- length(event.library$screenAlgorithm)
  cens.kScreen <- length(cens.library$screenAlgorithm)

  event.Z <- array(NA, dim = c(N, length(control$event.t.grid), event.k))
  cens.Z <- array(NA, dim = c(N, length(control$cens.t.grid), cens.k))

  event.libraryNames <- cbind(predAlgorithm = event.library$library$predAlgorithm,
                              screenAlgorithm = event.library$screenAlgorithm[event.library$library$rowScreen])
  cens.libraryNames <- cbind(predAlgorithm = cens.library$library$predAlgorithm,
                             screenAlgorithm = cens.library$screenAlgorithm[cens.library$library$rowScreen])

  if (p < 2 & !identical(event.library$screenAlgorithm, "All")) warning("Screening with p < 2.")
  if (p < 2 & !identical(cens.library$screenAlgorithm, "All")) warning("Screening with p < 2.")

  validRows <- .survCVFolds(N = N, id = id, event = event, cvControl = cvControl)
  if (is.null(id)) id <- seq(N)

  # ----------------------------------------------------------------------------
  # 6. Cross-Validation (Parallelized)
  # ----------------------------------------------------------------------------
  time_train_start <- proc.time()

  # --- Event CV ---
  if(verbose) message("Running Cross-Validation for Event Models...")

  event_args <- c(list(X = validRows, FUN = .crossValFUN,
                       validRows = validRows,
                       time = time, event = event, dataX = X, id = id, obsWeights = obsWeights,
                       t.grid = control$event.t.grid, library = event.library,
                       kScreen = event.kScreen, k = event.k, p = p, verbose = verbose),
                  parallel_args)




  event.crossValFUN_out <- do.call(lapply_fun, event_args)

  for(v in 1:cvControl$V) {
    event.Z[validRows[[v]], ,] <- event.crossValFUN_out[[v]]$out
  }

  event.errorsInCVLibrary <- apply(event.Z, 3, anyNA)
  if (sum(event.errorsInCVLibrary) > 0) event.Z[, , as.logical(event.errorsInCVLibrary)] <- 0
  if (all(event.Z == 0)) stop("All algorithms dropped from event library")

  # --- Censoring CV ---
  if(verbose) message("Running Cross-Validation for Censoring Models...")

  cens_args <- c(list(X = validRows, FUN = .crossValFUN,
                      validRows = validRows,
                      time = time, event = 1-event, dataX = X, id = id, obsWeights = obsWeights,
                      t.grid = control$cens.t.grid, library = cens.library,
                      kScreen = cens.kScreen, k = cens.k, p = p, verbose = verbose),
                 parallel_args)

  cens.crossValFUN_out <- do.call(lapply_fun, cens_args)

  for(v in 1:cvControl$V) {
    cens.Z[validRows[[v]], ,] <- cens.crossValFUN_out[[v]]$out
  }

  cens.errorsInCVLibrary <- apply(cens.Z, 3, anyNA)
  if (sum(cens.errorsInCVLibrary) > 0) cens.Z[, , as.logical(cens.errorsInCVLibrary)] <- 0
  if (all(cens.Z == 0)) stop("All algorithms dropped from censoring library")

  # ----------------------------------------------------------------------------
  # 7. Metalearner (With Selection)
  # ----------------------------------------------------------------------------
  getCoef <- .surviterativeSL(event.Z = event.Z, cens.Z = cens.Z, time = time, event = event, X = X,
                              obsWeights = obsWeights, id = id, control = control, verbose = verbose,
                              event.errorsInLibrary = event.errorsInCVLibrary,
                              cens.errorsInLibrary = cens.errorsInCVLibrary,
                              metalearner = metalearner)

  # if(verbose) {
  #   message("DEBUG: Metalearner coefficients returned.")
  # }

  event.coef <- getCoef$event.coef
  cens.coef <- getCoef$cens.coef
  event.cvRisks <- getCoef$event.cvRisks
  cens.cvRisks <- getCoef$cens.cvRisks

  names(event.coef) <- names(event.cvRisks) <- apply(event.libraryNames, 1, paste, collapse = "_")
  names(cens.coef) <- names(cens.cvRisks) <- apply(cens.libraryNames, 1, paste, collapse = "_")

  time_train <- proc.time() - time_train_start

  # ----------------------------------------------------------------------------
  # 8. Full Fits (Prediction on newdata)
  # ----------------------------------------------------------------------------
  if(verbose) message("Fitting full models on entire training set...")
  time_predict_start <- proc.time()

  # --- SCREENING ---
  event.whichScreen <- do.call(rbind, lapply(event.library$screenAlgorithm, FUN = .screenFun,
                                             list = list(time = time, event = event, id = id, X = X, obsWeights = obsWeights)))

  cens.whichScreen <- do.call(rbind, lapply(cens.library$screenAlgorithm, FUN = .screenFun,
                                            list = list(time = time, event = 1 - event, id = id, X = X, obsWeights = obsWeights)))

  # --- EVENT MODELS FIT ---
  event.pred <- lapply(seq(event.k), FUN = .predFun,
                       lib = event.library$library, time = time, event = event,
                       dataX = X, newdata = newdata, t.grid = new.times,
                       whichScreen = event.whichScreen, id = id,
                       obsWeights = obsWeights, verbose = verbose, control = control,
                       libraryNames = event.libraryNames)



  # Initialize 3D Array: [Patients x Times x Models]
  event.libraryPred <- array(NA, dim = c(nrow(newdata), length(new.times), event.k))

  # [SAFETY FIX] Loop with check to prevent crash on failed models
  for (j in 1:event.k) {
    if (!is.null(event.pred[[j]]$out)) {
      event.libraryPred[, , j] <- event.pred[[j]]$out
    }
  }

  # Flag models that produced NAs (or were NULL/skipped)
  event.errorsInLibrary <- apply(event.libraryPred, 3, anyNA)

  # --- CENSORING MODELS FIT ---
  cens.pred <- lapply(seq(cens.k), FUN = .predFun,
                      lib = cens.library$library, time = time, event = 1 - event,
                      dataX = X, newdata = newdata, t.grid = new.times,
                      whichScreen = cens.whichScreen, id = id,
                      obsWeights = obsWeights, verbose = verbose, control = control,
                      libraryNames = cens.libraryNames)

  cens.libraryPred <- array(NA, dim = c(nrow(newdata), length(new.times), cens.k))

  for (j in 1:cens.k) {
    if (!is.null(cens.pred[[j]]$out)) {
      cens.libraryPred[, , j] <- cens.pred[[j]]$out
    }
  }

  cens.errorsInLibrary <- apply(cens.libraryPred, 3, anyNA)

  # ----------------------------------------------------------------------------
  # 8b. 'Best' Model Selection Logic (Winner-Take-All Override)
  # ----------------------------------------------------------------------------
  if (selection == "best") {
    if(verbose) message("Best Model Selection: Selecting the single best learner...")

    # Event Winner
    best_idx_event <- which.min(event.cvRisks)
    event.coef <- rep(0, length(event.cvRisks))
    event.coef[best_idx_event] <- 1
    if(verbose) message(paste0("  Event Winner: ", names(event.cvRisks)[best_idx_event]))

    # Censoring Winner
    best_idx_cens <- which.min(cens.cvRisks)
    cens.coef <- rep(0, length(cens.cvRisks))
    cens.coef[best_idx_cens] <- 1
    if(verbose) message(paste0("  Censoring Winner: ", names(cens.cvRisks)[best_idx_cens]))

  } else if (selection != "ensemble") {
    # Safety catch in case the user types something like selection = "single"
    stop("Invalid 'selection' argument. Please use 'ensemble' or 'best'.")
  }

  # ----------------------------------------------------------------------------
  # 9. Final Ensemble & Output
  # ----------------------------------------------------------------------------

  # --- EVENT ENSEMBLE ---
  if (event.k == 1) {
    # If only one model exists, just take its prediction matrix
    event.predict <- event.libraryPred[, , 1]
  } else {
    event.predict <- matrix(NA_real_, nrow = nrow(newdata), ncol = length(new.times))

    # Identify which models in the library actually finished without error
    valid_idx <- !event.errorsInLibrary
    if (sum(valid_idx) == 0) stop("All event models failed prediction.")

    # Extract coefficients for ONLY the valid models and re-normalize if necessary
    # (Though NNLS usually handles this, we be safe)
    current_event_coef <- event.coef[valid_idx]

    for (j in seq_along(new.times)) {
      # [CRITICAL FIX]: Force to 2D matrix to prevent "non-conformable" errors
      # This handles cases where sum(valid_idx) is 1 or many.
      tmp_mat <- matrix(event.libraryPred[, j, valid_idx, drop = FALSE],
                        nrow = nrow(newdata),
                        ncol = sum(valid_idx))

      event.predict[, j] <- tmp_mat %*% current_event_coef
    }
  }

  # --- CENSORING ENSEMBLE ---
  if (cens.k == 1) {
    cens.predict <- cens.libraryPred[, , 1]
  } else {
    cens.predict <- matrix(NA_real_, nrow = nrow(newdata), ncol = length(new.times))

    valid_idx <- !cens.errorsInLibrary
    if (sum(valid_idx) == 0) stop("All censoring models failed prediction.")

    current_cens_coef <- cens.coef[valid_idx]

    for (j in seq_along(new.times)) {
      # [CRITICAL FIX]: Force to 2D matrix
      tmp_mat <- matrix(cens.libraryPred[, j, valid_idx, drop = FALSE],
                        nrow = nrow(newdata),
                        ncol = sum(valid_idx))

      cens.predict[, j] <- tmp_mat %*% current_cens_coef
    }
  }

  # --- TIMING & METADATA ---
  time_predict <- proc.time() - time_predict_start
  time_end <- proc.time()

  times <- list(everything = time_end - time_start,
                train = time_train,
                predict = time_predict)

  # --- CONSTRUCT FIT LIBRARIES ---
  if (control$saveFitLibrary) {
    # Extract the actual model objects from the wrapper output lists
    event.fitLibrary <- lapply(event.pred, "[[", "model_out")
    names(event.fitLibrary) <- apply(event.libraryNames, 1, paste, collapse = "_")

    cens.fitLibrary <- lapply(cens.pred, "[[", "model_out")
    names(cens.fitLibrary) <- apply(cens.libraryNames, 1, paste, collapse = "_")
  } else {
    event.fitLibrary <- NULL
    cens.fitLibrary <- NULL
  }

  # ----------------------------------------------------------------------------
  # 10. Return List (Full Diagnostic Suite)
  # ----------------------------------------------------------------------------
  out <- list(
    call = match.call(),
    event.predict = event.predict,
    cens.predict = cens.predict,
    eval.times = new.times,
    event.coef = event.coef,
    cens.coef = cens.coef,
    event.library.predict = event.libraryPred,
    cens.library.predict = cens.libraryPred,
    event.libraryNames = event.libraryNames,
    cens.libraryNames = cens.libraryNames,
    event.library = event.library,
    cens.library = cens.library,
    event.cvRisks = event.cvRisks,
    cens.cvRisks = cens.cvRisks,
    event.errorsInCVLibrary = event.errorsInCVLibrary,
    cens.errorsInCVLibrary = cens.errorsInCVLibrary,
    event.errorsInLibrary = event.errorsInLibrary,
    cens.errorsInLibrary = cens.errorsInLibrary,
    event.Z = event.Z,
    cens.Z = cens.Z,
    varNames = varNames,
    validRows = validRows,
    event.whichScreen = event.whichScreen,
    cens.whichScreen = cens.whichScreen,
    event.fitLibrary = event.fitLibrary,
    cens.fitLibrary = cens.fitLibrary,
    control = control,
    cvControl = cvControl,
    times = times
  )

  class(out) <- "SuperSurv"
  return(out)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SuperSurv)
library(survival)

# Load built-in METABRIC data
data("metabric", package = "SuperSurv")

# Quick 80/20 Train-Test split
set.seed(42)
n_total <- nrow(metabric)
train_idx <- sample(1:n_total, 0.8 * n_total)

train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

# Extract just the X covariates (assuming they are named x0, x1, etc.)
x_cols <- grep("^x", names(metabric), value = TRUE)
X_tr <- train[, x_cols]
X_te <- test[, x_cols]

# Define the prediction time grid (e.g., survival at 50, 100, 150, 200 months)
new.times <- c(50, 100, 150, 200)

## ----define-library-----------------------------------------------------------
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")

## ----fit-models, results='hide', message=FALSE, warning=FALSE-----------------
# Fit 1: Least Squares Meta-learner
fit_ls <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,                 # Predict on the test set immediately
  new.times = new.times,       # Our evaluation time grid
  event.library = my_library,
  cens.library = my_library,
  metalearner = "brier", 
  control = list(saveFitLibrary = TRUE), 
  verbose = T,             # Turn to TRUE in practice to see progress!
  selection = "ensemble",
  nFolds = 5                   # 5-fold CV for the meta-learner
)

# Fit 2: Negative Log-Likelihood Meta-learner
fit_nll <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  metalearner = "logloss",       # Swap to nloglik
  control = list(saveFitLibrary = TRUE), 
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 5
)

## ----object-interface---------------------------------------------------------
fit_ls

summary(fit_ls)

event_weights(fit_ls)

learner_names(fit_ls)

eval_times(fit_ls)

selected_variables(fit_ls, learner = 1)

## ----inspect-weights----------------------------------------------------------
cat("\n--- LEAST SQUARES METALEARNER ---\n")
summary(fit_ls)

cat("\n--- NLOGLIK METALEARNER ---\n")
summary(fit_nll)

## ----predict-new--------------------------------------------------------------
# Select 3 brand new patients from our test set
new_patients <- X_te[1:6, ]

# Generate predictions using the Least Squares ensemble
ensemble_preds <- predict(
  object = fit_ls, 
  newdata = new_patients, 
  new.times = new.times,
  type = "event"
)

cat("\n--- PREDICTED SURVIVAL PROBABILITIES ---\n")
final_matrix <- ensemble_preds
colnames(final_matrix) <- paste0("Time_", new.times)
rownames(final_matrix) <- paste0("Patient_", 1:6)

print(round(final_matrix, 4))

## ----plot-predict, fig.align='center', fig.width=6, fig.height=5--------------
# Plot the predicted survival curves for our 3 new patients (Rows 1, 2, and 3)
plot_predict(
  preds = ensemble_preds,
  eval_times = new.times,
  patient_idx = 1:6
)

library(SuperSurv)

data("metabric", package = "SuperSurv")
dat <- metabric[1:30, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]

fit <- SuperSurv(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X[1:5, , drop = FALSE],
  new.times = seq(20, 100, by = 20),
  event.library = "surv.km",
  cens.library = "surv.km",
  nFolds = 2,
  verbose = FALSE
)

event_w <- event_weights(fit)
censor_w <- censor_weights(fit)
all_weights <- coef(fit, type = "both")
event_learners <- learner_names(fit)
all_learners <- learner_names(fit, type = "both")
times <- eval_times(fit)
model_vars <- training_variables(fit)
selected_vars <- selected_variables(fit, learner = 1)
fit_summary <- summary(fit)
pred_both <- predict(fit)
pred_event <- predict(fit, type = "event")
pred_censor <- predict(fit, type = "censoring")

stopifnot(
  inherits(fit, "SuperSurv"),
  is.numeric(event_w),
  is.numeric(censor_w),
  length(event_w) == 1L,
  is.list(all_weights),
  identical(all_weights$event, event_w),
  is.numeric(all_weights$censoring),
  identical(event_learners, names(event_w)),
  identical(all_learners$event, event_learners),
  identical(times, seq(20, 100, by = 20)),
  identical(model_vars, colnames(X)),
  identical(selected_vars, colnames(X)),
  is.list(pred_both),
  is.matrix(pred_event),
  is.matrix(pred_censor),
  identical(pred_event, pred_both$event.predict),
  identical(pred_censor, pred_both$cens.predict),
  inherits(fit_summary, "summary.SuperSurv"),
  nrow(fit_summary$event) == 1L,
  nrow(fit_summary$censoring) == 1L
)

print(fit)
print(fit_summary)

safe_detect_cores <- getFromNamespace(".safe_detect_cores", "SuperSurv")

stopifnot(
  identical(safe_detect_cores(function() 2), 2L),
  identical(safe_detect_cores(function() NA_integer_), 1L),
  identical(safe_detect_cores(function() NaN), 1L),
  identical(safe_detect_cores(function() Inf), 1L),
  identical(safe_detect_cores(function() 0), 1L),
  identical(safe_detect_cores(function() c(1, 2)), 1L),
  identical(safe_detect_cores(function() "2"), 1L),
  identical(safe_detect_cores(function() stop("boom")), 1L)
)

---
title: "03. Ensemble vs. Best Model Selection"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{03. Ensemble vs. Best Model Selection}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_rpart <- requireNamespace("rpart", quietly = TRUE)
```

## Introduction

In the theoretical framework of Super Learning, there are two distinct ways to utilize the cross-validated risks of your base algorithms:

1. **The Ensemble Super Learner:** Calculates a weighted average (convex combination) of the base learners. This "soft selection" smooths out variance, integrates different feature spaces, and generally yields the lowest finite-sample prediction error.
2. **The "Best" Model Selector:** Identifies the single algorithm with the lowest cross-validated risk and assigns it a weight of `1.0` (and all others `0.0`). This is a "hard selection" or "winner-take-all" approach. 



While the Ensemble is asymptotically optimal, selecting the single best model is incredibly useful when interpretability is strictly tied to one specific algorithm family. Instead of manually cherry-picking the best model—which introduces researcher bias and invalidates post-selection inference—`SuperSurv` automates the selection using rigorous, internal cross-validation. 

You can easily toggle between these two paradigms using the `selection` argument.

## 1. Prepare the Data

We load the `metabric` dataset and define our evaluation time grid exactly as we did in the previous tutorials.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Standard Train/Test Split
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# Define a diverse library of base learners
my_library <- c("surv.coxph", "surv.weibull")
if (has_rpart) {
  my_library <- c(my_library, "surv.rpart")
}
```

## 2. Fit Both Super Learners

We will fit two separate `SuperSurv` models. The first will use the default Ensemble approach (`selection = "ensemble"`), and the second will use the winner-take-all approach (`selection = "best"`).

```{r fit-models, results='hide', message=FALSE, warning=FALSE}
# 1. The Ensemble Super Learner (Weighted Average)
fit_ensemble <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "ensemble",     # <-- Calculates fractional weights
  nFolds = 3
)

# 2. The 'Best' Super Learner (Winner-Take-All)
fit_best <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "best",         # <-- Selects the single best model
  nFolds = 3
)
```

## 3. Inspecting the Weights

The difference between the two methodologies is immediately obvious when we inspect the meta-learner coefficients with `event_weights()`. 

```{r inspect-weights}
cat("\n--- ENSEMBLE WEIGHTS (selection = 'ensemble') ---\n")
print(round(event_weights(fit_ensemble), 4))

cat("\n--- BEST MODEL WEIGHTS (selection = 'best') ---\n")
print(round(event_weights(fit_best), 4))
```

**Interpretation:** Notice that the Ensemble Super Learner distributes the weight across multiple models to minimize the overall loss function. The "Best" Super Learner simply looks at the cross-validated risk, identifies the champion, and gives it 100% of the weight. 

## 4. Evaluate and Compare Performance

A common question in clinical research is: *"Does the complexity of the Ensemble actually perform better than simply picking the best single model?"*

We can answer this instantly using `eval_summary()`. 

```{r evaluate-comparison}
# Evaluate the Ensemble
cat("Performance of the ENSEMBLE Super Learner:\n")
eval_summary(fit_ensemble, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)

cat("\nPerformance of the BEST MODEL Super Learner:\n")
eval_summary(fit_best, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)
```

By comparing the resulting Brier scores and C-indices, you can empirically justify whether the "soft selection" of the ensemble is mathematically necessary for your specific dataset, or if the "hard selection" of a single model is sufficient.

---
title: "8. Interpreting the Black Box with SHAP & survex"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{8. Interpreting the Black Box with SHAP & survex}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

Machine learning models are notoriously criticized as "black boxes." While they often achieve superior predictive performance, clinicians need to know *why* a model is making a specific prediction. 

`SuperSurv` solves this using **SHAP (SHapley Additive exPlanations)**. Because Shapley values possess the mathematical property of linearity, `SuperSurv` can calculate the SHAP values for every active base learner and seamlessly combine them using the meta-learner's weights. 

This tutorial covers Global feature importance, Local (patient-level) explanations, and Time-Dependent survival analysis using the `survex` package.

## 1. Setup and Model Fitting

Let's train a diverse Super Learner on our `metabric` dataset.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

my_library <- c("surv.coxph", "surv.weibull", "surv.rfsrc")

fit_sl <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = c("surv.coxph"),
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 3
)
```

## 2. Global Explanations (Kernel SHAP)

To calculate static risk SHAP values, we need an `X_explain` dataset (the patients we want to explain) and an `X_background` dataset (a reference population used to calculate the baseline average risk). 

```{r calc-shap, message=FALSE, warning=FALSE}
# Explain the first 50 test patients using 100 training patients as the background
X_explain_subset <- X_te[1:50, ]
X_background_subset <- X_tr[1:100, ]

# Calculate weighted Ensemble SHAP values using Kernel SHAP
shap_vals <- explain_kernel(
  model = fit_sl, 
  X_explain = X_explain_subset, 
  X_background = X_background_subset, 
  nsim = 20
)
```

### Global Importance Bar Plot
Which features drive the ensemble's mortality risk predictions across the entire cohort?

```{r plot-global, fig.width=6, fig.height=4}
plot_global_importance(shap_vals, top_n = 5)
```

### SHAP Beeswarm Summary Plot
The beeswarm plot is the gold standard for SHAP. It shows both the magnitude of a feature's impact and the direction of its effect.

```{r plot-beeswarm, fig.width=7, fig.height=5, warning=FALSE}
plot_beeswarm(shap_vals, data = X_explain_subset, top_n = 5)
```
*Interpretation: A red dot (high feature value) on the right side of the vertical zero-line indicates that higher values of this biomarker increase mortality risk.*

## 3. Local (Patient-Level) Explanations

In precision medicine, we often need to explain why a *specific* patient has a high risk score. The Waterfall plot breaks down the exact algorithmic logic for an individual.

```{r plot-waterfall, fig.width=6, fig.height=4}
# Explain Patient #1 from our test subset
plot_patient_waterfall(shap_vals, patient_index = 1, top_n = 5)
```

## 4. Feature Dependence Plots

Does a specific biomarker have a linear or non-linear relationship with mortality risk?

```{r plot-dependence, fig.width=6, fig.height=4, warning=FALSE}
plot_dependence(shap_vals, data = X_explain_subset, feature_name = "x0")
```

## 5. Patient Risk Trajectories (Survival Heatmap)

We can visualize patient trajectories over time using a survival heatmap generated directly from `SuperSurv` predictions.

```{r plot-heatmap, fig.width=7, fig.height=5}
# Plot the survival trajectories for the first 50 test patients
plot_survival_heatmap(fit_sl, newdata = X_explain_subset, times = new.times)
```
*Interpretation: Patients at the top experience rapid drops in survival probability (high risk), while patients at the bottom maintain high survival probabilities.*


## 6. Time-Dependent Explanations (`survex` Integration)

Traditional SHAP looks at an overall "risk score," but survival analysis is fundamentally about *time*. A feature might be highly predictive of early mortality (Time = 50) but irrelevant for long-term survival (Time = 200).

`SuperSurv` natively bridges to the `survex` package to evaluate dynamic feature importance across the entire survival curve $S(t)$.

```{r survex-bridge, message=FALSE, warning=FALSE}
library(survex)

# 1. Create the true survival object for the explanation subset
y_explain <- survival::Surv(test$duration[1:50], test$event[1:50])

# 2. Build the survex explainer using our custom function
surv_explainer <- explain_survex(
  model = fit_sl, 
  data = X_explain_subset, 
  y = y_explain, 
  times = new.times
)
```

Once the explainer is created, you have full access to the `survex` ecosystem. Let's look at three powerful time-dependent visualizations.

### A. Dynamic Feature Importance
Which features are driving the model's predictions at different clinical milestones?

```{r survex-importance, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
# Calculate time-dependent model parts (permutation feature importance)
time_importance <- model_parts(surv_explainer)

# Plot the dynamic importance over time
plot(time_importance)
```
*Interpretation: If a feature's curve rises over time, it means that biomarker becomes more critical for predicting long-term survival.*

### B. Time-Dependent Partial Dependence Profiles
How does the value of a specific continuous feature (e.g., `x0`) change the average survival probability over time? 

```{r survex-profile, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
# Calculate the partial dependence profile for feature 'x0'
pdp_time <- model_profile(surv_explainer, variables = "x0")

plot(pdp_time)
```
*Interpretation: This generates a 3D-like profile showing how different values of `x0` shift the entire survival curve.*

### C. SurvSHAP(t): Local Explanations Over Time
Just like the Waterfall plot explains a static risk score, SurvSHAP(t) explains exactly how a specific patient's features pushed their survival probability up or down *at every single time point*.

```{r survex-survshap, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
# Explain Patient #1 over time
patient_1_data <- X_explain_subset[1, , drop = FALSE]

# Calculate SurvSHAP(t)
survshap_t <- predict_parts(surv_explainer, new_observation = patient_1_data, type = "survshap")

plot(survshap_t)
```
*Interpretation: The solid black line is the model's average survival curve. The colored areas show how Patient 1's specific covariates (like their specific age or tumor grade) dragged their personal survival curve above or below the average over time.*

### D. Ecosystem Compatibility: Global Model Performance
Because `explain_survex()` creates a standard explainer object, you aren't just limited to SHAP values. You can utilize the entire `survex` ecosystem, including their built-in performance metrics. 

While `SuperSurv` provides its own comprehensive benchmarking suite (`plot_benchmark`), you can easily cross-validate your ensemble's Time-Dependent Brier Score and AUC using `survex`'s native functions:



```{r survex-performance, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
# Calculate time-dependent performance metrics via survex
survex_perf <- model_performance(surv_explainer)

# Plot the Brier score and AUC curves
plot(survex_perf)
```
*Note: The Brier score should ideally stay as low as possible over time, while the AUC should remain high. This serves as an excellent independent validation of the results generated by `SuperSurv`'s native `eval_summary()`!*

By utilizing these tools, `SuperSurv` ensures that your advanced machine learning ensembles remain completely transparent, dynamically interpretable, and ready for clinical deployment.

---
title: "10. Extending SuperSurv"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{10. Extending SuperSurv}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This vignette describes the practical contract used by `SuperSurv()` to call
survival learners and screening methods. The goal is to make small extensions
possible without changing the ensemble algorithm.

## How Learners Are Found

Learners are supplied to `SuperSurv()` by name:

```r
event.library <- c("surv.coxph", "my.surv.km")
```

During cross-validation and final fitting, `SuperSurv()` looks up each name as
an R function and calls it directly. A custom learner can therefore live in a
user script, package, or analysis file, as long as it is available in the R
session before calling `SuperSurv()`.

## Learner Wrapper Contract

A survival learner wrapper should have this shape:

```r
my.surv.learner <- function(time, event, X, newdata, new.times,
                            obsWeights, id, ...) {
  # fit model on X, time, event
  # predict survival probabilities for newdata at new.times
  list(pred = pred_matrix, fit = fitted_object)
}
```

The required arguments are:

| Argument | Meaning |
| --- | --- |
| `time` | numeric observed follow-up time |
| `event` | event indicator, with `1` for the event and `0` for censoring |
| `X` | training covariates as a data frame |
| `newdata` | covariates for prediction as a data frame |
| `new.times` | time grid where survival probabilities are needed |
| `obsWeights` | observation weights, possibly `NULL` |
| `id` | optional cluster or subject identifier, possibly `NULL` |
| `...` | learner-specific tuning parameters |

The returned `pred` must be a numeric matrix with:

- one row per row of `newdata`,
- one column per value of `new.times`,
- entries on the survival-probability scale, usually between `0` and `1`.

If possible, each row should be non-increasing across time. Wrappers commonly
clamp small numerical drift back into `[0, 1]`.

The returned `fit` can be any object needed for future prediction. When
`control = list(saveFitLibrary = TRUE)`, `SuperSurv()` stores these fitted
objects and `predict.SuperSurv()` later calls `predict()` on them.

## Prediction Method Contract

If a custom learner should support prediction on new data after fitting, store
a classed object in `fit` and define a matching S3 prediction method:

```r
predict.my.surv.learner <- function(object, newdata, new.times, ...) {
  pred_matrix
}
```

This method should return the same type of matrix as the wrapper's `pred`
component: `nrow(newdata)` rows by `length(new.times)` columns.

## Direct Survival Curves vs Risk Scores

Some survival learners directly return survival curves. In that case, the
wrapper usually interpolates the native prediction grid to `new.times`.

Other learners return a risk score or linear predictor. Those wrappers need one
extra calibration step:

1. fit the risk-score model,
2. estimate a baseline cumulative hazard on the training data,
3. predict risk scores for `newdata`,
4. convert scores to survival curves with `S(t | x) = exp(-exp(eta(x)) H0(t))`.

Several built-in Cox-style wrappers follow this pattern. The internal helper
`safe_breslow_step()` is used by some package wrappers, but it is not exported;
custom wrappers should either use a standard package prediction method that
already returns survival curves or implement their own baseline-hazard
calibration carefully.

## Minimal Custom Learner

The following learner ignores covariates and fits a weighted Kaplan-Meier curve.
It is intentionally simple, but it satisfies the full `SuperSurv()` learner
contract.

```{r custom-learner}
my.surv.km <- function(time, event, X, newdata, new.times,
                       obsWeights = NULL, id = NULL, ...) {
  if (is.null(obsWeights)) {
    obsWeights <- rep(1, length(time))
  }

  fit_km <- survival::survfit(
    survival::Surv(time, event) ~ 1,
    weights = obsWeights
  )

  step_surv <- stats::stepfun(fit_km$time, c(1, fit_km$surv), right = FALSE)
  surv_probs <- step_surv(new.times)

  pred <- matrix(
    surv_probs,
    nrow = nrow(newdata),
    ncol = length(new.times),
    byrow = TRUE
  )

  fit <- list(object = fit_km)
  class(fit) <- "my.surv.km"

  list(pred = pred, fit = fit)
}

predict.my.surv.km <- function(object, newdata, new.times, ...) {
  fit_km <- object$object
  step_surv <- stats::stepfun(fit_km$time, c(1, fit_km$surv), right = FALSE)
  surv_probs <- step_surv(new.times)

  matrix(
    surv_probs,
    nrow = nrow(newdata),
    ncol = length(new.times),
    byrow = TRUE
  )
}
```

You can test the wrapper outside the ensemble before adding it to a library:

```{r custom-learner-check}
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
times <- seq(50, 150, by = 50)

wrapper_out <- my.surv.km(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X[1:5, , drop = FALSE],
  new.times = times
)

dim(wrapper_out[["pred"]])
dim(predict(wrapper_out[["fit"]], newdata = X[1:5, , drop = FALSE], new.times = times))
```

The custom learner can be mixed with built-in learners:

```{r custom-fit, eval=FALSE}
fit <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_train,
  newdata = X_test,
  new.times = seq(50, 300, by = 50),
  event.library = c("surv.coxph", "my.surv.km"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE)
)

event_weights(fit)
```

## Screening Method Contract

Screening methods are also supplied by name. A screener should accept the same
basic training inputs and return a logical vector with one value per column of
`X`:

```r
my.screen.first3 <- function(time, event, X, obsWeights = NULL, id = NULL, ...) {
  keep <- rep(FALSE, ncol(X))
  keep[seq_len(min(3, ncol(X)))] <- TRUE
  names(keep) <- names(X)
  keep
}
```

Use a list entry to pair a learner with one or more screeners:

```{r custom-screener, eval=FALSE}
fit <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_train,
  newdata = X_test,
  new.times = seq(50, 300, by = 50),
  event.library = list(c("surv.coxph", "screen.all", "my.screen.first3")),
  cens.library = c("surv.coxph")
)

selected_variables(fit, learner = 2)
```

## Development Checklist

Before using a custom wrapper in a large ensemble, check:

- `pred` is a numeric matrix with dimensions `nrow(newdata)` by
  `length(new.times)`,
- predictions are on the survival-probability scale,
- `predict(wrapper_out[["fit"]], newdata, new.times)` returns the same shape when saved
  fits are needed,
- the wrapper tolerates `obsWeights = NULL` and `id = NULL`,
- all required packages are checked with `requireNamespace()`,
- any screener returns a logical vector aligned with the columns of `X`.

These rules are deliberately small. They allow new learners to be added without
changing the SuperSurv ensemble machinery.

---
title: "07. Parametric Survival Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{07. Parametric Survival Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

While the Cox model is semi-parametric (it leaves the baseline hazard unspecified), fully parametric models assume that survival times follow a specific mathematical distribution, such as the Weibull, Exponential, or Log-Normal distribution.



Parametric models are statistically powerful because they produce perfectly smooth survival curves. However, they are highly brittle: if you assume the data follows a Weibull distribution, but the true biological hazard has a completely different shape, the model will be heavily biased. 

`SuperSurv` acts as a **safety net**. You can include multiple parametric assumptions in your library. If a parametric assumption perfectly matches your data, `SuperSurv` will give it a high weight. If the assumption is wrong, the cross-validation risk will spike, and `SuperSurv` will safely assign it a weight of zero.

## 1. Setup and Library Definition

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# Define a library covering different parametric assumptions
parametric_library <- c("surv.coxph",       # Semi-parametric baseline
                        "surv.weibull",     # Assumes hazard increases/decreases monotonically
                        "surv.exponential", # Assumes constant hazard over time
                        "surv.lognormal")   # Assumes hazard rises then falls
```

## 2. Fitting the Parametric Ensemble

We run the ensemble exactly as before. Internally, `SuperSurv` will fit these Accelerated Failure Time (AFT) models and map their continuous survival predictions onto our discrete `new.times` evaluation grid.

```{r fit-parametric, results='hide', message=FALSE, warning=FALSE}
fit_parametric <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = parametric_library,
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 3
)
```

## 3. Evaluating the "Safety Net"

Let's look at the cross-validated risks and the final meta-learner weights.

```{r evaluate-parametric}
summary(fit_parametric)
```

### Interpretation
Look closely at the weights assigned to `surv.exponential`. The Exponential distribution assumes that the risk of the event (hazard) is completely constant over time. In real-world cancer datasets like `metabric`, this assumption is almost always false (risk usually increases with time or peaks shortly after surgery). 

Because the Exponential assumption fits the data poorly, its cross-validated risk will be high, and `SuperSurv` will smartly assign it a weight of $0.00$. 

By including parametric models in your `SuperSurv` library, you allow the data—not the researcher—to dictate which mathematical distributions are actually appropriate for your patient cohort!

---
title: "4. High-Dimensional Data & Variable Screening"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{4. High-Dimensional Data & Variable Screening}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

In modern clinical research and bioinformatics, it is common to encounter high-dimensional datasets where the number of predictors (genes, biomarkers, clinical features) far exceeds the number of observed events ($p \gg n$). 

Feeding thousands of raw variables directly into complex machine learning algorithms introduces two massive problems:
1. **Computational Bottlenecks:** Algorithms like Random Survival Forests become prohibitively slow.
2. **Overfitting:** Models will mathematically memorize background noise instead of true biological signals, destroying out-of-sample prediction accuracy.



`SuperSurv` elegantly handles this through **Variable Screening**. Screening acts as a computational filter *before* the models are fitted. It evaluates all predictors, drops the statistical noise, and only passes the most highly associated features into your base learners.

## 1. Simulating High-Dimensional Data

To demonstrate how screening works, we will use our standard `metabric` dataset, but we will artificially inject 100 columns of pure random noise. This simulates a high-dimensional genomic dataset where most measured features have absolutely no relationship to patient survival.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Create 100 columns of random noise (simulating irrelevant genes/biomarkers)
n_patients <- nrow(metabric)
noise_data <- matrix(rnorm(n_patients * 20), nrow = n_patients)
colnames(noise_data) <- paste0("noise_", 1:20)
# Combine the real data with the noise
high_dim_data <- cbind(metabric, noise_data)

# Standard Train/Test Split
train_idx <- sample(1:nrow(high_dim_data), 0.7 * nrow(high_dim_data))
train <- high_dim_data[train_idx, ]
test  <- high_dim_data[-train_idx, ]

X_tr <- train[, grep("^x|^noise", names(high_dim_data))]
X_te <- test[, grep("^x|^noise", names(high_dim_data))]
new.times <- seq(50, 200, by = 25)
```

## 2. Defining the Screening Algorithms

Just as we have a library for prediction algorithms, `SuperSurv` utilizes a library for screening algorithms. You can pair any prediction model with any screening function to create a streamlined pipeline. 

`SuperSurv` includes a comprehensive suite of built-in screeners:
* **`screen.marg`**: Marginal screening (keeps variables with strong univariate associations).
* **`screen.glmnet`**: Penalized Lasso screening (shrinks irrelevant feature coefficients to exact zero).
* **`screen.elasticnet`**: Elastic Net screening (combines Lasso and Ridge penalties; highly recommended for highly correlated biological features like gene expression data).
* **`screen.rfsrc`**: Tree-based screening (keeps features with high Random Forest variable importance).
* **`screen.var`**: Unsupervised screening (drops features with near-zero variance).
* **`screen.all`**: A baseline passthrough (keeps all variables without screening).



Let's build a library that tests different combinations of prediction and screening:

```{r define-library}
# We pass a list of vectors. The first element is the prediction model, 
# and the second element is the screening algorithm.

my_screen_library <- list(
  c("surv.coxph", "screen.all"),         # Baseline: Cox model with ALL variables
  c("surv.coxph", "screen.marg"),        # Screen by marginal association, then fit Cox
  c("surv.weibull", "screen.elasticnet"),# Screen via Elastic Net, then fit Weibull
  c("surv.rpart", "screen.var")          # Drop zero-variance noise, then fit a Tree
)

# For the censoring library, we use an unscreened approach 
cens_library <- c("surv.coxph")
```
## 3. Training with Embedded Screening

When we pass this paired library into `SuperSurv`, the meta-learner handles the cross-validation with absolute statistical rigor. 

**Crucial Methodological Detail:** The screening step is performed *inside* the cross-validation folds. If you screen the entire dataset before splitting it into folds, you cause "data leakage," leading to overly optimistic benchmark metrics. `SuperSurv` protects you from this automatically.

```{r fit-screen, results='hide', message=FALSE, warning=FALSE}
fit_highdim <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_screen_library, # Pass our screened library
  cens.library = cens_library,
  control = list(saveFitLibrary = TRUE),
  verbose = T,
  nFolds = 3
)
```

## 4. Evaluating the Screened Ensemble

Despite injecting 100 columns of pure noise, our ensemble still performs exceptionally well because the screening algorithms successfully filtered out the irrelevant variables before the base learners could overfit to them.

We can evaluate this exactly as we did in previous tutorials using `eval_summary()` and `plot_benchmark()`.

```{r evaluate-screen}
# Check the integrated performance
screened_performance <- eval_summary(
  object = fit_highdim,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

# Plot the benchmark
# plot_benchmark(fit_highdim, newdata = X_te, time = test$duration, event = test$event, eval_times = new.times)
```

By leveraging `SuperSurv`'s screening capabilities, you can scale your survival analysis to massive genomic and clinical datasets without sacrificing computational speed or predictive accuracy.


## 5. Identifying the Selected Features

After the Super Learner finishes its cross-validation and fitting, you likely want to know *which* variables actually survived the screening process. In clinical and biological research, these selected features often represent the most important biomarkers or predictive genes.

`SuperSurv` stores this screening information directly inside the fitted model object, specifically separated into `event.whichScreen` and `cens.whichScreen`. We can extract the logical matrix that indicates which columns were passed to each specific base learner.

```{r extract-features}
# The event.whichScreen object is a logical matrix.
# Rows correspond to the predictors, and columns correspond to the algorithms in our event library.

# 1. Get the logical vector for the 2nd model (Row 2)
is_selected <- fit_highdim$event.whichScreen[2, ] 

# 2. Map it to our original column names
selected_features <- colnames(X_tr)[is_selected]

cat("Total features evaluated:", ncol(X_tr), "\n")
cat("Features retained by Elastic Net:", length(selected_features), "\n\n")

cat("Selected Features:\n")
print(selected_features)
```

By extracting these features, you can seamlessly transition from pure predictive modeling into downstream biological inference or pathway analysis.

---
title: "1. SuperSurv with Ensemble"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{1. SuperSurv with Ensemble}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The core feature of the `SuperSurv` package is its ability to combine multiple base survival learners into a highly predictive meta-ensemble. This tutorial walks through preparing data, defining a library of models, fitting the Super Learner, and generating predictions for new patients.

## 1. Load Data & Prepare Matrices

We will use the built-in `metabric` dataset, extracting the covariates and performing a standard 80/20 train-test split.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

# Load built-in METABRIC data
data("metabric", package = "SuperSurv")

# Quick 80/20 Train-Test split
set.seed(42)
n_total <- nrow(metabric)
train_idx <- sample(1:n_total, 0.8 * n_total)

train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

# Extract just the X covariates (assuming they are named x0, x1, etc.)
x_cols <- grep("^x", names(metabric), value = TRUE)
X_tr <- train[, x_cols]
X_te <- test[, x_cols]

# Define the prediction time grid (e.g., survival at 50, 100, 150, 200 months)
new.times <- c(50, 100, 150, 200)
```

## 2. Define the Ensemble Library

We define a library of base survival models. For this quick demonstration, we use lightning-fast parametric and tree-based models.

```{r define-library}
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")
```


## 3. Train the SuperSurv Metalearner

Before we run the main `SuperSurv` engine, it is important to understand its key parameters. The Super Learner algorithm relies on cross-validation to assign weights to the base models, and it must model both the event and the censoring mechanism to avoid biased evaluations.

Here is the complete guide to the arguments you need to pass:

### The Data Inputs
* **`time`**: A numeric vector of the observed follow-up times for your training cohort.
* **`event`**: A numeric vector indicating the status at the observed time (typically `1` = event occurred, `0` = right-censored).
* **`X`**: A `data.frame` or matrix containing **only** the predictor variables (covariates) for the training set. Do not include the time or event columns here!
* **`newdata`**: (Optional) A `data.frame` of covariates for a validation or test set. If provided, `SuperSurv` will immediately generate predictions for these patients during the training phase, saving you a step.
* **`new.times`**: A numeric vector defining the exact time points where you want survival probabilities predicted (e.g., `seq(50, 200, by = 25)`).

### The Model Libraries
* **`event.SL.library`**: A character vector of the base algorithms used to predict the actual survival outcome (e.g., `"surv.coxph"`, `"surv.rpart"`).
* **`cens.SL.library`**: The library used to estimate the *censoring* mechanism over time. `SuperSurv` uses these predictions to calculate Inverse Probability of Censoring Weights (IPCW). You can use the exact same library as your event models, or a simpler one.

### The Meta-Learner & Tuning
* **`metalearner`**: The optimization algorithm used to calculate the final ensemble weights. `SuperSurv` offers two distinct approaches:
    * `"least_squares"` (Default): Optimizes weights by minimizing the IPCW Brier score. Excellent for overall prediction accuracy.
    * `"nloglik"`: Optimizes weights by minimizing the negative log-likelihood. Excellent for improving hazard discrimination.
* **`nFolds`**: The number of cross-validation folds used to train the meta-learner. `V = 5` or `V = 10` is standard. This cross-validation is what prevents the ensemble from overfitting to the base learners.
* **`control = list(saveFitLibrary = TRUE)`**: This tells the engine to save the fitted base learners into the final object. This is **required** if you want to use the `predict()` method on new patients later.
* **`verbose`**: Set to `TRUE` to print progress messages to the console. Highly recommended for large datasets or complex libraries so you can track the cross-validation progress.

Let's fit two models with `verbose = FALSE` to see how the meta-learner choice affects the final ensemble weights.

```{r fit-models, results='hide', message=FALSE, warning=FALSE}
# Fit 1: Least Squares Meta-learner
fit_ls <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,                 # Predict on the test set immediately
  new.times = new.times,       # Our evaluation time grid
  event.library = my_library,
  cens.library = my_library,
  metalearner = "brier", 
  control = list(saveFitLibrary = TRUE), 
  verbose = T,             # Turn to TRUE in practice to see progress!
  selection = "ensemble",
  nFolds = 5                   # 5-fold CV for the meta-learner
)

# Fit 2: Negative Log-Likelihood Meta-learner
fit_nll <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  metalearner = "logloss",       # Swap to nloglik
  control = list(saveFitLibrary = TRUE), 
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 5
)
```


## 4. Inspect the Ensemble Weights and Risks

The defining feature of the Super Learner is that it does not just pick the single "best" model; it finds the optimal weighted combination of all models based on their cross-validated performance.

Let's inspect the weights (`event.coef`) and the cross-validated risks (`event.cvRisks`) for both of our meta-learners.

```{r inspect-weights}
cat("\n--- LEAST SQUARES METALEARNER ---\n")
print(round(fit_ls$event.coef, 4))
cat("CV Risks (Lower is Better):\n")
print(round(fit_ls$event.cvRisks, 4))

cat("\n--- NLOGLIK METALEARNER ---\n")
print(round(fit_nll$event.coef, 4))
cat("CV Risks (Lower is Better):\n")
print(round(fit_nll$event.cvRisks, 4))
```

### How to Interpret This:
* **`event.coef`**: These are the final weights assigned to each base learner. A weight of `0` means the meta-learner completely dropped that model because it did not contribute to overall predictive accuracy. A high weight means that model heavily influences the final ensemble prediction. Notice how the `least_squares` and `nloglik` algorithms might distribute these weights differently based on their optimization goals!
* **`event.cvRisks`**: This is the cross-validated error for each individual base learner. The meta-learner uses these errors to calculate the optimal weights. Lower risk always indicates better performance.

## 5. Generating Predictions on New Data

If you passed `newdata` during the training phase, `SuperSurv` already calculated the predictions for your test set. However, in a real-world clinical setting, you will often train the model once and then predict on brand new patients months later. 

Because we set `control = list(saveFitLibrary = TRUE)` during training, we can use the standard R `predict()` method.

```{r predict-new}
# Select 3 brand new patients from our test set
new_patients <- X_te[1:6, ]

# Generate predictions using the Least Squares ensemble
ensemble_preds <- predict(
  object = fit_ls, 
  newdata = new_patients, 
  new.times = new.times
)

cat("\n--- PREDICTED SURVIVAL PROBABILITIES ---\n")
final_matrix <- ensemble_preds$event.predict
colnames(final_matrix) <- paste0("Time_", new.times)
rownames(final_matrix) <- paste0("Patient_", 1:6)

print(round(final_matrix, 4))
```

### Understanding the Output Matrix:
The `predict()` function returns a list, but the most important element is `event.SL.predict`. 
* **Rows ($N$)**: Represent individual patients.
* **Columns ($T$)**: Represent the specific time points we defined in `new.times`.
* **Values**: The estimated probability that the patient will *survive* past that specific time point. As time increases (moving left to right across a row), the survival probability naturally decreases.

## 6. Visualizing Patient-Specific Predictions

While raw probability matrices ($N \times T$) are perfect for downstream coding and performance benchmarking, they are difficult to interpret clinically. Doctors and researchers need to see the actual survival trajectories.

`SuperSurv` includes a built-in `plot_predict()` function to effortlessly translate this matrix into publication-ready survival curves for individual patients.


```{r plot-predict, fig.align='center', fig.width=6, fig.height=5}
# Plot the predicted survival curves for our 3 new patients (Rows 1, 2, and 3)
plot_predict(
  preds = ensemble_preds,
  eval_times = new.times,
  patient_idx = 1:6
)
```

## 7. Next Steps

You now know how to prepare data, define a model library, choose a meta-learner, and generate patient-specific survival curves. 

However, before deploying a model in clinical practice, you must rigorously prove that the ensemble actually outperforms the individual base learners. Head over to **Tutorial 2: Model Performance & Benchmarking** to learn how to generate time-dependent Brier Score, AUC, and Uno's C-index plots!

---
title: "2. Model Performance & Benchmarking"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{2. Model Performance & Benchmarking}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

Once a `SuperSurv` ensemble is trained, we must rigorously prove that it outperforms the individual base learners. Because survival data involves right-censoring, we cannot use standard classification metrics like simple accuracy.

Instead, we evaluate the model across three critical dimensions:
1. **Calibration:** Does the predicted survival probability match the actual observed survival rate?
2. **Discrimination (AUC & C-index):** Can the model correctly rank which patient will survive longer?
3. **Overall Accuracy (Brier Score):** A combined measure of both calibration and discrimination.

This tutorial demonstrates how to extract these metrics and visualize the benchmark comparisons using `SuperSurv`'s built-in evaluation suite.

## 1. Data Preparation & The Extrapolation Rule

We begin by loading the `metabric` dataset and defining our evaluation time grid (`new.times`). 

**Crucial Methodological Note:** Your `new.times` grid should *never* exceed the maximum observed follow-up time in your training cohort. For example, if your training data only spans 1 to 100 days, predicting survival at day 150 is extrapolation. Non-parametric models (like Survival Trees) mathematically cannot extrapolate, and parametric models (like Weibulls) will generate highly unreliable tail estimates. Always bind your evaluation grid within the limits of your observed data!



```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

# Our max follow-up is well beyond 200, so this grid is safe.
new.times <- seq(50, 200, by = 25) 
```

## 2. Train the Benchmark Ensemble

We will fit an ensemble consisting of a Cox model, a Weibull model, and a Survival Tree using the default Least Squares meta-learner.

```{r fit-models, results='hide', message=FALSE, warning=FALSE}
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)
```

## 3. Extracting Integrated Metrics

The `eval_summary()` function automatically generates predictions on your test set and returns a clean, comparative table of the *integrated* metrics across your entire time grid.

```{r evaluate-performance}
# Evaluate performance directly using the fitted model and test data
performance_results <- eval_summary(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)
```

*Note: Look for the model with the lowest IBS (Integrated Brier Score) and the highest iAUC/Uno's C-index.*

## 4. Visualizing Longitudinal Benchmarks

A single integrated number rarely tells the whole story. Different models excel at different follow-up periods (e.g., a Cox model might dominate short-term survival, while a Random Forest dominates long-term). 

The `plot_benchmark()` function generates a stacked dashboard to visualize this dynamic performance over time.

```{r plot-benchmark, fig.align='center', fig.width=3, fig.height=9}
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)
```

### Interpreting the Curves:
- **IPCW Brier Score Plot:** Look for the curve that stays the *lowest*. The `SuperSurv` ensemble should ideally hug the bottom edge.
- **Time-Dependent AUC Plot:** Look for the curve that stays the *highest* (closest to 1.0).
- **Uno's C-index Plot:** Evaluates the global C-index using the specific predictions generated at each time point. Look for the curve that stays the *highest*.

## 5. Assessing Clinical Calibration

Before deploying a model to the clinic, doctors need to know if the probabilities are reliable. If the model predicts a patient has a 40% chance of surviving past Time = 100, do exactly 40% of similar patients actually survive?

We use `plot_calibration()` to evaluate this at a specific clinical milestone. The function groups patients into risk quantiles and plots their predicted probability against the actual observed Kaplan-Meier survival rate.

```{r plot-calibration, fig.align='center'}
# Assess calibration specifically at Time = 150
plot_calibration(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_time = 150,
  bins = 5 # Group patients into 5 risk quintiles
)
```

**Interpretation:** A perfectly calibrated model will follow the 45-degree dashed black line. Points above the line indicate the model is *under-predicting* survival (being too pessimistic), while points below the line indicate it is *over-predicting* survival (being too optimistic).

---
title: "06. Machine Learning with Random Survival Forests"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{06. Machine Learning with Random Survival Forests}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
has_rfsrc <- requireNamespace("randomForestSRC", quietly = TRUE)
```

## Introduction

While classical models like the Cox Proportional Hazards model are highly interpretable, they rely on strict assumptions of linear, additive effects. Random Survival Forests (RSF) overcome this by building a non-linear ensemble of decision trees. 

In this tutorial, we will demonstrate how to run a standalone RSF using `SuperSurv`'s unified wrappers, evaluate its individual performance, and finally include it in a Super Learner ensemble.

## 1. Prepare the Data

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

new.times <- seq(50, 200, by = 25) 
```

## 2. Standalone Machine Learning

`SuperSurv` provides unified wrappers that automatically handle the training and standardization of survival probabilities. You can use these completely independent of the Super Learner ensemble.

```{r standalone-rf, eval=requireNamespace("randomForestSRC", quietly = TRUE)}
# 1. Fit the standalone wrapper
rf_standalone <- surv.rfsrc(
  time = train$duration,
  event = train$event,
  X = X_tr,
  new.times = new.times 
)

# 2. Extract the fitted model object and prediction matrix
rf_fit <- rf_standalone[["fit"]]
rf_pred_matrix <- rf_standalone[["pred"]]
```

Because our plotting functions are universally compatible, we can plot individual patient curves directly from this standalone matrix:



```{r plot-standalone, fig.align='center', eval=requireNamespace("randomForestSRC", quietly = TRUE)}
# Plot the first 3 patients in our training set
plot_predict(preds = rf_pred_matrix, eval_times = new.times, patient_idx = 1:3)
```

## 3. Evaluating the Standalone Model

We can also pass this standalone model directly into our evaluation suite to test its performance on new data. 

```{r eval-standalone, fig.align='center', fig.height=4, fig.width= 9, eval=requireNamespace("randomForestSRC", quietly = TRUE)}
# The function automatically detects this is a single model and plots it!
plot_benchmark(
  object = rf_fit,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)
```



```{r}

# plot_calibration(
#   object   = rf_fit,
#   newdata  = X_te,
#   time     = test$duration,
#   event    = test$event,
#   eval_time = 150,
#   bins     = 2
# )

```




## 4. Train the Benchmark Ensemble

While the standalone RSF is powerful, we can objectively evaluate if it outperforms classical models by putting them together in a `SuperSurv` ensemble. 

```{r fit-models, results='hide', message=FALSE, warning=FALSE, eval=requireNamespace("randomForestSRC", quietly = TRUE)}
my_library <- c("surv.coxph", "surv.weibull", "surv.rfsrc")

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = c("surv.coxph"), 
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)
```

## 5. Visualizing Ensemble vs. Base Learners

When we pass the `SuperSurv` ensemble into the exact same benchmark function, it automatically unpacks the library and plots the ensemble against all its constituent models.



```{r plot-ensemble-benchmark, fig.align='center', fig.height=9, eval=requireNamespace("randomForestSRC", quietly = TRUE)}
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)
```

By incorporating advanced machine learning algorithms into your library, `SuperSurv` mathematically guarantees that your final predictions adapt to the complexity of your data, achieving the lowest possible Brier Score.

---
title: "9. Causal Effects and Adjusted Marginal Contrasts (RMST)"
description: "Estimating absolute treatment effects and covariate-adjusted group differences using G-computation."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{9. Causal Effects and Adjusted Marginal Contrasts (RMST)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Moving Beyond the Hazard Ratio

In clinical trials and observational studies, researchers often wish to compare survival outcomes between two groups. Historically, this is answered using the Hazard Ratio (HR) from a Cox Proportional Hazards model. However, the HR is non-collapsible—meaning the omission of unmeasured covariates will mathematically bias the effect toward the null—and strictly relies on the proportional hazards assumption. If survival curves cross, the HR becomes mathematically invalid.

`SuperSurv` solves this by evaluating group differences on the absolute time scale using the **Restricted Mean Survival Time (RMST)** via G-computation (Standardization) on top of our Ensemble Super Learner. 



RMST calculates the area under the survival curve up to a specific time horizon, $\tau$. By comparing the expected RMST if *everyone* in the dataset belonged to Group 1 versus if *everyone* belonged to Group 0, we obtain a robust, absolute measure of the difference:

$$ \Delta \text{RMST} = E[Y(1)] - E[Y(0)] = \text{RMST}_{\text{Group 1}}(\tau) - \text{RMST}_{\text{Group 0}}(\tau) $$

## Philosophy: "Causal Effect" vs. "Marginal Contrast"

How you interpret this $\Delta \text{RMST}$ depends entirely on the nature of your exposure variable. The math of G-computation is identical for both, but the statistical terminology must be used responsibly.



1. **Causal Average Treatment Effect (ATE):** You can claim a *Causal Effect* if your variable is a **manipulable intervention**. Examples include administering a drug, performing a surgery, or applying a policy. 
   * *Interpretation:* "Administering this drug causally adds an average of 4.2 months of life over a 5-year period compared to the placebo."
2. **Adjusted Marginal Contrast:** You must claim an *Adjusted Marginal Contrast* if your variable is an **immutable trait or biological group**. Examples include biological sex, race, or a genetic biomarker. Because you cannot "causally" intervene to change someone's genetics, we are simply comparing two groups while rigorously adjusting for all other confounding variables.
   * *Interpretation:* "After adjusting for all baseline clinical covariates, the presence of this biomarker is marginally associated with 4.2 additional months of survival over a 5-year period."

## Estimating the Effect with `SuperSurv`

Let's demonstrate this using the built-in `metabric` dataset. We will evaluate the effect of the binary biomarker **`x4`** (1 = present, 0 = absent). Because `x4` is a biomarker, we will interpret the result as an **Adjusted Marginal Contrast**.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
set.seed(123)

# Load built-in data
data("metabric", package = "SuperSurv")

# Define predictors and time grid
X <- metabric[, grep("^x", names(metabric))]
new.times <- seq(10, 150, by = 10)
```

### 1. Train the Super Learner
First, train the ensemble. We must set `control = list(saveFitLibrary = TRUE)` so the models are saved for the G-computation prediction phase. 

```{r train-model, eval=FALSE}
fit <- SuperSurv(
  time = metabric$duration,
  event = metabric$event,
  X = X,
  newdata = X,
  new.times = new.times,
  event.library = c("surv.coxph", "surv.rfsrc"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE) 
)
```

### 2. Calculate the Counterfactual RMST
We use the `estimate_causal_rmst()` function. This forces the biomarker `x4` to 1 for all patients, predicts their survival curves, and calculates the RMST, then repeats with `x4` forced to 0.

```{r causal-effect, eval=FALSE}
# Estimate the adjusted difference up to tau = 100 months
results <- estimate_marginal_rmst(
  fit = fit, 
  data = metabric, 
  trt_col = "x4", 
  times = new.times, 
  tau = 100
)

print(results$ATE_RMST)
```

**Interpretation:** If the resulting $\Delta$RMST value is `4.2`, we interpret this marginal contrast as: *"After adjusting for complex baseline covariates via the Super Learner ensemble, patients with biomarker x4 live an average of 4.2 months longer over a 100-month horizon compared to those without the biomarker."*

### 3. Visualizing the Effect Over Time
The difference between groups might be near zero early on but substantial later. We can visualize how the $\Delta$RMST evolves across different time horizons using `plot_causal_rmst_curve()`.



```{r plot-curve, eval=FALSE}
# Plot the Delta RMST across a sequence of tau values
tau_grid <- seq(20, 140, by = 20)
plot_marginal_rmst_curve(
  fit = fit, 
  data = metabric, 
  trt_col = "x4", 
  times = new.times, 
  tau_seq = tau_grid
)
```

### 4. Diagnostic: Predicted RMST vs. Observed Time
To evaluate how well our model's restricted expectations align with reality, we can plot the predicted RMST for the observed data against their true survival times. Patients who experienced the event should lie close to the diagonal line up to $\tau$.



```{r plot-obs, eval=FALSE}
plot_rmst_vs_obs(
  fit = fit, 
  data = metabric, 
  time_col = "duration", 
  event_col = "event", 
  times = new.times, 
  tau = 350
)
```

---
title: "10. Scaling Up with Parallel Processing"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{10. Scaling Up with Parallel Processing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

The Double Super Learner is an incredibly powerful statistical framework, but it is computationally demanding. If you define 5 base algorithms and use 10-fold cross-validation, `SuperSurv` has to fit a minimum of 50 separate machine learning models just for the Event ensemble, plus another set for the Censoring ensemble!



By default, R executes these models sequentially (one after the other). However, `SuperSurv` natively supports parallel processing using the modern `future` and `future.apply` ecosystem. This allows you to distribute the cross-validation folds across multiple CPU cores, dramatically reducing computation time.

## 1. Prerequisites

To use parallel processing, you need to have the `future` and `future.apply` packages installed. 

```{r eval=FALSE}
install.packages(c("future", "future.apply"))
```

## 2. Setting Up the Parallel Environment

`SuperSurv` relies on you to define your parallel "plan" before running the function. This gives you complete control over how many resources the package is allowed to consume.

```{r parallel-setup, message=FALSE, warning=FALSE, eval=FALSE}
library(SuperSurv)
library(future)
library(survival)

data("metabric", package = "SuperSurv")

# 1. Define the parallel plan
# 'multisession' opens background R sessions. 
# We tell it to use 4 CPU cores (workers).
plan(multisession, workers = 4)
```

## 3. Running SuperSurv in Parallel

Once the `plan` is set, simply add `parallel = TRUE` to your `SuperSurv` call. The internal cross-validation loop will automatically detect your workers and distribute the folds simultaneously.

```{r run-parallel, eval=FALSE}
X <- metabric[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# 2. Run the model with parallel = TRUE
fit_parallel <- SuperSurv(
  time = metabric$duration,
  event = metabric$event,
  X = X,
  newX = X,
  new.times = new.times,
  event.library = c("surv.coxph", "surv.weibull", "surv.rfsrc"),
  cens.library = c("surv.coxph"),
  parallel = TRUE,     # <--- The magic argument
  nFolds = 5
)
```

## 4. Closing the Environment

It is a best practice to close the background workers and return to standard, sequential processing once your intensive models are finished fitting. This frees up memory on your machine.

```{r close-parallel, eval=FALSE}
# 3. Return to sequential processing
plan(sequential)
```

## A Note on Mathematical Reproducibility

In standard parallel processing, random number generation (used heavily in cross-validation splits and Random Forests) can become disorganized, leading to results that change slightly every time you run the code. 

`SuperSurv` handles this safely under the hood. When `parallel = TRUE`, the package automatically invokes `future.seed = TRUE`, ensuring that your parallelized ensemble yields the exact same mathematically reproducible results as your sequential ensemble, just much faster!

---
title: "11. Extending SuperSurv"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{11. Extending SuperSurv}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This vignette describes the practical contract used by `SuperSurv()` to call
survival learners and screening methods. The goal is to make small extensions
possible without changing the ensemble algorithm.

## How Learners Are Found

Learners are supplied to `SuperSurv()` by name:

```r
event.library <- c("surv.coxph", "my.surv.km")
```

During cross-validation and final fitting, `SuperSurv()` looks up each name as
an R function and calls it directly. A custom learner can therefore live in a
user script, package, or analysis file, as long as it is available in the R
session before calling `SuperSurv()`.

## Learner Wrapper Contract

A survival learner wrapper should have this shape:

```r
my.surv.learner <- function(time, event, X, newdata, new.times,
                            obsWeights, id, ...) {
  # fit model on X, time, event
  # predict survival probabilities for newdata at new.times
  list(pred = pred_matrix, fit = fitted_object)
}
```

The required arguments are:

| Argument | Meaning |
| --- | --- |
| `time` | numeric observed follow-up time |
| `event` | event indicator, with `1` for the event and `0` for censoring |
| `X` | training covariates as a data frame |
| `newdata` | covariates for prediction as a data frame |
| `new.times` | time grid where survival probabilities are needed |
| `obsWeights` | observation weights, possibly `NULL` |
| `id` | optional cluster or subject identifier, possibly `NULL` |
| `...` | learner-specific tuning parameters |

The returned `pred` must be a numeric matrix with:

- one row per row of `newdata`,
- one column per value of `new.times`,
- entries on the survival-probability scale, usually between `0` and `1`.

If possible, each row should be non-increasing across time. Wrappers commonly
clamp small numerical drift back into `[0, 1]`.

The returned `fit` can be any object needed for future prediction. When
`control = list(saveFitLibrary = TRUE)`, `SuperSurv()` stores these fitted
objects and `predict.SuperSurv()` later calls `predict()` on them.

## Prediction Method Contract

If a custom learner should support prediction on new data after fitting, store
a classed object in `fit` and define a matching S3 prediction method:

```r
predict.my.surv.learner <- function(object, newdata, new.times, ...) {
  pred_matrix
}
```

This method should return the same type of matrix as the wrapper's `pred`
component: `nrow(newdata)` rows by `length(new.times)` columns.

## Direct Survival Curves vs Risk Scores

Some survival learners directly return survival curves. In that case, the
wrapper usually interpolates the native prediction grid to `new.times`.

Other learners return a risk score or linear predictor. Those wrappers need one
extra calibration step:

1. fit the risk-score model,
2. estimate a baseline cumulative hazard on the training data,
3. predict risk scores for `newdata`,
4. convert scores to survival curves with `S(t | x) = exp(-exp(eta(x)) H0(t))`.

Several built-in Cox-style wrappers follow this pattern. The internal helper
`safe_breslow_step()` is used by some package wrappers, but it is not exported;
custom wrappers should either use a standard package prediction method that
already returns survival curves or implement their own baseline-hazard
calibration carefully.

## Minimal Custom Learner

The following learner ignores covariates and fits a weighted Kaplan-Meier curve.
It is intentionally simple, but it satisfies the full `SuperSurv()` learner
contract.

```{r custom-learner}
my.surv.km <- function(time, event, X, newdata, new.times,
                       obsWeights = NULL, id = NULL, ...) {
  if (is.null(obsWeights)) {
    obsWeights <- rep(1, length(time))
  }

  fit_km <- survival::survfit(
    survival::Surv(time, event) ~ 1,
    weights = obsWeights
  )

  step_surv <- stats::stepfun(fit_km$time, c(1, fit_km$surv), right = FALSE)
  surv_probs <- step_surv(new.times)

  pred <- matrix(
    surv_probs,
    nrow = nrow(newdata),
    ncol = length(new.times),
    byrow = TRUE
  )

  fit <- list(object = fit_km)
  class(fit) <- "my.surv.km"

  list(pred = pred, fit = fit)
}

predict.my.surv.km <- function(object, newdata, new.times, ...) {
  fit_km <- object$object
  step_surv <- stats::stepfun(fit_km$time, c(1, fit_km$surv), right = FALSE)
  surv_probs <- step_surv(new.times)

  matrix(
    surv_probs,
    nrow = nrow(newdata),
    ncol = length(new.times),
    byrow = TRUE
  )
}
```

You can test the wrapper outside the ensemble before adding it to a library:

```{r custom-learner-check}
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
times <- seq(50, 150, by = 50)

wrapper_out <- my.surv.km(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X[1:5, , drop = FALSE],
  new.times = times
)

dim(wrapper_out[["pred"]])
dim(predict(wrapper_out[["fit"]], newdata = X[1:5, , drop = FALSE], new.times = times))
```

The custom learner can be mixed with built-in learners:

```{r custom-fit, eval=FALSE}
fit <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_train,
  newdata = X_test,
  new.times = seq(50, 300, by = 50),
  event.library = c("surv.coxph", "my.surv.km"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE)
)

event_weights(fit)
```

## Screening Method Contract

Screening methods are also supplied by name. A screener should accept the same
basic training inputs and return a logical vector with one value per column of
`X`:

```r
my.screen.first3 <- function(time, event, X, obsWeights = NULL, id = NULL, ...) {
  keep <- rep(FALSE, ncol(X))
  keep[seq_len(min(3, ncol(X)))] <- TRUE
  names(keep) <- names(X)
  keep
}
```

Use a list entry to pair a learner with one or more screeners:

```{r custom-screener, eval=FALSE}
fit <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_train,
  newdata = X_test,
  new.times = seq(50, 300, by = 50),
  event.library = list(c("surv.coxph", "screen.all", "my.screen.first3")),
  cens.library = c("surv.coxph")
)

selected_variables(fit, learner = 2)
```

## Development Checklist

Before using a custom wrapper in a large ensemble, check:

- `pred` is a numeric matrix with dimensions `nrow(newdata)` by
  `length(new.times)`,
- predictions are on the survival-probability scale,
- `predict(wrapper_out[["fit"]], newdata, new.times)` returns the same shape when saved
  fits are needed,
- the wrapper tolerates `obsWeights = NULL` and `id = NULL`,
- all required packages are checked with `requireNamespace()`,
- any screener returns a logical vector aligned with the columns of `X`.

These rules are deliberately small. They allow new learners to be added without
changing the SuperSurv ensemble machinery.

---
title: "05. Advanced Hyperparameter Tuning & Grid Search"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{05. Advanced Hyperparameter Tuning & Grid Search}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

A single machine learning configuration is rarely optimal for every dataset. A Random Forest with deep trees might overfit on small data, while shallow trees might underfit on complex data. 

In the Super Learner framework, we handle "tuning" by adding multiple versions of the same algorithm to our library, each with different hyperparameters. The meta-learner automatically evaluates them via cross-validation and assigns the highest weight to the best configuration.

`SuperSurv` provides the `create_grid` function to automate this process entirely.

## 1. Prepare the Data

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

new.times <- seq(50, 200, by = 25) 
```

## 2. Defining the Search Space

Let's assume we want to tune a Random Survival Forest (`surv.rfsrc`). Instead of writing manual wrapper functions for every combination of parameters, we define a list of the values we want to test.

```{r, message=FALSE, warning=FALSE}

# Define our hyperparameter search space
rf_tuning_params <- list(
  nodesize = c(3, 15, 30), # Deep vs. Shallow trees
  ntree = c(100, 500)      # Number of trees in the forest
)
```

## 3. Generating the Tuning Grid

We pass our base learner name and parameter list into `create_grid()`. The function will automatically generate unique R functions for all 6 combinations in your global environment and return their names!

```{r create-grid}
# Automatically generate 6 different Random Forest wrappers
rf_grid <- create_grid(
  base_learner = "surv.rfsrc", 
  grid_params = rf_tuning_params
)

# View the dynamically generated function names
print(rf_grid)
```

## 4. Fitting the Tuned Ensemble

You can now pass this dynamically generated character vector directly into your `SuperSurv` library. We can combine our tuned ML grid with a baseline Cox model to ensure we always have a linear fallback.

```{r fit-tuned-model, eval=FALSE}
# Combine baseline with our 6 tuned forests
my_ml_library <- c("surv.coxph", rf_grid)

# Fit the ensemble (pseudo-code)
fit_tuned <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_ml_library,
  cens.library = c("surv.coxph"),
  nFolds = 3
)

# Inspect which hyperparameter combination won!
round(event_weights(fit_tuned), 3)
```

By leveraging `create_grid()`, you can easily execute a massive hyperparameter search space across XGBoost, SVMs, or Random Forests, mathematically guaranteeing that your final ensemble minimizes the cross-validated risk without any manual trial-and-error!

---
title: "03. Ensemble vs. Best Model Selection"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{03. Ensemble vs. Best Model Selection}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

In the theoretical framework of Super Learning, there are two distinct ways to utilize the cross-validated risks of your base algorithms:

1. **The Ensemble Super Learner:** Calculates a weighted average (convex combination) of the base learners. This "soft selection" smooths out variance, integrates different feature spaces, and generally yields the lowest finite-sample prediction error.
2. **The "Best" Model Selector:** Identifies the single algorithm with the lowest cross-validated risk and assigns it a weight of `1.0` (and all others `0.0`). This is a "hard selection" or "winner-take-all" approach. 



While the Ensemble is asymptotically optimal, selecting the single best model is incredibly useful when interpretability is strictly tied to one specific algorithm family. Instead of manually cherry-picking the best model—which introduces researcher bias and invalidates post-selection inference—`SuperSurv` automates the selection using rigorous, internal cross-validation. 

You can easily toggle between these two paradigms using the `selection` argument.

## 1. Prepare the Data

We load the `metabric` dataset and define our evaluation time grid exactly as we did in the previous tutorials.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Standard Train/Test Split
train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]
new.times <- seq(50, 200, by = 25)

# Define a diverse library of base learners
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")
```

## 2. Fit Both Super Learners

We will fit two separate `SuperSurv` models. The first will use the default Ensemble approach (`selection = "ensemble"`), and the second will use the winner-take-all approach (`selection = "best"`).

```{r fit-models, results='hide', message=FALSE, warning=FALSE}
# 1. The Ensemble Super Learner (Weighted Average)
fit_ensemble <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "ensemble",     # <-- Calculates fractional weights
  nFolds = 3
)

# 2. The 'Best' Super Learner (Winner-Take-All)
fit_best <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  selection = "best",         # <-- Selects the single best model
  nFolds = 3
)
```

## 3. Inspecting the Weights

The difference between the two methodologies is immediately obvious when we inspect the meta-learner coefficients with `event_weights()`. 

```{r inspect-weights}
cat("\n--- ENSEMBLE WEIGHTS (selection = 'ensemble') ---\n")
print(round(event_weights(fit_ensemble), 4))

cat("\n--- BEST MODEL WEIGHTS (selection = 'best') ---\n")
print(round(event_weights(fit_best), 4))
```

**Interpretation:** Notice that the Ensemble Super Learner distributes the weight across multiple models to minimize the overall loss function. The "Best" Super Learner simply looks at the cross-validated risk, identifies the champion, and gives it 100% of the weight. 

## 4. Evaluate and Compare Performance

A common question in clinical research is: *"Does the complexity of the Ensemble actually perform better than simply picking the best single model?"*

We can answer this instantly using `eval_summary()`. 

```{r evaluate-comparison}
# Evaluate the Ensemble
cat("Performance of the ENSEMBLE Super Learner:\n")
eval_summary(fit_ensemble, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)

cat("\nPerformance of the BEST MODEL Super Learner:\n")
eval_summary(fit_best, newdata = X_te, time = test$duration, 
             event = test$event, eval_times = new.times)
```

By comparing the resulting Brier scores and C-indices, you can empirically justify whether the "soft selection" of the ensemble is mathematically necessary for your specific dataset, or if the "hard selection" of a single model is sufficient.

---
title: "06. Machine Learning with Random Survival Forests"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{06. Machine Learning with Random Survival Forests}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

While classical models like the Cox Proportional Hazards model are highly interpretable, they rely on strict assumptions of linear, additive effects. Random Survival Forests (RSF) overcome this by building a non-linear ensemble of decision trees. 

In this tutorial, we will demonstrate how to run a standalone RSF using `SuperSurv`'s unified wrappers, evaluate its individual performance, and finally include it in a Super Learner ensemble.

## 1. Prepare the Data

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(123)

train_idx <- sample(1:nrow(metabric), 0.7 * nrow(metabric))
train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

X_tr <- train[, grep("^x", names(metabric))]
X_te <- test[, grep("^x", names(metabric))]

new.times <- seq(50, 200, by = 25) 
```

## 2. Standalone Machine Learning

`SuperSurv` provides unified wrappers that automatically handle the training and standardization of survival probabilities. You can use these completely independent of the Super Learner ensemble.

```{r standalone-rf}
# 1. Fit the standalone wrapper
rf_standalone <- surv.rfsrc(
  time = train$duration,
  event = train$event,
  X = X_tr,
  new.times = new.times 
)

# 2. Extract the fitted model object and prediction matrix
rf_fit <- rf_standalone[["fit"]]
rf_pred_matrix <- rf_standalone[["pred"]]
```

Because our plotting functions are universally compatible, we can plot individual patient curves directly from this standalone matrix:



```{r plot-standalone, fig.align='center'}
# Plot the first 3 patients in our training set
plot_predict(preds = rf_pred_matrix, eval_times = new.times, patient_idx = 1:3)
```

## 3. Evaluating the Standalone Model

We can also pass this standalone model directly into our evaluation suite to test its performance on new data. 

```{r eval-standalone, fig.align='center', fig.height=4, fig.width= 9}
# The function automatically detects this is a single model and plots it!
plot_benchmark(
  object = rf_fit,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)
```



```{r}

# plot_calibration(
#   object   = rf_fit,
#   newdata  = X_te,
#   time     = test$duration,
#   event    = test$event,
#   eval_time = 150,
#   bins     = 2
# )

```




## 4. Train the Benchmark Ensemble

While the standalone RSF is powerful, we can objectively evaluate if it outperforms classical models by putting them together in a `SuperSurv` ensemble. 

```{r fit-models, results='hide', message=FALSE, warning=FALSE}
my_library <- c("surv.coxph", "surv.weibull", "surv.rfsrc")

fit_supersurv <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = c("surv.coxph"), 
  control = list(saveFitLibrary = TRUE),
  verbose = FALSE,
  nFolds = 3
)
```

## 5. Visualizing Ensemble vs. Base Learners

When we pass the `SuperSurv` ensemble into the exact same benchmark function, it automatically unpacks the library and plots the ensemble against all its constituent models.



```{r plot-ensemble-benchmark, fig.align='center', fig.height=9}
plot_benchmark(
  object = fit_supersurv,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)
```

By incorporating advanced machine learning algorithms into your library, `SuperSurv` mathematically guarantees that your final predictions adapt to the complexity of your data, achieving the lowest possible Brier Score.

---
title: "01. SuperSurv with Ensemble"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{01. SuperSurv with Ensemble}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The core feature of the `SuperSurv` package is its ability to combine multiple base survival learners into a highly predictive meta-ensemble. This tutorial walks through preparing data, defining a library of models, fitting the Super Learner, and generating predictions for new patients.

## 1. Load Data & Prepare Matrices

We will use the built-in `metabric` dataset, extracting the covariates and performing a standard 80/20 train-test split.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

# Load built-in METABRIC data
data("metabric", package = "SuperSurv")

# Quick 80/20 Train-Test split
set.seed(42)
n_total <- nrow(metabric)
train_idx <- sample(1:n_total, 0.8 * n_total)

train <- metabric[train_idx, ]
test  <- metabric[-train_idx, ]

# Extract just the X covariates (assuming they are named x0, x1, etc.)
x_cols <- grep("^x", names(metabric), value = TRUE)
X_tr <- train[, x_cols]
X_te <- test[, x_cols]

# Define the prediction time grid (e.g., survival at 50, 100, 150, 200 months)
new.times <- c(50, 100, 150, 200)
```

## 2. Define the Ensemble Library

We define a library of base survival models. For this quick demonstration, we use lightning-fast parametric and tree-based models.

```{r define-library}
my_library <- c("surv.coxph", "surv.weibull", "surv.rpart")
```


## 3. Train the SuperSurv Metalearner

Before we run the main `SuperSurv` engine, it is important to understand its key parameters. The Super Learner algorithm relies on cross-validation to assign weights to the base models, and it must model both the event and the censoring mechanism to avoid biased evaluations.

Here is the complete guide to the arguments you need to pass:

### The Data Inputs
* **`time`**: A numeric vector of the observed follow-up times for your training cohort.
* **`event`**: A numeric vector indicating the status at the observed time (typically `1` = event occurred, `0` = right-censored).
* **`X`**: A `data.frame` or matrix containing **only** the predictor variables (covariates) for the training set. Do not include the time or event columns here!
* **`newdata`**: (Optional) A `data.frame` of covariates for a validation or test set. If provided, `SuperSurv` will immediately generate predictions for these patients during the training phase, saving you a step.
* **`new.times`**: A numeric vector defining the exact time points where you want survival probabilities predicted (e.g., `seq(50, 200, by = 25)`).

### The Model Libraries
* **`event.library`**: A character vector of the base algorithms used to predict the actual survival outcome (e.g., `"surv.coxph"`, `"surv.rpart"`).
* **`cens.library`**: The library used to estimate the *censoring* mechanism over time. `SuperSurv` uses these predictions to calculate Inverse Probability of Censoring Weights (IPCW). You can use the exact same library as your event models, or a simpler one.

### The Meta-Learner & Tuning
* **`metalearner`**: The optimization algorithm used to calculate the final ensemble weights. `SuperSurv` offers two distinct approaches:
    * `"brier"` (Default): Optimizes weights by minimizing the IPCW Brier score. Excellent for overall prediction accuracy.
    * `"logloss"`: Optimizes weights by minimizing the negative log-likelihood. Excellent for improving hazard discrimination.
* **`nFolds`**: The number of cross-validation folds used to train the meta-learner. `V = 5` or `V = 10` is standard. This cross-validation is what prevents the ensemble from overfitting to the base learners.
* **`control = list(saveFitLibrary = TRUE)`**: This tells the engine to save the fitted base learners into the final object. This is **required** if you want to use the `predict()` method on new patients later.
* **`verbose`**: Set to `TRUE` to print progress messages to the console. Highly recommended for large datasets or complex libraries so you can track the cross-validation progress.

Let's fit two models with `verbose = FALSE` to see how the meta-learner choice affects the final ensemble weights.

```{r fit-models, results='hide', message=FALSE, warning=FALSE}
# Fit 1: Least Squares Meta-learner
fit_ls <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,                 # Predict on the test set immediately
  new.times = new.times,       # Our evaluation time grid
  event.library = my_library,
  cens.library = my_library,
  metalearner = "brier", 
  control = list(saveFitLibrary = TRUE), 
  verbose = T,             # Turn to TRUE in practice to see progress!
  selection = "ensemble",
  nFolds = 5                   # 5-fold CV for the meta-learner
)

# Fit 2: Negative Log-Likelihood Meta-learner
fit_nll <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_library,
  cens.library = my_library,
  metalearner = "logloss",       # Swap to nloglik
  control = list(saveFitLibrary = TRUE), 
  verbose = FALSE,
  selection = "ensemble",
  nFolds = 5
)
```


## 4. Package Object Interface

`SuperSurv` fits behave like ordinary R model objects. Use the print and summary methods for a quick overview, and use accessors for common fitted-model details.

```{r object-interface}
fit_ls

summary(fit_ls)

event_weights(fit_ls)

learner_names(fit_ls)

eval_times(fit_ls)

selected_variables(fit_ls, learner = 1)
```


## 5. Inspect the Ensemble Weights and Risks

The defining feature of the Super Learner is that it does not just pick the single "best" model; it finds the optimal weighted combination of all models based on their cross-validated performance.

Let's inspect the weights and cross-validated risks for both of our meta-learners.

```{r inspect-weights}
cat("\n--- LEAST SQUARES METALEARNER ---\n")
summary(fit_ls)

cat("\n--- NLOGLIK METALEARNER ---\n")
summary(fit_nll)
```

### How to Interpret This:
* **`event_weights(fit)`**: These are the final weights assigned to each base learner. A weight of `0` means the meta-learner completely dropped that model because it did not contribute to overall predictive accuracy. A high weight means that model heavily influences the final ensemble prediction. Notice how the `least_squares` and `nloglik` algorithms might distribute these weights differently based on their optimization goals!
* **`summary(fit)`**: This reports the cross-validated risk for each individual base learner. The meta-learner uses these risks to calculate the optimal weights. Lower risk always indicates better performance.

## 6. Generating Predictions on New Data

If you passed `newdata` during the training phase, `SuperSurv` already calculated the predictions for your test set. However, in a real-world clinical setting, you will often train the model once and then predict on brand new patients months later. 

Because we set `control = list(saveFitLibrary = TRUE)` during training, we can use the standard R `predict()` method.

```{r predict-new}
# Select 3 brand new patients from our test set
new_patients <- X_te[1:6, ]

# Generate predictions using the Least Squares ensemble
ensemble_preds <- predict(
  object = fit_ls, 
  newdata = new_patients, 
  new.times = new.times,
  type = "event"
)

cat("\n--- PREDICTED SURVIVAL PROBABILITIES ---\n")
final_matrix <- ensemble_preds
colnames(final_matrix) <- paste0("Time_", new.times)
rownames(final_matrix) <- paste0("Patient_", 1:6)

print(round(final_matrix, 4))
```

### Understanding the Output Matrix:
The `predict()` function returns a list, but the most important element is `event.SL.predict`. 
* **Rows ($N$)**: Represent individual patients.
* **Columns ($T$)**: Represent the specific time points we defined in `new.times`.
* **Values**: The estimated probability that the patient will *survive* past that specific time point. As time increases (moving left to right across a row), the survival probability naturally decreases.

## 7. Visualizing Patient-Specific Predictions

While raw probability matrices ($N \times T$) are perfect for downstream coding and performance benchmarking, they are difficult to interpret clinically. Doctors and researchers need to see the actual survival trajectories.

`SuperSurv` includes a built-in `plot_predict()` function to effortlessly translate this matrix into publication-ready survival curves for individual patients.


```{r plot-predict, fig.align='center', fig.width=6, fig.height=5}
# Plot the predicted survival curves for our 3 new patients (Rows 1, 2, and 3)
plot_predict(
  preds = ensemble_preds,
  eval_times = new.times,
  patient_idx = 1:6
)
```

## 8. Next Steps

You now know how to prepare data, define a model library, choose a meta-learner, and generate patient-specific survival curves. 

However, before deploying a model in clinical practice, you must rigorously prove that the ensemble actually outperforms the individual base learners. Head over to **Tutorial 2: Model Performance & Benchmarking** to learn how to generate time-dependent Brier Score, AUC, and Uno's C-index plots!

---
title: "04. High-Dimensional Data & Variable Screening"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{04. High-Dimensional Data & Variable Screening}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

In modern clinical research and bioinformatics, it is common to encounter high-dimensional datasets where the number of predictors (genes, biomarkers, clinical features) far exceeds the number of observed events ($p \gg n$). 

Feeding thousands of raw variables directly into complex machine learning algorithms introduces two massive problems:
1. **Computational Bottlenecks:** Algorithms like Random Survival Forests become prohibitively slow.
2. **Overfitting:** Models will mathematically memorize background noise instead of true biological signals, destroying out-of-sample prediction accuracy.



`SuperSurv` elegantly handles this through **Variable Screening**. Screening acts as a computational filter *before* the models are fitted. It evaluates all predictors, drops the statistical noise, and only passes the most highly associated features into your base learners.

## 1. Simulating High-Dimensional Data

To demonstrate how screening works, we will use our standard `metabric` dataset, but we will artificially inject 100 columns of pure random noise. This simulates a high-dimensional genomic dataset where most measured features have absolutely no relationship to patient survival.

```{r setup, message=FALSE, warning=FALSE}
library(SuperSurv)
library(survival)

data("metabric", package = "SuperSurv")
set.seed(42)

# Create 100 columns of random noise (simulating irrelevant genes/biomarkers)
n_patients <- nrow(metabric)
noise_data <- matrix(rnorm(n_patients * 20), nrow = n_patients)
colnames(noise_data) <- paste0("noise_", 1:20)
# Combine the real data with the noise
high_dim_data <- cbind(metabric, noise_data)

# Standard Train/Test Split
train_idx <- sample(1:nrow(high_dim_data), 0.7 * nrow(high_dim_data))
train <- high_dim_data[train_idx, ]
test  <- high_dim_data[-train_idx, ]

X_tr <- train[, grep("^x|^noise", names(high_dim_data))]
X_te <- test[, grep("^x|^noise", names(high_dim_data))]
new.times <- seq(50, 200, by = 25)
```

## 2. Defining the Screening Algorithms

Just as we have a library for prediction algorithms, `SuperSurv` utilizes a library for screening algorithms. You can pair any prediction model with any screening function to create a streamlined pipeline. 

`SuperSurv` includes a comprehensive suite of built-in screeners:
* **`screen.marg`**: Marginal screening (keeps variables with strong univariate associations).
* **`screen.glmnet`**: Penalized Lasso screening (shrinks irrelevant feature coefficients to exact zero).
* **`screen.elasticnet`**: Elastic Net screening (combines Lasso and Ridge penalties; highly recommended for highly correlated biological features like gene expression data).
* **`screen.rfsrc`**: Tree-based screening (keeps features with high Random Forest variable importance).
* **`screen.var`**: Unsupervised screening (drops features with near-zero variance).
* **`screen.all`**: A baseline passthrough (keeps all variables without screening).



Let's build a library that tests different combinations of prediction and screening:

```{r define-library}
# We pass a list of vectors. The first element is the prediction model, 
# and the second element is the screening algorithm.

my_screen_library <- list(
  c("surv.coxph", "screen.all"),         # Baseline: Cox model with ALL variables
  c("surv.coxph", "screen.marg"),        # Screen by marginal association, then fit Cox
  c("surv.weibull", "screen.elasticnet"),# Screen via Elastic Net, then fit Weibull
  c("surv.rpart", "screen.var")          # Drop zero-variance noise, then fit a Tree
)

# For the censoring library, we use an unscreened approach 
cens_library <- c("surv.coxph")
```
## 3. Training with Embedded Screening

When we pass this paired library into `SuperSurv`, the meta-learner handles the cross-validation with absolute statistical rigor. 

**Crucial Methodological Detail:** The screening step is performed *inside* the cross-validation folds. If you screen the entire dataset before splitting it into folds, you cause "data leakage," leading to overly optimistic benchmark metrics. `SuperSurv` protects you from this automatically.

```{r fit-screen, results='hide', message=FALSE, warning=FALSE}
fit_highdim <- SuperSurv(
  time = train$duration,
  event = train$event,
  X = X_tr,
  newdata = X_te,
  new.times = new.times,
  event.library = my_screen_library, # Pass our screened library
  cens.library = cens_library,
  control = list(saveFitLibrary = TRUE),
  verbose = T,
  nFolds = 3
)
```

## 4. Evaluating the Screened Ensemble

Despite injecting 100 columns of pure noise, our ensemble still performs exceptionally well because the screening algorithms successfully filtered out the irrelevant variables before the base learners could overfit to them.

We can evaluate this exactly as we did in previous tutorials using `eval_summary()` and `plot_benchmark()`.

```{r evaluate-screen}
# Check the integrated performance
screened_performance <- eval_summary(
  object = fit_highdim,
  newdata = X_te,
  time = test$duration,
  event = test$event,
  eval_times = new.times
)

# Plot the benchmark
# plot_benchmark(fit_highdim, newdata = X_te, time = test$duration, event = test$event, eval_times = new.times)
```

By leveraging `SuperSurv`'s screening capabilities, you can scale your survival analysis to massive genomic and clinical datasets without sacrificing computational speed or predictive accuracy.


## 5. Identifying the Selected Features

After the Super Learner finishes its cross-validation and fitting, you likely want to know *which* variables actually survived the screening process. In clinical and biological research, these selected features often represent the most important biomarkers or predictive genes.

`SuperSurv` provides accessor functions for fitted model details, so you can inspect screening results without relying on internal list components.

```{r extract-features}
# Get the variables retained by the second event learner.
selected_features <- selected_variables(fit_highdim, learner = 2)

cat("Total features evaluated:", ncol(X_tr), "\n")
cat("Features retained by Elastic Net:", length(selected_features), "\n\n")

cat("Selected Features:\n")
print(selected_features)
```

By extracting these features, you can seamlessly transition from pure predictive modeling into downstream biological inference or pathway analysis.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screener.R
\name{screen.var}
\alias{screen.var}
\title{High Variance Screening Algorithm (Unsupervised)}
\usage{
screen.var(
  time,
  event,
  X,
  obsWeights = NULL,
  keep_fraction = 0.5,
  minscreen = 2,
  ...
)
}
\arguments{
\item{time}{Numeric vector of observed follow-up times (Ignored internally).}

\item{event}{Numeric vector of event indicators (Ignored internally).}

\item{X}{Training covariate data.frame or matrix.}

\item{obsWeights}{Numeric vector of observation weights (Ignored internally).}

\item{keep_fraction}{Numeric value between 0 and 1. The fraction of highest-variance
features to retain. Defaults to 0.5 (keeps the top 50\%).}

\item{minscreen}{Integer. Minimum number of covariates to return. Defaults to 2.}

\item{...}{Additional ignored arguments.}
}
\value{
A logical vector of the same length as the number of columns in \code{X},
indicating which variables passed the screening algorithm (\code{TRUE} to keep,
\code{FALSE} to drop).
}
\description{
An unsupervised screening algorithm that filters out low-variance features.
This is particularly useful for high-dimensional genomic or transcriptomic data
where many features remain relatively constant across all observations.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:6]
X <- dat[, x_cols, drop = FALSE]

screen.var(
  time = dat$duration,
  event = dat$event,
  X = X,
  keep_fraction = 0.5,
  minscreen = 2
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmst.R
\name{plot_marginal_rmst_curve}
\alias{plot_marginal_rmst_curve}
\title{Plot Adjusted Marginal RMST Contrast Over Time}
\usage{
plot_marginal_rmst_curve(
  fit,
  data,
  trt_col,
  times,
  tau_seq,
  inference = FALSE,
  B = 200,
  seed = NULL,
  ci_level = 0.95
)
}
\arguments{
\item{fit}{A fitted \code{SuperSurv} ensemble object.}

\item{data}{A \code{data.frame} containing the covariates and the binary grouping variable.}

\item{trt_col}{Character string. The exact name of the binary grouping variable in \code{data}.}

\item{times}{Numeric vector of time points matching the prediction grid.}

\item{tau_seq}{Numeric vector. A sequence of restriction times (\code{tau}) to evaluate and plot.}

\item{inference}{Logical; if \code{TRUE}, compute perturbation-based confidence intervals.
Defaults to \code{FALSE}.}

\item{B}{Integer. Number of perturbation replicates used when \code{inference = TRUE}.
Defaults to \code{200}.}

\item{seed}{Optional integer seed for reproducibility.}

\item{ci_level}{Numeric scalar in \code{(0,1)} specifying the confidence level for the
confidence interval. Defaults to \code{0.95}.}
}
\value{
A \code{ggplot} object visualizing the adjusted marginal RMST contrast curve.
}
\description{
Generates a curve showing how the adjusted marginal restricted mean survival
time (RMST) contrast evolves across a sequence of restriction times.
}
\details{
If \code{inference = TRUE}, the function additionally displays perturbation-based
Wald confidence intervals at each value of \code{tau}.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:80, ]
x_cols <- grep("^x", names(dat), value = TRUE)[1:5]
X <- dat[, x_cols, drop = FALSE]
new.times <- seq(20, 120, by = 20)

fit <- SuperSurv(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X,
  new.times = new.times,
  event.library = c("surv.coxph", "surv.glmnet"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE)
)

tau_grid <- seq(40, 120, by = 20)
plot_marginal_rmst_curve(
  fit = fit,
  data = dat,
  trt_col = "x4",
  times = new.times,
  tau_seq = tau_grid,
  inference = TRUE,
  B = 100,
  seed = 123
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.SuperSurv.R
\name{predict.SuperSurv}
\alias{predict.SuperSurv}
\title{Predict method for SuperSurv fits}
\usage{
\method{predict}{SuperSurv}(
  object,
  newdata,
  new.times,
  type = c("both", "event", "censoring"),
  onlySL = FALSE,
  threshold = 1e-04,
  ...
)
}
\arguments{
\item{object}{A fitted object of class \code{SuperSurv}.}

\item{newdata}{A data.frame of new covariate values.}

\item{new.times}{A numeric vector of times at which to predict survival.}

\item{type}{Character string specifying the prediction output. Use
\code{"event"} for the event survival matrix, \code{"censoring"} for the
censoring survival matrix, or \code{"both"} for the full list of outputs.}

\item{onlySL}{Logical. If TRUE, only uses models with weights > threshold.}

\item{threshold}{Numeric. The weight threshold for onlySL.}

\item{...}{Additional ignored arguments.}
}
\value{
If \code{type = "event"} or \code{type = "censoring"}, a numeric
matrix with rows corresponding to observations and columns corresponding to
\code{new.times}. If \code{type = "both"}, a list containing:
\itemize{
\item \code{event.predict}: A numeric matrix of final event survival predictions.
\item \code{event.library.predict}: A 3D numeric array of event learner predictions.
\item \code{cens.predict}: A numeric matrix of final censoring survival predictions.
\item \code{cens.library.predict}: A 3D numeric array of censoring learner predictions.
}
}
\description{
Obtains predicted survival probabilities from a fitted SuperSurv ensemble.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  newX <- X[1:10, , drop = FALSE]
  new.times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = new.times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  pred_event <- predict(
    object = fit,
    newdata = newX,
    new.times = new.times,
    type = "event"
  )

  dim(pred_event)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmst.R
\name{estimate_marginal_rmst}
\alias{estimate_marginal_rmst}
\title{Estimate an Adjusted Marginal RMST Contrast}
\usage{
estimate_marginal_rmst(
  fit,
  data,
  trt_col,
  times,
  tau,
  inference = FALSE,
  B = 200,
  seed = NULL,
  ci_level = 0.95
)
}
\arguments{
\item{fit}{A fitted object of class \code{"SuperSurv"}.}

\item{data}{A \code{data.frame} containing the covariates used for
standardization, including the binary grouping variable specified by
\code{trt_col}.}

\item{trt_col}{Character string giving the name of the binary grouping
variable in \code{data}. The variable is set to 1 and 0, respectively, to
generate the two standardized prediction regimes.}

\item{times}{Numeric vector of prediction time points corresponding to the
evaluation grid used for survival prediction.}

\item{tau}{Numeric scalar giving the restriction horizon for RMST. Must not
exceed \code{max(times)}.}

\item{inference}{Logical; if \code{TRUE}, compute perturbation-based standard
errors, confidence intervals, and a Wald-type p-value. Defaults to
\code{FALSE}.}

\item{B}{Integer giving the number of perturbation replicates when
\code{inference = TRUE}. Defaults to \code{200}.}

\item{seed}{Optional integer seed for reproducibility of the perturbation
procedure.}

\item{ci_level}{Numeric scalar in \code{(0, 1)} specifying the confidence
level for the Wald-type confidence interval. Defaults to \code{0.95}.}
}
\value{
A list containing:
\itemize{
\item \code{ATE_RMST}: The estimated adjusted marginal RMST contrast
\eqn{\widehat{\Delta}_{RMST}(\tau)}.
\item \code{mean_RMST_Treated}: The average predicted RMST under
\code{A = 1}.
\item \code{mean_RMST_Control}: The average predicted RMST under
\code{A = 0}.
\item \code{tau}: The restriction horizon used for integration.
\item \code{patient_rmst_treated}: Vector of individual-level predicted
RMST values under \code{A = 1}.
\item \code{patient_rmst_control}: Vector of individual-level predicted
RMST values under \code{A = 0}.
\item \code{patient_delta_rmst}: Vector of individual-level predicted RMST
contrasts.
\item \code{inference}: Logical indicator for whether perturbation-based
inference was requested.
\item \code{B}: Number of perturbation replicates used when
\code{inference = TRUE}; otherwise \code{NULL}.
\item \code{SE_RMST}: Perturbation-based standard error of the RMST
contrast; otherwise \code{NULL}.
\item \code{CI_RMST}: Wald-type confidence interval for the RMST contrast;
otherwise \code{NULL}.
\item \code{z_value}: Wald-type test statistic; otherwise \code{NULL}.
\item \code{p_value}: Two-sided Wald-type p-value; otherwise \code{NULL}.
\item \code{perturb_reps}: Vector of perturbation replicate estimates;
otherwise \code{NULL}.
}
}
\description{
Computes a covariate-adjusted marginal contrast on the restricted mean
survival time (RMST) scale using standardization (g-computation) based on a
fitted \code{SuperSurv} model.
}
\details{
For a binary grouping variable \code{trt_col}, the function predicts
counterfactual survival curves under \code{A = 1} and \code{A = 0} for every
individual in the supplied dataset, integrates each curve up to the
restriction time \code{tau}, and averages the resulting individual-level
RMST differences. The resulting contrast is generally interpreted as an
adjusted marginal contrast. When \code{trt_col} corresponds to a manipulable
intervention and additional identification assumptions hold, the same
standardized procedure may also support a causal interpretation.

If \code{inference = TRUE}, the function additionally performs a
perturbation-based inference procedure conditional on the fitted
\code{SuperSurv} model. In this implementation, the fitted learner library,
hyperparameters, base learners, and ensemble weights are held fixed, and
random positive weights are applied to the individual-level RMST contrasts to
estimate a perturbation-based standard error, Wald-type confidence interval,
and p-value.

The function uses the empirical distribution of the observed covariates in
\code{data} as the standardization distribution. RMST is evaluated
numerically from the predicted survival matrix using a left Riemann sum over
the supplied grid \code{times}.

The perturbation-based inference implemented here is conditional on the
fitted \code{SuperSurv} model. It does not re-tune hyperparameters, reselect
the learner library, or refit the base learners under each perturbation.
Instead, it perturbs the aggregation of the individual-level predicted RMST
contrasts. This yields a lightweight uncertainty quantification procedure for
the standardized RMST contrast given the final fitted ensemble.
}
\examples{
\dontrun{
data("metabric", package = "SuperSurv")
x_cols <- grep("^x", names(metabric), value = TRUE)
X <- metabric[, x_cols]
new.times <- seq(10, 150, by = 10)

fit <- SuperSurv(
  time = metabric$duration,
  event = metabric$event,
  X = X,
  newdata = X,
  new.times = new.times,
  event.library = c("surv.coxph", "surv.rfsrc"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE),
  nFolds = 3
)

rmst_res <- estimate_marginal_rmst(
  fit = fit,
  data = metabric,
  trt_col = "x4",
  times = new.times,
  tau = 100,
  inference = TRUE,
  B = 200,
  seed = 123
)

rmst_res$ATE_RMST
rmst_res$SE_RMST
rmst_res$CI_RMST
format.pval(rmst_res$p_value, digits = 3, eps = 1e-16)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/control.R
\name{SuperSurv.CV.control}
\alias{SuperSurv.CV.control}
\title{Control parameters for Cross-Validation in SuperSurv}
\usage{
SuperSurv.CV.control(
  V = 10L,
  stratifyCV = TRUE,
  shuffle = TRUE,
  validRows = NULL
)
}
\arguments{
\item{V}{Number of folds. Default is 10.}

\item{stratifyCV}{Logical. If TRUE, ensures event rates are balanced across folds.}

\item{shuffle}{Logical. If TRUE, shuffles rows before splitting.}

\item{validRows}{Optional custom list of indices for folds.}
}
\value{
A list of CV parameters.
}
\description{
Control parameters for Cross-Validation in SuperSurv
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explainability.R
\name{plot_beeswarm}
\alias{plot_beeswarm}
\title{Beeswarm Summary Plot for SuperSurv SHAP}
\usage{
plot_beeswarm(shap_values, data, top_n = 10)
}
\arguments{
\item{shap_values}{The output from \code{explain_kernel()}.}

\item{data}{The covariate data used (X_explain)}

\item{top_n}{Number of features to display}
}
\value{
A \code{ggplot} object visualizing the SHAP values.
}
\description{
Beeswarm Summary Plot for SuperSurv SHAP
}
\examples{
if (requireNamespace("fastshap", quietly = TRUE) &&
    requireNamespace("ggforce", quietly = TRUE) &&
    requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  new.times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = new.times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  shap_values <- explain_kernel(
    model = fit,
    X_explain = X[1:20, , drop = FALSE],
    X_background = X[21:50, , drop = FALSE],
    nsim = 5
  )

  plot_beeswarm(
    shap_values = shap_values,
    data = X[1:20, , drop = FALSE],
    top_n = 5
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.bart}
\alias{surv.bart}
\title{Wrapper for BART (Bayesian Additive Regression Trees)}
\usage{
surv.bart(
  time,
  event,
  X,
  newdata = NULL,
  new.times,
  obsWeights = NULL,
  id = NULL,
  ntree = 10,
  ndpost = 30,
  nskip = 10,
  ...
)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Times at which to obtain predicted survivals.}

\item{obsWeights}{Observation weights (Note: BART does not natively support weights).}

\item{id}{Optional cluster/individual ID indicator.}

\item{ntree}{Number of trees (default: 50).}

\item{ndpost}{Number of posterior draws (default: 1000).}

\item{nskip}{Number of burn-in draws (default: 250).}

\item{...}{Additional arguments passed to \code{\link[BART]{mc.surv.bart}}.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
If the model fails to fit, this may be an object of class \code{try-error}.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at the specified \code{new.times} grid.
}
}
\description{
Final Production Wrapper for BART (Tunable & Robust).
Uses the \code{\link[BART]{mc.surv.bart}} function.
Automatically reshapes the flat output vector into a survival matrix
and interpolates the predictions to the requested new.times.
}
\examples{
\dontrun{
if (.Platform$OS.type != "windows" &&
  requireNamespace("BART", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:20, ]
  x_cols <- grep("^x", names(dat))[1:3]
  X <- dat[, x_cols, drop = FALSE]
  newX <- X[1:5, , drop = FALSE]
  times <- seq(50, 150, by = 50)

  fit <- surv.bart(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = newX,
    new.times = times,
    ntree = 3,
    ndpost = 5,
    nskip = 5
  )

  dim(fit[["pred"]])
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmst.R
\name{plot_rmst_vs_obs}
\alias{plot_rmst_vs_obs}
\title{Plot Predicted RMST vs. Observed Survival Times}
\usage{
plot_rmst_vs_obs(fit, data, time_col, event_col, times, tau)
}
\arguments{
\item{fit}{A fitted \code{SuperSurv} ensemble object.}

\item{data}{A \code{data.frame} containing the patient covariates, times, and events.}

\item{time_col}{Character string. The exact name of the observed follow-up time column in \code{data}.}

\item{event_col}{Character string. The exact name of the event indicator column in \code{data} (e.g., 1 for event, 0 for censored).}

\item{times}{Numeric vector of time points matching the prediction grid.}

\item{tau}{Numeric. A single truncation time limit up to which the RMST is calculated.}
}
\value{
A \code{ggplot} object comparing predicted RMST to observed outcomes.
}
\description{
Evaluates the calibration of the causal RMST estimator by plotting the model's
predicted RMST for each patient against their actual observed follow-up time.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:80, ]
x_cols <- grep("^x", names(dat))[1:5]
X <- dat[, x_cols, drop = FALSE]
new.times <- seq(20, 120, by = 20)

fit <- SuperSurv(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = X,
  new.times = new.times,
  event.library = c("surv.coxph", "surv.glmnet"),
  cens.library = c("surv.coxph"),
  control = list(saveFitLibrary = TRUE)
)

plot_rmst_vs_obs(
  fit = fit,
  data = dat,
  time_col = "duration",
  event_col = "event",
  times = new.times,
  tau = 350
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screener.R
\name{screen.rfsrc}
\alias{screen.rfsrc}
\title{Random Survival Forest Screening Algorithm}
\usage{
screen.rfsrc(
  time,
  event,
  X,
  obsWeights = NULL,
  minscreen = 2,
  ntree = 100,
  ...
)
}
\arguments{
\item{time}{Numeric vector of observed follow-up times.}

\item{event}{Numeric vector of event indicators (1 = event, 0 = censored).}

\item{X}{Training covariate data.frame or matrix.}

\item{obsWeights}{Numeric vector of observation weights.}

\item{minscreen}{Integer. Minimum number of covariates to return. Defaults to 2.}

\item{ntree}{Integer. Number of trees to grow. Defaults to 100 for fast screening.}

\item{...}{Additional arguments passed to \code{\link[randomForestSRC]{rfsrc}}.}
}
\value{
A logical vector of the same length as the number of columns in \code{X},
indicating which variables passed the screening algorithm (\code{TRUE} to keep,
\code{FALSE} to drop).
}
\description{
This screening algorithm uses the \code{randomForestSRC} package to select covariates
based on their Variable Importance (VIMP). It grows a fast forest and retains features
with a VIMP greater than zero.
}
\examples{
if (requireNamespace("randomForestSRC", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:40, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]

  screen.rfsrc(
    time = dat$duration,
    event = dat$event,
    X = X,
    minscreen = 2,
    ntree = 10
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_benchmark}
\alias{plot_benchmark}
\title{Plot Longitudinal Benchmark Metrics}
\usage{
plot_benchmark(
  object,
  newdata,
  time,
  event,
  eval_times,
  metrics = c("brier", "auc", "cindex"),
  verbose = FALSE
)
}
\arguments{
\item{object}{A fitted SuperSurv object OR a fitted standalone learner.}

\item{newdata}{A data.frame of test covariates.}

\item{time}{Numeric vector of observed follow-up times for the test set.}

\item{event}{Numeric vector of event indicators for the test set.}

\item{eval_times}{Numeric vector of times at which to evaluate predictions.}

\item{metrics}{Character vector specifying which plots to return.
Options: "brier", "auc", "cindex". Defaults to all three.}

\item{verbose}{Logical; if TRUE, progress messages are shown. Defaults to FALSE.}
}
\value{
A combined patchwork ggplot object, or a single ggplot if only one metric is selected.
}
\description{
Generates time-dependent performance curves comparing the SuperSurv ensemble
against its base learners, or evaluates a single standalone learner.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:120, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  eval_times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = eval_times,
    event.library = c("surv.coxph", "surv.ranger"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  plot_benchmark(
    object = fit,
    newdata = X,
    time = dat$duration,
    event = dat$event,
    eval_times = eval_times,
    metrics = c("brier")
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screener.R
\name{screen.elasticnet}
\alias{screen.elasticnet}
\title{Elastic Net Screening Algorithm}
\usage{
screen.elasticnet(
  time,
  event,
  X,
  obsWeights = NULL,
  alpha = 0.5,
  minscreen = 2,
  nfolds = 10,
  nlambda = 100,
  ...
)
}
\arguments{
\item{time}{Numeric vector of observed follow-up times.}

\item{event}{Numeric vector of event indicators (1 = event, 0 = censored).}

\item{X}{Training covariate data.frame or matrix.}

\item{obsWeights}{Numeric vector of observation weights.}

\item{alpha}{Numeric penalty exponent for \code{glmnet}. Defaults to 0.5 (Elastic Net).}

\item{minscreen}{Integer. Minimum number of covariates to return. Defaults to 2.}

\item{nfolds}{Integer. Number of folds for cross-validation. Defaults to 10.}

\item{nlambda}{Integer. Number of penalty parameters to search over. Defaults to 100.}

\item{...}{Additional arguments passed to \code{screen.glmnet}.}
}
\value{
A logical vector of the same length as the number of columns in \code{X},
indicating which variables passed the screening algorithm (\code{TRUE} to keep,
\code{FALSE} to drop).
}
\description{
This screening algorithm uses \code{\link[glmnet]{cv.glmnet}} to select covariates.
Unlike LASSO (\code{alpha = 1}), which drops correlated features, Elastic Net
(\code{alpha = 0.5} by default) shrinks correlated groups of features together,
making it ideal for selecting entire biological pathways.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:40, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]

  screen.elasticnet(
    time = dat$duration,
    event = dat$event,
    X = X,
    alpha = 0.5,
    minscreen = 2,
    nfolds = 3,
    nlambda = 20
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{summary.SuperSurv}
\alias{summary.SuperSurv}
\alias{print.summary.SuperSurv}
\title{Summarize a SuperSurv fit}
\usage{
\method{summary}{SuperSurv}(object, ...)

\method{print}{summary.SuperSurv}(x, digits = max(3L, getOption("digits") - 3L), ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{...}{Additional arguments ignored.}

\item{x}{A summary object produced by \code{summary.SuperSurv()}.}

\item{digits}{Number of significant digits to use for displayed weights and
risks.}
}
\value{
An object of class \code{"summary.SuperSurv"}, a list containing the
matched call, selection mode, event and censoring learner summaries,
prediction dimensions, and timing information.
}
\description{
Summarizes the fitted event and censoring ensembles, including learner names,
ensemble weights, cross-validated risks, error flags, prediction dimensions,
and recorded timing information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screener.R
\name{screen.glmnet}
\alias{screen.glmnet}
\title{GLMNET (Lasso) Screening}
\usage{
screen.glmnet(
  time,
  event,
  X,
  obsWeights = NULL,
  alpha = 1,
  minscreen = 2,
  nfolds = 10,
  nlambda = 100,
  ...
)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{obsWeights}{Observation weights.}

\item{alpha}{Penalty exponent (1 = lasso).}

\item{minscreen}{Minimum number of covariates to return. Defaults to 2.}

\item{nfolds}{Number of CV folds.}

\item{nlambda}{Number of penalty parameters.}

\item{...}{Additional ignored arguments.}
}
\value{
A logical vector of the same length as the number of columns in \code{X},
indicating which variables passed the screening algorithm (\code{TRUE} to keep,
\code{FALSE} to drop).
}
\description{
GLMNET (Lasso) Screening
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:40, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]

  screen.glmnet(
    time = dat$duration,
    event = dat$event,
    X = X,
    alpha = 1,
    minscreen = 2,
    nfolds = 3,
    nlambda = 20
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{eval_times}
\alias{eval_times}
\alias{eval_times.SuperSurv}
\title{Access SuperSurv prediction evaluation times}
\usage{
eval_times(object, ...)

\method{eval_times}{SuperSurv}(object, ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{...}{Additional arguments ignored.}
}
\value{
A numeric vector of prediction evaluation times.
}
\description{
Returns the time grid used for the fitted \code{SuperSurv} predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.km}
\alias{surv.km}
\title{Kaplan-Meier Prediction Algorithm}
\usage{
surv.km(time, event, X, newdata, new.times, obsWeights, id, ...)
}
\arguments{
\item{time}{Numeric vector of observed follow-up times.}

\item{event}{Numeric vector of event indicators (1 = event, 0 = censored).}

\item{X}{Training covariate data.frame (Ignored by KM).}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Numeric vector of times at which to predict survival.}

\item{obsWeights}{Numeric vector of observation weights.}

\item{id}{Optional vector indicating subject/cluster identities.}

\item{...}{Additional ignored arguments.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: A list containing the fitted \code{\link[survival]{survfit}} object.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at \code{new.times}.
}
}
\description{
This prediction algorithm ignores all covariates and computes the marginal
Kaplan-Meier survival estimator using the \code{\link[survival]{survfit}} function.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:30, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
newX <- X[1:5, , drop = FALSE]
times <- seq(50, 150, by = 50)

fit <- surv.km(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = newX,
  new.times = times,
  obsWeights = rep(1, nrow(dat)),
  id = NULL
)

dim(fit[["pred"]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{eval_cindex}
\alias{eval_cindex}
\title{Calculate Concordance Index (Harrell's or Uno's)}
\usage{
eval_cindex(time, event, S_mat, times, eval_time, method = "uno")
}
\arguments{
\item{time}{Numeric vector of observed follow-up times.}

\item{event}{Numeric vector of event indicators (1 = event, 0 = censored).}

\item{S_mat}{A numeric matrix of predicted survival probabilities.}

\item{times}{Numeric vector of evaluation times matching the columns of \code{S_mat}.}

\item{eval_time}{Numeric. The specific time point at which to extract predictions.}

\item{method}{Character. Either "harrell" or "uno". Defaults to "uno".}
}
\value{
A numeric value representing the chosen C-index.
}
\description{
Calculate Concordance Index (Harrell's or Uno's)
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
newX <- X[1:10, , drop = FALSE]
times <- seq(50, 150, by = 50)

fit <- surv.coxph(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = newX,
  new.times = times,
  obsWeights = rep(1, nrow(dat)),
  id = NULL
)

eval_cindex(
  time = dat$duration[1:10],
  event = dat$event[1:10],
  S_mat = fit[["pred"]],
  times = times,
  eval_time = 100,
  method = "uno"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screener.R
\name{screen.all}
\alias{screen.all}
\title{Keep All Variables Screener}
\usage{
screen.all(X, ...)
}
\arguments{
\item{X}{Training covariate data.frame.}

\item{...}{Additional ignored arguments.}
}
\value{
A logical vector of the same length as the number of columns in \code{X},
indicating which variables passed the screening algorithm (\code{TRUE} to keep,
\code{FALSE} to drop).
}
\description{
Keep All Variables Screener
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:20, ]
x_cols <- grep("^x", names(dat))[1:5]
X <- dat[, x_cols, drop = FALSE]

screen.all(X)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explainability.R
\name{explain_kernel}
\alias{explain_kernel}
\title{Explain Predictions with Global SHAP (Kernel SHAP)}
\usage{
explain_kernel(
  model,
  X_explain,
  X_background,
  nsim = 20,
  only_best = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{model}{A fitted SuperSurv object OR a single wrapper output.}

\item{X_explain}{The dataset you want to explain (e.g., \code{X_test[1:10, ]}).}

\item{X_background}{The reference dataset for fastshap (e.g., \code{X_train[1:100, ]}).}

\item{nsim}{Number of simulations. Defaults to 20.}

\item{only_best}{Logical. If TRUE and model is SuperSurv, only explains the highest-weighted base learner.}

\item{verbose}{Logical; if \code{TRUE}, progress messages are shown.}
}
\value{
A data.frame of class \code{c("explain", "data.frame")} containing the calculated SHAP values. The columns correspond to the covariates in \code{X_explain}.
}
\description{
Explain Predictions with Global SHAP (Kernel SHAP)
}
\examples{
if (requireNamespace("fastshap", quietly = TRUE) &&
    requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  new.times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = new.times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  shap_values <- explain_kernel(
    model = fit,
    X_explain = X[1:10, , drop = FALSE],
    X_background = X[11:40, , drop = FALSE],
    nsim = 5
  )

  dim(shap_values)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.parametric}
\alias{surv.parametric}
\title{Universal Parametric Survival Wrapper}
\usage{
surv.parametric(
  time,
  event,
  X,
  newdata,
  new.times,
  obsWeights,
  id,
  dist = "weibull",
  ...
)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Times at which to obtain predicted survivals.}

\item{obsWeights}{Observation weights.}

\item{id}{Optional cluster/individual ID indicator.}

\item{dist}{Distribution for the AFT model (default: "weibull").}

\item{...}{Additional arguments passed to \code{\link[survival]{survreg}}.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
If the model fails to fit, this may be an object of class \code{try-error}.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at the specified \code{new.times} grid.
}
}
\description{
Final Production Wrapper for AFT Models (Weibull, Exponential, LogNormal, LogLogistic).
Replaces individual wrappers with one robust, vectorized function.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:30, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
newX <- X[1:5, , drop = FALSE]
times <- seq(50, 150, by = 50)

fit <- surv.parametric(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = newX,
  new.times = times,
  obsWeights = rep(1, nrow(dat)),
  id = NULL,
  dist = "weibull"
)

dim(fit[["pred"]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{training_variables}
\alias{training_variables}
\alias{training_variables.SuperSurv}
\title{Access SuperSurv training variable names}
\usage{
training_variables(object, ...)

\method{training_variables}{SuperSurv}(object, ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{...}{Additional arguments ignored.}
}
\value{
A character vector of training variable names.
}
\description{
Returns the covariate names used to fit a \code{SuperSurv} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{eval_brier}
\alias{eval_brier}
\title{IPCW Brier Score and Integrated Brier Score (IBS)}
\usage{
eval_brier(time, event, S_mat, times, tmin = min(times), tmax = max(times))
}
\arguments{
\item{time}{Numeric vector of observed follow-up times.}

\item{event}{Numeric vector of event indicators (1 = event, 0 = censored).}

\item{S_mat}{A numeric matrix of predicted survival probabilities
(rows = observations, columns = time points).}

\item{times}{Numeric vector of evaluation times matching the columns of \code{S_mat}.}

\item{tmin}{Numeric. Lower bound for IBS integration. Defaults to \code{min(times)}.}

\item{tmax}{Numeric. Upper bound for IBS integration. Defaults to \code{max(times)}.}
}
\value{
A list containing:
\itemize{
\item \code{brier_scores}: A numeric vector of Brier scores at each time point.
\item \code{ibs}: The Integrated Brier Score over the range \verb{\code{tmin}, \code{tmax}}.
\item \code{times}: The time grid used.
}
}
\description{
Calculates the Inverse Probability of Censoring Weighted (IPCW) Brier Score
over a grid of times, and computes the Integrated Brier Score (IBS) using
trapezoidal integration.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
newX <- X[1:10, , drop = FALSE]
times <- seq(50, 150, by = 50)

fit <- surv.coxph(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = newX,
  new.times = times,
  obsWeights = rep(1, nrow(dat)),
  id = NULL
)

eval_brier(
  time = dat$duration[1:10],
  event = dat$event[1:10],
  S_mat = fit[["pred"]],
  times = times
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv.R
\name{SuperSurv}
\alias{SuperSurv}
\title{Super Learner for conditional survival functions}
\usage{
SuperSurv(
  time,
  event,
  X,
  newdata = NULL,
  new.times,
  event.library,
  cens.library,
  id = NULL,
  verbose = FALSE,
  control = list(),
  cvControl = list(),
  obsWeights = NULL,
  metalearner = "brier",
  selection = "ensemble",
  nFolds = 10,
  parallel = FALSE
)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame for prediction (defaults to X).}

\item{new.times}{Times at which to obtain predicted survivals.}

\item{event.library}{Character vector of prediction algorithms for the event.}

\item{cens.library}{Character vector of prediction algorithms for censoring.}

\item{id}{Cluster identification variable.}

\item{verbose}{Logical. If TRUE, prints progress messages.}

\item{control}{List of control parameters for the Super Learner.}

\item{cvControl}{List of control parameters for cross-validation.}

\item{obsWeights}{Observation weights.}

\item{metalearner}{Character string specifying the optimizer (e.g., "brier" or "logloss").}

\item{selection}{Character. Specifies how the meta-learner combines the base models.
Use \code{"ensemble"} (default) to calculate a weighted average (convex combination)
of the base learners. Use \code{"best"} to act as a Discrete Super Learner, which
assigns a weight of 1.0 to the single model with the lowest cross-validated risk.}

\item{nFolds}{Number of cross-validation folds (default: 10).}

\item{parallel}{Logical. If TRUE, uses future.apply for parallel execution.}
}
\value{
A list of class \code{SuperSurv} containing:
\itemize{
\item \code{call}: The matched function call.
\item \code{event.predict}: Matrix of in-sample cross-validated survival predictions.
\item \code{cens.predict}: Matrix of in-sample cross-validated censoring predictions.
\item \code{eval.times}: Numeric vector of prediction evaluation times.
\item \code{event.coef}: Numeric vector of optimized ensemble weights for the event.
\item \code{cens.coef}: Numeric vector of optimized ensemble weights for censoring.
\item \code{event.library.predict}: 3D array of cross-validated predictions from individual event learners.
\item \code{event.libraryNames}: Data frame detailing the algorithms and screeners used.
\item \code{event.fitLibrary}: List of the fitted base learner models (if \code{saveFitLibrary = TRUE}).
\item \code{times}: The time grid used for evaluation.
}
}
\description{
Orchestrates the cross-validation, metalearner optimization, and prediction
for an ensemble of survival base learners.
}
\details{
\strong{Extending the learner library.} Custom base learners can be added by
defining a wrapper function and passing its name in \code{event.library} or
\code{cens.library}. A learner wrapper should accept \code{time},
\code{event}, \code{X}, \code{newdata}, \code{new.times},
\code{obsWeights}, \code{id}, and \code{...}, and should return a list with
\code{pred}, a numeric survival-probability matrix with
\code{nrow(newdata)} rows and \code{length(new.times)} columns, and
\code{fit}, the fitted object used for future prediction. If saved fits are
needed, give \code{fit} a class and provide a corresponding
\code{predict.<class>()} method that returns the same matrix shape.

Screening methods can also be supplied by name. A screener should accept the
training inputs and return a logical vector aligned with the columns of
\code{X}. See \code{vignette("extending-supersurv", package = "SuperSurv")}
for a practical custom learner and screener example.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  new.times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = new.times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  fit
  event_weights(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.aorsf}
\alias{surv.aorsf}
\title{Wrapper for AORSF (Oblique Random Survival Forest)}
\usage{
surv.aorsf(
  time,
  event,
  X,
  newdata,
  new.times,
  obsWeights,
  id,
  n_tree = 500,
  leaf_min_events = 5,
  mtry = NULL,
  ...
)
}
\arguments{
\item{time}{Observed follow-up time; i.e. minimum of the event and censoring times.}

\item{event}{Observed event indicator; i.e, whether the follow-up time corresponds to an event or censoring.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction. Should have the same variable names and structure as \code{X}.}

\item{new.times}{Times at which to obtain the predicted survivals.}

\item{obsWeights}{Observation weights.}

\item{id}{Optional cluster/individual ID indicator.}

\item{n_tree}{Number of trees to grow (default: 500).}

\item{leaf_min_events}{Minimum number of events in a leaf node (default: 5).}

\item{mtry}{Number of predictors evaluated at each node.}

\item{...}{Additional arguments passed to \code{\link[aorsf]{orsf}}.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
If the model fails to fit, this may be an object of class \code{try-error}.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at the specified \code{new.times} grid.
}
}
\description{
Final Production Wrapper for AORSF (Tunable & Robust).
}
\examples{
if (requireNamespace("aorsf", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:30, ]
  x_cols <- grep("^x", names(dat))[1:3]
  X <- dat[, x_cols, drop = FALSE]
  newX <- X[1:5, , drop = FALSE]
  times <- seq(50, 150, by = 50)

  fit <- surv.aorsf(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = newX,
    new.times = times,
    obsWeights = rep(1, nrow(dat)),
    id = NULL,
    n_tree = 10,
    leaf_min_events = 2
  )

  dim(fit[["pred"]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{coef.SuperSurv}
\alias{coef.SuperSurv}
\title{Extract SuperSurv ensemble coefficients}
\usage{
\method{coef}{SuperSurv}(object, type = c("event", "censoring", "both"), ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{type}{Character string specifying which weights to return. Use
\code{"event"} for the event-time ensemble, \code{"censoring"} for the
censoring ensemble, or \code{"both"} for both sets of weights.}

\item{...}{Additional arguments ignored.}
}
\value{
For \code{type = "event"} or \code{type = "censoring"}, a named
numeric vector of ensemble weights. For \code{type = "both"}, a list with
elements \code{event} and \code{censoring}.
}
\description{
Returns the optimized convex-combination weights from a fitted
\code{SuperSurv} object.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  new.times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = new.times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  coef(fit)
  coef(fit, type = "both")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_wrappers.R
\name{list_wrappers}
\alias{list_wrappers}
\title{List Available Wrappers and Screeners in SuperSurv}
\usage{
list_wrappers(what = "both")
}
\arguments{
\item{what}{Character string. If \code{"both"} (default), lists both prediction
and screening functions. If \code{"surv"}, lists only prediction models.
If \code{"screen"}, lists only screening algorithms. Otherwise, lists all exports.}
}
\value{
An invisible character vector containing the requested function names.
}
\description{
This function prints all built-in prediction algorithms and feature screening
algorithms available in the \code{SuperSurv} package.
}
\examples{
list_wrappers()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{eval_summary}
\alias{eval_summary}
\title{Evaluate SuperSurv predictions on test data}
\usage{
eval_summary(
  object,
  newdata,
  time,
  event,
  eval_times,
  risk_time = stats::median(eval_times),
  verbose = FALSE
)
}
\arguments{
\item{object}{A fitted \code{SuperSurv} object.}

\item{newdata}{A data.frame of test covariates.}

\item{time}{Numeric vector of observed follow-up times for the test set.}

\item{event}{Numeric vector of event indicators for the test set.}

\item{eval_times}{Numeric vector of times at which to evaluate survival
predictions.}

\item{risk_time}{Numeric. The specific time horizon used when extracting risk
scores for Uno C-index. Defaults to the median of \code{eval_times}.}

\item{verbose}{Logical; if \code{TRUE}, progress messages are shown.}
}
\value{
An object of class \code{"SuperSurv_eval"} containing benchmark
metrics for the ensemble and base learners.
}
\description{
Computes the integrated Brier score (IBS), Uno C-index, and integrated area
under the curve (iAUC) for the SuperSurv ensemble and all individual base
learners.
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:80, ]
x_cols <- grep("^x", names(dat))[1:3]

fit <- SuperSurv(
  time = dat$duration,
  event = dat$event,
  X = dat[, x_cols, drop = FALSE],
  new.times = seq(50, 200, by = 50),
  event.library = c("surv.coxph", "surv.km"),
  cens.library = c("surv.coxph", "surv.km")
)

res <- eval_summary(
  object = fit,
  newdata = dat[, x_cols, drop = FALSE],
  time = dat$duration,
  event = dat$event,
  eval_times = seq(50, 200, by = 50)
)

res

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{learner_names}
\alias{learner_names}
\alias{learner_names.SuperSurv}
\title{Access SuperSurv learner names}
\usage{
learner_names(object, ...)

\method{learner_names}{SuperSurv}(object, type = c("event", "censoring", "both"), ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{...}{Additional arguments ignored.}

\item{type}{Character string specifying whether to return event learner
names, censoring learner names, or both.}
}
\value{
For \code{type = "event"} or \code{type = "censoring"}, a character
vector of learner names. For \code{type = "both"}, a list with elements
\code{event} and \code{censoring}.
}
\description{
Returns the fitted learner names from a \code{SuperSurv} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{create_grid}
\alias{create_grid}
\title{Create a Tuning Grid of Survival Learners}
\usage{
create_grid(base_learner, grid_params)
}
\arguments{
\item{base_learner}{Character string of the base learner function name (e.g., "surv.gbm").}

\item{grid_params}{List of numeric/character vectors containing hyperparameter values.}
}
\value{
A character vector of class \code{"SuperSurv_grid"} containing the
newly generated function names.
}
\description{
Dynamically generates custom wrapper functions for a specified base learner
across a grid of hyperparameters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.gbm}
\alias{surv.gbm}
\title{Wrapper function for Gradient Boosting (GBM) prediction algorithm}
\usage{
surv.gbm(
  time,
  event,
  X,
  newdata,
  new.times,
  obsWeights,
  id,
  n.trees = 1000,
  interaction.depth = 2,
  shrinkage = 0.01,
  cv.folds = 5,
  n.minobsinnode = 10,
  ...
)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Times at which to obtain the predicted survivals.}

\item{obsWeights}{Observation weights.}

\item{id}{Optional cluster/individual ID indicator.}

\item{n.trees}{Integer specifying the total number of trees to fit (default: 1000).}

\item{interaction.depth}{Maximum depth of variable interactions (default: 2).}

\item{shrinkage}{A shrinkage parameter applied to each tree (default: 0.01).}

\item{cv.folds}{Number of cross-validation folds to perform internally for optimal tree selection (default: 5).}

\item{n.minobsinnode}{Minimum number of observations in the trees terminal nodes (default: 10).}

\item{...}{Additional arguments passed to \code{\link[gbm]{gbm}}.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
If the model fails to fit, this may be an object of class \code{try-error}.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at the specified \code{new.times} grid.
}
}
\description{
Final Production Wrapper for GBM (Tunable & Robust).
Estimates a Cox proportional hazards model via gradient boosting.
Uses the Breslow estimator with a step-function approach for the baseline hazard.
Includes internal safeguards against C++ crashes and small cross-validation folds.
}
\examples{
if (requireNamespace("gbm", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:30, ]
  x_cols <- grep("^x", names(dat))[1:3]
  X <- dat[, x_cols, drop = FALSE]
  newX <- X[1:5, , drop = FALSE]
  times <- seq(50, 150, by = 50)

  fit <- surv.gbm(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = newX,
    new.times = times,
    obsWeights = rep(1, nrow(dat)),
    id = NULL,
    n.trees = 20,
    interaction.depth = 1,
    shrinkage = 0.05,
    cv.folds = 0,
    n.minobsinnode = 3
  )

  dim(fit[["pred"]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screener.R
\name{screen.marg}
\alias{screen.marg}
\title{Marginal Cox Regression Screening}
\usage{
screen.marg(time, event, X, obsWeights = NULL, minscreen = 2, min.p = 0.1, ...)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{obsWeights}{Observation weights.}

\item{minscreen}{Minimum number of covariates to return. Defaults to 2.}

\item{min.p}{Threshold p-value. Defaults to 0.1.}

\item{...}{Additional ignored arguments.}
}
\value{
A logical vector of the same length as the number of columns in \code{X},
indicating which variables passed the screening algorithm (\code{TRUE} to keep,
\code{FALSE} to drop).
}
\description{
Marginal Cox Regression Screening
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:40, ]
x_cols <- grep("^x", names(dat))[1:5]
X <- dat[, x_cols, drop = FALSE]

screen.marg(
  time = dat$duration,
  event = dat$event,
  X = X,
  minscreen = 2,
  min.p = 0.2
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{print.SuperSurv}
\alias{print.SuperSurv}
\title{Print a SuperSurv fit}
\usage{
\method{print}{SuperSurv}(x, digits = max(3L, getOption("digits") - 3L), ...)
}
\arguments{
\item{x}{A fitted object of class \code{"SuperSurv"}.}

\item{digits}{Number of significant digits to use for displayed weights.}

\item{...}{Additional arguments ignored.}
}
\value{
The input object \code{x}, invisibly.
}
\description{
Prints a concise description of a fitted \code{SuperSurv} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmst.R
\name{get_rmst}
\alias{get_rmst}
\title{Calculate Restricted Mean Survival Time (RMST)}
\usage{
get_rmst(surv_matrix, times, tau)
}
\arguments{
\item{surv_matrix}{Matrix of survival probabilities (rows: patients, cols: time points)}

\item{times}{Vector of time points corresponding to the columns}

\item{tau}{The restriction time horizon}
}
\value{
A vector of RMST values for each patient
}
\description{
Calculate Restricted Mean Survival Time (RMST)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.lognormal}
\alias{surv.lognormal}
\title{Parametric Survival Prediction Wrapper (Log-Normal)}
\usage{
surv.lognormal(time, event, X, newdata, new.times, obsWeights, id, ...)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Times at which to obtain the predicted survivals.}

\item{obsWeights}{Observation weights.}

\item{id}{Cluster identification variable.}

\item{...}{Additional ignored arguments.}
}
\value{
A list containing the fitted model and predictions.
}
\description{
Parametric Survival Prediction Wrapper (Log-Normal)
}
\examples{
data("metabric", package = "SuperSurv")
dat <- metabric[1:30, ]
x_cols <- grep("^x", names(dat))[1:3]
X <- dat[, x_cols, drop = FALSE]
newX <- X[1:5, , drop = FALSE]
times <- seq(50, 150, by = 50)

fit <- surv.lognormal(
  time = dat$duration,
  event = dat$event,
  X = X,
  newdata = newX,
  new.times = times,
  obsWeights = rep(1, nrow(dat)),
  id = NULL
)

dim(fit[["pred"]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{selected_variables}
\alias{selected_variables}
\alias{selected_variables.SuperSurv}
\title{Access variables selected by SuperSurv screeners}
\usage{
selected_variables(object, ...)

\method{selected_variables}{SuperSurv}(object, type = c("event", "censoring"), learner = NULL, ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{...}{Additional arguments ignored.}

\item{type}{Character string specifying whether to inspect event or censoring
learners.}

\item{learner}{Optional learner index or learner name. If omitted, selected
variables are returned for every learner of the requested type.}
}
\value{
A named list of character vectors, or a single character vector when
\code{learner} has length one.
}
\description{
Returns the variables retained by a screening step for one or more fitted
event or censoring learners.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.gam}
\alias{surv.gam}
\title{Wrapper for Generalized Additive Cox Regression (GAM)}
\usage{
surv.gam(time, event, X, newdata, new.times, obsWeights, id, cts.num = 5, ...)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Times at which to obtain predicted survivals.}

\item{obsWeights}{Observation weights (Note: Ignored, as mgcv uses weights for the event indicator).}

\item{id}{Optional cluster/individual ID indicator.}

\item{cts.num}{Cutoff of unique values at which a numeric covariate receives a smooth term (s).}

\item{...}{Additional arguments passed to \code{\link[mgcv]{gam}}.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
If the model fails to fit, this may be an object of class \code{try-error}.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at the specified \code{new.times} grid.
}
}
\description{
Final Production Wrapper for GAM (Tunable & Robust).
Uses \code{\link[mgcv]{gam}} to fit an additive combination of smooth
and linear functions.
}
\examples{
if (requireNamespace("mgcv", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:30, ]
  x_cols <- grep("^x", names(dat))[1:3]
  X <- dat[, x_cols, drop = FALSE]
  newX <- X[1:5, , drop = FALSE]
  times <- seq(50, 150, by = 50)

  fit <- surv.gam(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = newX,
    new.times = times,
    obsWeights = rep(1, nrow(dat)),
    id = NULL,
    cts.num = 5
  )

  dim(fit[["pred"]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_predict}
\alias{plot_predict}
\title{Plot Predicted Survival Curves}
\usage{
plot_predict(preds, eval_times, patient_idx = 1)
}
\arguments{
\item{preds}{A list containing SuperSurv predictions OR a raw prediction matrix.}

\item{eval_times}{Numeric vector of times at which predictions were evaluated.}

\item{patient_idx}{Integer vector. Defaults to 1.}
}
\value{
A ggplot object.
}
\description{
Plot Predicted Survival Curves
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:120, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  eval_times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = eval_times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  preds <- predict(fit, newdata = X, new.times = eval_times, type = "event")

  plot_predict(
    preds = preds,
    eval_times = eval_times,
    patient_idx = c(1, 2)
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/control.R
\name{SuperSurv.control}
\alias{SuperSurv.control}
\title{Control parameters for the SuperSurv Ensemble}
\usage{
SuperSurv.control(
  max.SL.iter = 20,
  event.t.grid = NULL,
  cens.t.grid = NULL,
  saveFitLibrary = TRUE,
  initWeightAlg = "surv.coxph",
  initWeight = "censoring",
  ...
)
}
\arguments{
\item{max.SL.iter}{Maximum iterations for the iterative weighting algorithm. Default 20.}

\item{event.t.grid}{Optional time grid for event risk calculation.}

\item{cens.t.grid}{Optional time grid for censoring risk calculation.}

\item{saveFitLibrary}{Logical. If TRUE (default), saves models for future predictions.}

\item{initWeightAlg}{The learner used for the very first step of IPCW.}

\item{initWeight}{Whether to start by fitting "censoring" or "event" weights.}

\item{...}{Additional ignored arguments.}
}
\value{
A list of control parameters.
}
\description{
Control parameters for the SuperSurv Ensemble
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explainability.R
\name{plot_survival_heatmap}
\alias{plot_survival_heatmap}
\title{Survival Probability Heatmap}
\usage{
plot_survival_heatmap(object, newdata, times)
}
\arguments{
\item{object}{A fitted SuperSurv object}

\item{newdata}{Test covariates (e.g., \code{X_te[1:50, ]})}

\item{times}{The time grid to visualize}
}
\value{
A \code{ggplot} object visualizing the SHAP values.
}
\description{
Survival Probability Heatmap
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  plot_survival_heatmap(
    object = fit,
    newdata = X[1:20, , drop = FALSE],
    times = times
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{eval_timeROC}
\alias{eval_timeROC}
\title{Time-Dependent AUC and Integrated AUC}
\usage{
eval_timeROC(time, event, S_mat, times)
}
\arguments{
\item{time}{Numeric vector of observed follow-up times.}

\item{event}{Numeric vector of event indicators (1 = event, 0 = censored).}

\item{S_mat}{A numeric matrix of predicted survival probabilities.}

\item{times}{Numeric vector of evaluation times matching the columns of \code{S_mat}.}
}
\value{
A list containing the \code{AUC_curve} at each time point, the
\code{times}, and the integrated AUC \code{iAUC}.
}
\description{
Evaluates the cumulative/dynamic time-dependent AUC and integrated AUC (iAUC)
using inverse probability of censoring weighting (IPCW).
}
\examples{
 data("metabric", package = "SuperSurv")
 dat <- metabric[1:40, ]
 x_cols <- grep("^x", names(dat))[1:3]
 X <- dat[, x_cols, drop = FALSE]
 newX <- X[1:10, , drop = FALSE]
 times <- seq(50, 150, by = 50)

 fit <- surv.coxph(
   time = dat$duration,
   event = dat$event,
   X = X,
   newdata = newX,
   new.times = times,
   obsWeights = rep(1, nrow(dat)),
   id = NULL
 )

 eval_timeROC(
   time = dat$duration[1:10],
   event = dat$event[1:10],
   S_mat = fit[["pred"]],
   times = times
 )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper.R
\name{surv.xgboost}
\alias{surv.xgboost}
\title{Wrapper for XGBoost (Robust CV-Tuned + Safe Prediction)}
\usage{
surv.xgboost(
  time,
  event,
  X,
  newdata = NULL,
  new.times,
  obsWeights,
  id,
  nrounds = 1000,
  early_stopping_rounds = 10,
  eta = 0.05,
  max_depth = 2,
  min_child_weight = 5,
  lambda = 10,
  subsample = 0.7,
  ...
)
}
\arguments{
\item{time}{Observed follow-up time.}

\item{event}{Observed event indicator.}

\item{X}{Training covariate data.frame.}

\item{newdata}{Test covariate data.frame to use for prediction.}

\item{new.times}{Times at which to obtain the predicted survivals.}

\item{obsWeights}{Observation weights.}

\item{id}{Optional cluster/individual ID indicator.}

\item{nrounds}{Max number of boosting iterations (default: 1000).}

\item{early_stopping_rounds}{Rounds with no improvement to trigger early stopping (default: 10).}

\item{eta}{Learning rate (default: 0.05).}

\item{max_depth}{Maximum tree depth (default: 2).}

\item{min_child_weight}{Minimum sum of instance weight in a child (default: 5).}

\item{lambda}{L2 regularization term on weights (default: 10).}

\item{subsample}{Subsample ratio of the training instances (default: 0.7).}

\item{...}{Additional arguments passed to \code{\link[xgboost]{xgb.train}}.}
}
\value{
A list containing:
\itemize{
\item \code{fit}: The fitted model object (e.g., the raw \code{coxph} or \code{xgb.Booster} object).
If the model fails to fit, this may be an object of class \code{try-error}.
\item \code{pred}: A numeric matrix of cross-validated survival predictions
evaluated at the specified \code{new.times} grid.
}
}
\description{
Estimates a Cox proportional hazards model via XGBoost.
Incorporates safe Breslow hazard calculation and matrix alignment to prevent C++ crashes.
}
\examples{
if (requireNamespace("xgboost", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:30, ]
  x_cols <- grep("^x", names(dat))[1:3]
  X <- dat[, x_cols, drop = FALSE]
  newX <- X[1:5, , drop = FALSE]
  times <- seq(50, 150, by = 50)

  fit <- surv.xgboost(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = newX,
    new.times = times,
    obsWeights = rep(1, nrow(dat)),
    id = NULL,
    nrounds = 5,
    early_stopping_rounds = 2,
    max_depth = 1
  )

  dim(fit[["pred"]])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSurv-methods.R
\name{event_weights}
\alias{event_weights}
\alias{event_weights.SuperSurv}
\alias{censor_weights}
\alias{censor_weights.SuperSurv}
\title{Access SuperSurv ensemble weights}
\usage{
event_weights(object, ...)

\method{event_weights}{SuperSurv}(object, ...)

censor_weights(object, ...)

\method{censor_weights}{SuperSurv}(object, ...)
}
\arguments{
\item{object}{A fitted object of class \code{"SuperSurv"}.}

\item{...}{Additional arguments ignored.}
}
\value{
A named numeric vector of ensemble weights.
}
\description{
Extracts the fitted event or censoring ensemble weights from a
\code{SuperSurv} object.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  new.times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = new.times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  event_weights(fit)
  censor_weights(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{metabric}
\alias{metabric}
\title{METABRIC Breast Cancer Dataset}
\format{
A data frame with the clinical and genomic variables:
\describe{
\item{duration}{Survival time.}
\item{event}{Event indicator (1 = event, 0 = censored).}
\item{x0}{Feature x0.}
\item{x1}{Feature x1.}
\item{x2}{Feature x2.}
\item{x3}{Feature x3.}
\item{x4}{Feature x4.}
\item{x5}{Feature x5.}
\item{x6}{Feature x6.}
\item{x7}{Feature x7.}
\item{x8}{Feature x8.}
}
}
\usage{
metabric
}
\description{
A subset of the Molecular Taxonomy of Breast Cancer International Consortium
(METABRIC) dataset to demonstrate the SuperSurv package. 9 Covariates: 4 gene indicators (MKI67, EGFR, PGR, and ERBB2) and 5 clinical features (age at diagnosis, and indicators for hormone treatment, radiotherapy, and chemotherapy)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_calibration}
\alias{plot_calibration}
\title{Plot Survival Calibration Curve}
\usage{
plot_calibration(object, newdata, time, event, eval_time, bins = 5)
}
\arguments{
\item{object}{A fitted SuperSurv object OR a standalone base learner.}

\item{newdata}{A data.frame of test covariates.}

\item{time}{Numeric vector of observed follow-up times for the test set.}

\item{event}{Numeric vector of event indicators for the test set.}

\item{eval_time}{Numeric. A single time point at which to assess calibration.}

\item{bins}{Integer. Defaults to 5.}
}
\value{
A ggplot object.
}
\description{
Plot Survival Calibration Curve
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:120, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  eval_times <- seq(20, 120, by = 20)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = eval_times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  plot_calibration(
    object = fit,
    newdata = X,
    time = dat$duration,
    event = dat$event,
    eval_time = 100,
    bins = 4
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explainability.R
\name{explain_survex}
\alias{explain_survex}
\title{Create a Time-Dependent Survex Explainer}
\usage{
explain_survex(model, data, y, times, label = NULL)
}
\arguments{
\item{model}{A fitted SuperSurv object OR a single wrapper output.}

\item{data}{Covariate data for explanation (data.frame).}

\item{y}{The survival object (\code{Surv(time, event)}).}

\item{times}{The time grid for evaluation.}

\item{label}{Optional character string to name the explainer.}
}
\value{
An explainer object of class \code{survex_explainer} created by
\code{\link[survex]{explain_survival}}, which can be passed to DALEX and survex functions for further model diagnostics and plotting.
}
\description{
Bridges a fitted SuperSurv ensemble or a single base learner to the
\code{survex} package for Time-Dependent SHAP and Model Parts.
}
\examples{
if (requireNamespace("survex", quietly = TRUE) &&
    requireNamespace("glmnet", quietly = TRUE)) {
  data("metabric", package = "SuperSurv")
  dat <- metabric[1:80, ]
  x_cols <- grep("^x", names(dat))[1:5]
  X <- dat[, x_cols, drop = FALSE]
  times <- seq(20, 120, by = 20)
  y <- survival::Surv(dat$duration, dat$event)

  fit <- SuperSurv(
    time = dat$duration,
    event = dat$event,
    X = X,
    newdata = X,
    new.times = times,
    event.library = c("surv.coxph", "surv.ridge"),
    cens.library = c("surv.coxph"),
    control = list(saveFitLibrary = TRUE)
  )

  explainer <- explain_survex(
    model = fit,
    data = X,
    y = y,
    times = times,
    label = "SuperSurv_demo"
  )

  class(explainer)
}
}
